@@ -45,11 +45,10 @@
 #define Y6 64
 #define Y7 128
 
-struct dim2d
-{
-    int w;
-    int h;
-};
+#define subvp_supported "sub-viewport supported: yes"
+#define subvp_enabled "sub-viewport supported: yes, enabled: yes"
+#define mall_supported "mall supported: yes"
+#define mall_enabled "mall supported: yes, enabled: yes"
 
 uint32_t igt_amd_create_bo(int fd, uint64_t size)
 {
@@ -325,6 +324,16 @@ static bool igt_amd_output_has_dsc(int drm_fd, char *connector_name)
 	return igt_amd_output_has_debugfs(drm_fd, connector_name, DEBUGFS_DSC_CLOCK_EN);
 }
 
+/**
+ * igt_amd_output_has_odm_combine_segments: check if connector has dsc debugfs entry
+ * @drm_fd: DRM file descriptor
+ * @connector_name: The connector's name, on which we're reading the status
+ */
+bool igt_amd_output_has_odm_combine_segments(int drm_fd, char *connector_name)
+{
+	return igt_amd_output_has_debugfs(drm_fd, connector_name, DEBUGFS_ODM_COMBINE_SEG);
+}
+
 /**
  * is_dp_dsc_supported: Checks if connector is DSC capable
  * @display: A pointer to an #igt_display_t structure
@@ -344,10 +353,11 @@ bool is_dp_dsc_supported(int drm_fd, char *connector_name)
 	}
 
 	ret = igt_debugfs_simple_read(fd, DEBUGFS_DSC_FEC_SUPPORT, buf, sizeof(buf));
-	close(fd);
+	if (ret < 0)
+		igt_info("Reading %s for connector %s failed.\n",
+			 DEBUGFS_DSC_FEC_SUPPORT, connector_name);
 
-	igt_assert_f(ret >= 0, "Reading %s for connector %s failed.\n",
-		     DEBUGFS_DSC_FEC_SUPPORT, connector_name);
+	close(fd);
 
 	return strstr(buf, "DSC_Sink_Support: yes");
 }
@@ -371,10 +381,11 @@ bool is_dp_fec_supported(int drm_fd, char *connector_name)
 	}
 
 	ret = igt_debugfs_simple_read(fd, DEBUGFS_DSC_FEC_SUPPORT, buf, sizeof(buf));
-	close(fd);
+	if (ret < 0)
+		igt_info("Reading %s for connector %s failed.\n",
+			 DEBUGFS_DSC_FEC_SUPPORT, connector_name);
 
-	igt_assert_f(ret >= 0, "Reading %s for connector %s failed.\n",
-		     DEBUGFS_DSC_FEC_SUPPORT, connector_name);
+	close(fd);
 
 	return strstr(buf, "FEC_Sink_Support: yes");
 }
@@ -790,6 +801,9 @@ int igt_amd_trigger_hotplug(int drm_fd, char *connector_name)
         int wr_len;
         const char *enable_hpd = "1";
 
+	if (!igt_amd_output_has_hpd(drm_fd, connector_name))
+		return 0;
+
         fd = igt_debugfs_connector_dir(drm_fd, connector_name, O_RDONLY);
         igt_assert(fd >= 0);
         hpd_fd = openat(fd, DEBUGFS_HPD_TRIGGER, O_WRONLY);
@@ -799,6 +813,8 @@ int igt_amd_trigger_hotplug(int drm_fd, char *connector_name)
         wr_len = write(hpd_fd, enable_hpd, strlen(enable_hpd));
         close(hpd_fd);
         igt_assert_eq(wr_len, strlen(enable_hpd));
+	/* wait for hotplug process complete by kernel and user mode components */
+	sleep(30);
 
         return 0;
 }
@@ -998,6 +1014,169 @@ bool igt_amd_output_has_ilr_setting(int drm_fd, char *connector_name)
 	return igt_amd_output_has_debugfs(drm_fd, connector_name, DEBUGFS_EDP_ILR_SETTING);
 }
 
+/**
+ * igt_amd_output_has_replay_cap: check if eDP connector has replay_capability debugfs entry
+ * @drm_fd: DRM file descriptor
+ * @connector_name: The connector's name, on which we're reading the status
+ */
+bool igt_amd_output_has_replay_cap(int drm_fd, char *connector_name)
+{
+	return igt_amd_output_has_debugfs(drm_fd, connector_name, DEBUGFS_EDP_REPLAY_CAP);
+}
+
+/**
+ * igt_amd_replay_support_sink: check if sink device support Panel Replay
+ * @drm_fd: DRM file descriptor
+ * @connector_name: The connector's name, on which we're reading the status
+ */
+bool igt_amd_replay_support_sink(int drm_fd, char *connector_name)
+{
+	char buf[128];
+	int ret;
+	int fd;
+
+	fd = igt_debugfs_connector_dir(drm_fd, connector_name, O_RDONLY);
+	if (fd < 0) {
+		igt_info("output %s: debugfs not found\n", connector_name);
+
+		return false;
+	}
+
+	ret = igt_debugfs_simple_read(fd, DEBUGFS_EDP_REPLAY_CAP, buf, sizeof(buf));
+	igt_assert_f(ret >= 0, "Reading %s for connector %s failed.\n",
+		 DEBUGFS_EDP_REPLAY_CAP, connector_name);
+	close(fd);
+
+	if (ret < 1)
+		return false;
+
+	return strstr(buf, "Sink support: yes");
+}
+
+/**
+ * igt_amd_replay_support_drv: check if driver support Panel Replay
+ * @drm_fd: DRM file descriptor
+ * @connector_name: The connector's name, on which we're reading the status
+ */
+bool igt_amd_replay_support_drv(int drm_fd, char *connector_name)
+{
+	char buf[128];
+	int ret;
+	int fd;
+
+	fd = igt_debugfs_connector_dir(drm_fd, connector_name, O_RDONLY);
+	if (fd < 0) {
+		igt_info("output %s: debugfs not found\n", connector_name);
+
+		return false;
+	}
+
+	ret = igt_debugfs_simple_read(fd, DEBUGFS_EDP_REPLAY_CAP, buf, sizeof(buf));
+	igt_assert_f(ret >= 0, "Reading %s for connector %s failed.\n",
+		 DEBUGFS_EDP_REPLAY_CAP, connector_name);
+	close(fd);
+
+	if (ret < 1)
+		return false;
+
+	return strstr(buf, "Driver support: yes") && strstr(buf, "Config support: yes");
+}
+
+/**
+ * igt_amd_output_has_replay_state: check if eDP connector has replay_state debugfs entry
+ * @drm_fd: DRM file descriptor
+ * @connector_name: The connector's name, on which we're reading the status
+ */
+bool igt_amd_output_has_replay_state(int drm_fd, char *connector_name)
+{
+	return igt_amd_output_has_debugfs(drm_fd, connector_name, DEBUGFS_EDP_REPLAY_STATE);
+}
+
+/*
+ * Convert raw panel replay state to emum panel replay state.
+ */
+static enum replay_state convert_replay_state(uint32_t raw_state)
+{
+	switch (raw_state) {
+	case 0:
+		return REPLAY_STATE_0;
+	case 0x10:
+		return REPLAY_STATE_1;
+	case 0x11:
+		return REPLAY_STATE_1A;
+	case 0x20:
+		return REPLAY_STATE_2;
+	case 0x21:
+		return REPLAY_STATE_2A;
+	case 0x30:
+		return REPLAY_STATE_3;
+	case 0x31:
+		return REPLAY_STATE_3INIT;
+	case 0x40:
+		return REPLAY_STATE_4;
+	case 0x41:
+		return REPLAY_STATE_4A;
+	case 0x42:
+		return REPLAY_STATE_4B;
+	case 0x43:
+		return REPLAY_STATE_4C;
+	case 0x44:
+		return REPLAY_STATE_4D;
+	case 0x45:
+		return REPLAY_STATE_4E;
+	case 0x4A:
+		return REPLAY_STATE_4B_LOCKED;
+	case 0x4B:
+		return REPLAY_STATE_4C_UNLOCKED;
+	case 0x50:
+		return REPLAY_STATE_5;
+	case 0x51:
+		return REPLAY_STATE_5A;
+	case 0x52:
+		return REPLAY_STATE_5B;
+	case 0x5A:
+		return REPLAY_STATE_5A_LOCKED;
+	case 0x5B:
+		return REPLAY_STATE_5B_UNLOCKED;
+	case 0x60:
+		return REPLAY_STATE_6;
+	case 0x61:
+		return REPLAY_STATE_6A;
+	case 0x62:
+		return REPLAY_STATE_6B;
+	default:
+		return REPLAY_STATE_INVALID;
+	}
+}
+
+/**
+ * @brief Read Panel Replay State from debugfs interface
+ * @param drm_fd DRM file descriptor
+ * @param connector_name The connector's name, on which we're reading the status
+ * @return Panel Replay state
+ */
+enum replay_state igt_amd_read_replay_state(int drm_fd, char *connector_name)
+{
+	char buf[4];
+	int fd, ret, raw_state;
+
+	fd = igt_debugfs_connector_dir(drm_fd, connector_name, O_RDONLY);
+	if (fd < 0) {
+		igt_info("Couldn't open connector %s debugfs directory\n", connector_name);
+
+		return -1;
+	}
+
+	ret = igt_debugfs_simple_read(fd, DEBUGFS_EDP_REPLAY_STATE, buf, sizeof(buf));
+	close(fd);
+
+	igt_assert_f(ret >= 0, "Reading %s for connector %s failed.\n",
+		 DEBUGFS_EDP_REPLAY_STATE, connector_name);
+
+	raw_state = strtol(buf, NULL, 10);
+	return convert_replay_state(raw_state);
+}
+
 /**
  * igt_amd_output_has_psr_cap: check if eDP connector has psr_capability debugfs entry
  * @drm_fd: DRM file descriptor
@@ -1035,7 +1214,9 @@ bool igt_amd_psr_support_sink(int drm_fd, char *connector_name, enum psr_mode mo
 		return false;
 
 	if (mode == PSR_MODE_1)
-		return strstr(buf, "Sink support: yes [0x01]");
+		return strstr(buf, "Sink support: yes [0x01]") ||
+		       strstr(buf, "Sink support: yes [0x03]") ||
+		       strstr(buf, "Sink support: yes [0x04]");
 	else
 		return strstr(buf, "Sink support: yes [0x03]") ||
 		       strstr(buf, "Sink support: yes [0x04]");
@@ -1109,6 +1290,160 @@ int igt_amd_read_psr_state(int drm_fd, char *connector_name)
 	return strtol(buf, NULL, 10);
 }
 
+/**
+ * igt_amd_allow_edp_hotplug_detect: notify kernel read edp edid all the time
+ * @drm_fd: DRM file descriptor
+ * @connector_name: The connector's name
+ * @enable: allow or disable kernel read eDP EDID for each display detection
+ * example usage: echo 0x1 >
+ * /sys/kernel/debug/dri/0/eDP-1/allow_edp_hotplug_detection
+ */
+void igt_amd_allow_edp_hotplug_detect(int drm_fd, char *connector_name, bool enable)
+{
+	int fd, hpd_fd, wr_len;
+	const char *allow_hotplug_detect = "1";
+	const char *dis_allow_hotplug_detect = "0";
+
+	fd = igt_debugfs_connector_dir(drm_fd, connector_name, O_RDONLY);
+	igt_assert(fd >= 0);
+	hpd_fd = openat(fd, DEBUGFS_ALLOW_EDP_HOTPLUG_DETECT, O_WRONLY);
+	close(fd);
+	igt_assert(hpd_fd >= 0);
+
+	if (enable) {
+		wr_len = write(hpd_fd, allow_hotplug_detect, strlen(allow_hotplug_detect));
+		igt_assert_eq(wr_len, strlen(allow_hotplug_detect));
+	} else {
+		wr_len = write(hpd_fd, dis_allow_hotplug_detect, strlen(dis_allow_hotplug_detect));
+		igt_assert_eq(wr_len, strlen(dis_allow_hotplug_detect));
+	}
+
+	close(hpd_fd);
+}
+
+/**
+ * igt_amd_disallow_edp_enter_psr: notify kernel skip edp psr setup and enable
+ * @drm_fd: DRM file descriptor
+ * @connector_name: The connector's name
+ * @enable: skip kernel eDP psr setup and enable -- disallow edp enter psr
+ * example usage: disallow psr
+ * echo 0x1 >
+ * /sys/kernel/debug/dri/0/eDP-1/disallow_edp_enter_psr
+ *
+ * expected IGT sequence is as below:
+ * 1. disable eDP PHY and notify eDP rx with dpcd 0x600 = 2.
+ *    for example, kmstest_set_connector_dpms off will do this.
+ * 2. echo 0x1 /sys/kernel/debug/dri/0/eDP-X/disallow_edp_enter_psr
+ * 3. enable eDP PHY and notify eDP rx with dpcd 0x600 = 1 but
+ *    without dpcd 0x170 = 5.
+ *    for example, kmstest_set_connector_dpms on will do this.
+ * 4. read crc from rx dpcd 0x270, 0x246, etc.
+ *    igt_pipe_crc_collect_crc will do this.
+ * 5. echo 0x0 /sys/kernel/debug/dri/0/eDP-X/disallow_edp_enter_psr.
+ *    this will let eDP back to normal with psr setup dpcd 0x170 = 5.
+ */
+void igt_amd_disallow_edp_enter_psr(int drm_fd, char *connector_name, bool enable)
+{
+	int fd, ret, wr_len;
+	const char *allow_edp_psr = "1";
+	const char *dis_allow_edp_psr = "0";
+
+	fd = igt_debugfs_connector_dir(drm_fd, connector_name, O_RDONLY);
+	igt_assert(fd >= 0);
+	ret = openat(fd, DEBUGFS_DISALLOW_EDP_ENTER_PSR, O_WRONLY);
+	close(fd);
+	igt_assert(ret >= 0);
+
+	if (enable) {
+		wr_len = write(ret, allow_edp_psr, strlen(allow_edp_psr));
+		igt_assert_eq(wr_len, strlen(allow_edp_psr));
+	} else {
+		wr_len = write(ret, dis_allow_edp_psr, strlen(dis_allow_edp_psr));
+		igt_assert_eq(wr_len, strlen(dis_allow_edp_psr));
+	}
+
+	close(ret);
+}
+
+static bool get_dm_capabilities(int drm_fd, char *buf, size_t size)
+{
+	int ret, fd;
+	bool has_capablities = amd_has_debugfs(drm_fd, DEBUGFS_DM_CAPABILITIES);
+
+	if (!has_capablities)
+		return false;
+
+	fd = igt_debugfs_dir(drm_fd);
+	if (fd < 0) {
+		igt_warn("Couldn't open debugfs directory\n");
+		return -1;
+	}
+
+	ret = igt_debugfs_simple_read(fd, DEBUGFS_DM_CAPABILITIES, buf, (int) size);
+	igt_assert_f(ret >= 0, "Reading %s failed.\n", DEBUGFS_DM_CAPABILITIES);
+
+	close(fd);
+
+	if (ret < 0)
+		return false;
+
+	return true;
+}
+
+/**
+ * @brief check if AMDGPU mall_capable interface entry exist and defined
+ *
+ * @param drm_fd DRM file descriptor
+ * @param supported will be set to true if hardware supports MALL
+ * @param enabled will be set to true if MALL is currently in use
+ */
+void igt_amd_get_mall_status(int drm_fd, bool *supported, bool *enabled)
+{
+	char buf[1024];
+	char *mall_loc;
+
+	*supported = false;
+	*enabled = false;
+
+	if (!get_dm_capabilities(drm_fd, buf, 1024))
+		return;
+
+	mall_loc = strstr(buf, mall_supported);
+	if (mall_loc)
+		*supported = true;
+
+	mall_loc = strstr(buf, mall_enabled);
+	if (mall_loc && *supported)
+		*enabled = true;
+}
+
+/**
+ * @brief check if AMDGPU sub-viewport support exists
+ *
+ * @param drm_fd DRM file descriptor
+ * @param supported set to true if harwdare supports subviewport
+ * @param enabled set tot true if subviewport is currently in use
+ */
+void igt_amd_get_subvp_status(int drm_fd, bool *supported, bool *enabled)
+{
+	char buf[1024];
+	char *subvp_loc;
+
+	*supported = false;
+	*enabled = false;
+
+	if (!get_dm_capabilities(drm_fd, buf, 1024))
+		return;
+
+	subvp_loc = strstr(buf, subvp_supported);
+	if (subvp_loc)
+		*supported = true;
+
+	subvp_loc = strstr(buf, subvp_enabled);
+	if (subvp_loc && *supported)
+		*enabled = true;
+}
+
 /**
  * @brief check if AMDGPU DM visual confirm debugfs interface entry exist and defined
  *
