@@ -55,7 +55,11 @@
 #include <limits.h>
 #include <locale.h>
 #include <uwildmat/uwildmat.h>
+#ifndef ANDROID
 #include <glib.h>
+#else
+#include "android/glib.h"
+#endif
 
 #include "drmtest.h"
 #include "i915/gem_create.h"
@@ -82,7 +86,9 @@
 #include "runnercomms.h"
 
 #define UNW_LOCAL_ONLY
+#ifdef HAVE_LIBUNWIND
 #include <libunwind.h>
+#endif
 #include <elfutils/libdwfl.h>
 
 #ifdef HAVE_LIBGEN_H
@@ -112,11 +118,11 @@
  * never run any code which might fail (like trying to do privileged operations
  * or opening device driver nodes).
  *
- * To allow this i-g-t provides #igt_fixture code blocks for setup code outside
+ * To allow this i-g-t provides #igt_fixture() code blocks for setup code outside
  * of subtests and automatically skips the subtest code blocks themselves. For
  * special cases igt_only_list_subtests() is also provided. For setup code only
- * shared by a group of subtest encapsulate the #igt_fixture block and all the
- * subtestest in a #igt_subtest_group block.
+ * shared by a group of subtest encapsulate the #igt_fixture() block and all the
+ * subtestest in a #igt_subtest_group() block.
  *
  * # Magic Control Blocks
  *
@@ -135,7 +141,7 @@
  *   issues.
  *
  * - Code blocks with magic control flow are implemented with setjmp()
- *   and longjmp(). This applies to #igt_fixture, #igt_subtest,
+ *   and longjmp(). This applies to #igt_fixture(), #igt_subtest,
  *   #igt_subtest_with_dynamic and #igt_dynamic
  *   blocks and all the three variants to finish test: igt_success(),
  *   igt_skip() and igt_fail(). Mostly this is of no concern, except
@@ -375,6 +381,9 @@ static int _igt_dynamic_tests_executed = -1;
 
 static void print_backtrace(void)
 {
+#ifndef HAVE_LIBUNWIND
+	igt_info("Stack trace: not implemented\n");
+#else
 	unw_cursor_t cursor;
 	unw_context_t uc;
 	int stack_num = 0;
@@ -432,6 +441,7 @@ static void print_backtrace(void)
 
 	if (dwfl)
 		dwfl_end(dwfl);
+#endif
 }
 
 __attribute__((format(printf, 2, 3)))
@@ -758,11 +768,11 @@ void __igt_assert_in_outer_scope(void)
 bool __igt_fixture(void)
 {
 	internal_assert(!in_fixture,
-			"nesting multiple igt_fixtures is invalid\n");
+			"nesting multiple igt_fixture()s is invalid\n");
 	internal_assert(!in_subtest,
-			"nesting igt_fixture in igt_subtest is invalid\n");
+			"nesting igt_fixture() in igt_subtest is invalid\n");
 	internal_assert(test_with_subtests,
-			"igt_fixture in igt_simple_main is invalid\n");
+			"igt_fixture() in igt_simple_main() is invalid\n");
 
 	if (igt_only_list_subtests())
 		return false;
@@ -1324,6 +1334,7 @@ out:
 	if (!igt_only_list_subtests()) {
 		bind_fbcon(false);
 		igt_kmsg(KMSG_INFO "%s: executing\n", command_str);
+		igt_trace("%s: executing\n", command_str);
 		print_version();
 		igt_srandom();
 
@@ -1488,8 +1499,8 @@ static bool valid_name_for_subtest(const char *subtest_name)
 bool __igt_run_subtest(const char *subtest_name, const char *file, const int line)
 {
 	internal_assert(!igt_can_fail(),
-			"igt_subtest can be nested only in igt_main"
-			" or igt_subtest_group\n");
+			"igt_subtest can be nested only in igt_main()"
+			" or igt_subtest_group()\n");
 
 	if (!valid_name_for_subtest(subtest_name)) {
 		igt_critical("Invalid subtest name \"%s\".\n",
@@ -1528,6 +1539,7 @@ bool __igt_run_subtest(const char *subtest_name, const char *file, const int lin
 
 	igt_kmsg(KMSG_INFO "%s: starting subtest %s\n",
 		 command_str, subtest_name);
+	igt_trace("%s: starting subtest %s\n", command_str, subtest_name);
 	_subtest_starting_message(_SUBTEST_TYPE_NORMAL, subtest_name);
 
 	_igt_log_buffer_reset();
@@ -1588,6 +1600,17 @@ const char *igt_subtest_name(void)
 	return in_subtest;
 }
 
+/**
+ * igt_dynamic_subtest_name:
+ *
+ * Returns: The name of the currently executed dynamic subtest or NULL if called from
+ * outside a dynamic subtest block.
+ */
+const char *igt_dynamic_subtest_name(void)
+{
+	return in_dynamic_subtest;
+}
+
 /**
  * igt_only_list_subtests:
  *
@@ -1604,7 +1627,7 @@ bool igt_only_list_subtests(void)
 void __igt_subtest_group_save(int *save, int *desc)
 {
 	internal_assert(test_with_subtests,
-			"igt_subtest_group is not allowed in igt_simple_main\n");
+			"igt_subtest_group() is not allowed in igt_simple_main()\n");
 
 	if (__current_description[0] != '\0') {
 		struct description_node *new = calloc(1, sizeof(*new));
@@ -1681,6 +1704,8 @@ __noreturn static void exit_subtest(const char *result)
 				igt_time_elapsed(thentime, &now));
 	igt_kmsg(KMSG_INFO "%s: finished subtest %s, %s\n",
 		 command_str, *subtest_name, result);
+	igt_trace("%s: finished subtest %s, %s\n",
+		  command_str, *subtest_name, result);
 
 	igt_terminate_spins();
 
@@ -1780,7 +1805,7 @@ void igt_skip(const char *f, ...)
 		skip_subtests_henceforth = SKIP;
 		internal_assert(in_fixture,
 			"skipping is allowed only in fixtures, subtests"
-			" or igt_simple_main\n");
+			" or igt_simple_main()\n");
 		__igt_fixture_end();
 	} else {
 		igt_exitcode = IGT_EXIT_SKIP;
@@ -1901,12 +1926,11 @@ void igt_fail(int exitcode)
 		failed_one = true;
 	}
 
-	/* Silent exit, parent will do the yelling. */
+	_igt_log_buffer_dump();
+
 	if (test_child)
 		exit(exitcode);
 
-	_igt_log_buffer_dump();
-
 	if (test_multi_fork_child)
 		exit(exitcode);
 
@@ -1914,7 +1938,7 @@ void igt_fail(int exitcode)
 		exit_subtest("FAIL");
 	} else {
 		internal_assert(igt_can_fail(), "failing test is only allowed"
-				" in fixtures, subtests and igt_simple_main\n");
+				" in fixtures, subtests and igt_simple_main()\n");
 
 		if (in_fixture) {
 			skip_subtests_henceforth = FAIL;
@@ -1950,8 +1974,8 @@ void igt_fatal_error(void)
 /**
  * igt_can_fail:
  *
- * Returns true if called from either an #igt_fixture, #igt_subtest or a
- * testcase without subtests, i.e. #igt_simple_main. Returns false otherwise. In
+ * Returns true if called from either an #igt_fixture(), #igt_subtest or a
+ * testcase without subtests, i.e. #igt_simple_main(). Returns false otherwise. In
  * other words, it checks whether it's legal to call igt_fail(), igt_skip_on()
  * and all the convenience macros build around those.
  *
@@ -1968,7 +1992,7 @@ bool igt_can_fail(void)
  * @fmt: format string containing description
  * @...: argument used by the format string
  *
- * Attach a description to the following #igt_subtest or #igt_subtest_group
+ * Attach a description to the following #igt_subtest or #igt_subtest_group()
  * block.
  *
  * Check #igt_describe for more details.
@@ -2064,6 +2088,7 @@ static void write_stderr(const char *str)
 	__write_stderr(str, strlen(str));
 }
 
+#ifdef HAVE_LIBUNWIND
 static const char hex[] = "0123456789abcdef";
 
 static void
@@ -2216,9 +2241,13 @@ xprintf(const char *fmt, ...)
 	xprintfmt(fmt, ap);
 	va_end(ap);
 }
+#endif /* HAVE_LIBUNWIND */
 
 static void print_backtrace_sig_safe(void)
 {
+#ifndef HAVE_LIBUNWIND
+	igt_info("Stack trace: not implemented\n");
+#else
 	unw_cursor_t cursor;
 	unw_context_t uc;
 	int stack_num = 0;
@@ -2238,6 +2267,7 @@ static void print_backtrace_sig_safe(void)
 				(unsigned int) off);
 
 	}
+#endif
 }
 
 void __igt_fail_assert(const char *domain, const char *file, const int line,
@@ -2332,7 +2362,7 @@ void __igt_abort(const char *domain, const char *file, const int line,
  *
  * It is an error to normally exit a test calling igt_exit() - without it the
  * result reporting will be wrong. To avoid such issues it is highly recommended
- * to use #igt_main or #igt_simple_main instead of a hand-rolled main() function.
+ * to use #igt_main() or #igt_simple_main() instead of a hand-rolled main() function.
  */
 void igt_exit(void)
 {
@@ -2568,7 +2598,7 @@ static pthread_mutex_t print_mutex = PTHREAD_MUTEX_INITIALIZER;
 bool __igt_fork(void)
 {
 	internal_assert(!test_with_subtests || in_subtest,
-			"forking is only allowed in subtests or igt_simple_main\n");
+			"forking is only allowed in subtests or igt_simple_main()\n");
 	internal_assert(!test_child,
 			"forking is not allowed from already forked children\n");
 
@@ -2596,6 +2626,7 @@ bool __igt_fork(void)
 		test_child = true;
 		pthread_mutex_init(&print_mutex, NULL);
 		pthread_mutex_init(&ahnd_map_mutex, NULL);
+		pthread_mutex_init(&log_buffer_mutex, NULL);
 		ahnd_map = igt_map_create(igt_map_hash_64, igt_map_equal_64);
 		child_pid = getpid();
 		child_tid = -1;
@@ -2623,7 +2654,7 @@ static void dyn_children_exit_handler(int sig)
 bool __igt_multi_fork(void)
 {
 	internal_assert(!test_with_subtests || in_subtest,
-			"multi-forking is only allowed in subtests or igt_simple_main\n");
+			"multi-forking is only allowed in subtests or igt_simple_main()\n");
 	internal_assert(!test_child,
 			"multi-forking is not allowed from already forked children\n");
 	internal_assert(!test_multi_fork_child,
@@ -3088,10 +3119,10 @@ bool igt_run_in_simulation(void)
  * igt_skip() internally and hence is fully subtest aware.
  *
  * Note that in contrast to all other functions which use igt_skip() internally
- * it is allowed to use this outside of an #igt_fixture block in a test with
+ * it is allowed to use this outside of an #igt_fixture() block in a test with
  * subtests. This is because in contrast to most other test requirements,
  * checking for simulation mode doesn't depend upon the present hardware and it
- * so makes a lot of sense to have this check in the outermost #igt_main block.
+ * so makes a lot of sense to have this check in the outermost #igt_main() block.
  */
 void igt_skip_on_simulation(void)
 {
@@ -3099,7 +3130,7 @@ void igt_skip_on_simulation(void)
 		return;
 
 	if (!igt_can_fail()) {
-		igt_fixture
+		igt_fixture()
 			igt_require(!igt_run_in_simulation());
 	} else
 		igt_require(!igt_run_in_simulation());
@@ -3305,21 +3336,33 @@ void igt_reset_timeout(void)
 	igt_set_timeout(0, NULL);
 }
 
-FILE *__igt_fopen_data(const char* igt_srcdir, const char* igt_datadir,
-		       const char* filename)
+/**
+ * __igt_fopen_data:
+ * @igt_srcdir:  Directory path for source files.
+ * @igt_datadir: Directory path for data files
+ * @igt_imgdir: Directory path for image files.
+ * @filename: Name of the file to be opened.
+ *
+ * This function attempts to open a data file from a list of specified
+ * directories. A file pointer to the opened file. If the file cannot
+ * be opened, it returns NULL and logs a critical error message.
+ *
+ */
+FILE *__igt_fopen_data(const char *igt_srcdir, const char *igt_datadir,
+		       const char *igt_imgdir, const char *filename)
 {
 	char path[PATH_MAX];
 	FILE *fp;
+	const char *dirs[] = {igt_datadir, igt_srcdir, igt_imgdir,
+			      getenv("IGT_DATA_PATH"), "./data"};
 
-	snprintf(path, sizeof(path), "%s/%s", igt_datadir, filename);
-	fp = fopen(path, "r");
-	if (!fp) {
-		snprintf(path, sizeof(path), "%s/%s", igt_srcdir, filename);
-		fp = fopen(path, "r");
-	}
-	if (!fp) {
-		snprintf(path, sizeof(path), "./%s", filename);
-		fp = fopen(path, "r");
+	for (int i = 0; i < ARRAY_SIZE(dirs); i++) {
+		if (dirs[i]) {
+			snprintf(path, sizeof(path), "%s/%s", dirs[i], filename);
+			fp = fopen(path, "r");
+			if (fp)
+				break;
+		}
 	}
 
 	if (!fp)
@@ -3483,21 +3526,105 @@ void igt_srandom(void)
 }
 
 /* IGT wrappers around libpciaccess init/cleanup functions */
+static bool pci_system_initialized;
+static pthread_mutex_t pci_system_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+void igt_pci_system_cleanup(void)
+{
+	pthread_mutex_lock(&pci_system_mutex);
+	if (pci_system_initialized) {
+		pci_system_cleanup();
+		pci_system_initialized = false;
+	}
+	pthread_mutex_unlock(&pci_system_mutex);
+}
 
 static void pci_system_exit_handler(int sig)
 {
-	pci_system_cleanup();
-}
-
-static void __pci_system_init(void)
-{
-	if (!igt_warn_on_f(pci_system_init(), "Could not initialize libpciaccess global data\n"))
-		igt_install_exit_handler(pci_system_exit_handler);
+	igt_pci_system_cleanup();
 }
 
 int igt_pci_system_init(void)
 {
-	static pthread_once_t once_control = PTHREAD_ONCE_INIT;
+	int ret = 0;
+	bool install_handler = false;
 
-	return pthread_once(&once_control, __pci_system_init);
+	pthread_mutex_lock(&pci_system_mutex);
+	if (!pci_system_initialized) {
+		ret = igt_warn_on_f(pci_system_init(),
+				    "Could not initialize libpciaccess global data\n");
+		if (!ret) {
+			pci_system_initialized = true;
+			install_handler = true;
+		}
+	}
+	pthread_mutex_unlock(&pci_system_mutex);
+
+	if (install_handler)
+		igt_install_exit_handler(pci_system_exit_handler);
+
+	return ret;
+}
+
+int igt_pci_system_reinit(void)
+{
+	igt_pci_system_cleanup();
+
+	return igt_pci_system_init();
+}
+
+/**
+ * igt_emit_ignore_dmesg_regex:
+ * @ignore_dmesg: string regex
+ *
+ * Emits a string for igt_runner to ignore next dmesg warns or errors which
+ * matches it.
+ */
+void igt_emit_ignore_dmesg_regex(const char *ignore_dmesg_regex)
+{
+	static const char mark_ignore_dmesg[] = "add ignored dmesg regex: ";
+	GError *err = NULL;
+	GRegex *re;
+
+	re = g_regex_new(ignore_dmesg_regex, G_REGEX_OPTIMIZE, 0, &err);
+	if (err) {
+		igt_debug("regexp: '%s'\n", err->message);
+		g_error_free(err);
+		igt_assert_f(re, "Error in regexp\n");
+	}
+
+	g_regex_unref(re);
+	igt_kmsg(KMSG_INFO "%s%s\n", mark_ignore_dmesg, ignore_dmesg_regex);
+}
+
+/**
+ * @igt_measured_usleep: Helper to model accurate sleep time for tests
+ * @usec: usec to sleep
+ * Return: usec slept
+ */
+unsigned int igt_measured_usleep(unsigned int usec)
+{
+	struct timespec ts = { };
+	unsigned int slept_usec;
+
+	slept_usec = igt_nsec_elapsed(&ts) / NSEC_PER_USEC;
+	igt_assert(slept_usec == 0);
+	do {
+		usleep(usec - slept_usec);
+		slept_usec = igt_nsec_elapsed(&ts) / NSEC_PER_USEC;
+	} while (slept_usec < usec);
+
+	return igt_nsec_elapsed(&ts) / NSEC_PER_USEC;
+}
+
+/**
+ * igt_core_get_igt_hook: To allow for using hooks from other libraries, this
+ * function returns the pointer to the struct that is likely already
+ * initialized.
+ *
+ * Returns: Pointer to the igt_hook structure.
+ */
+struct igt_hook *igt_core_get_igt_hook(void)
+{
+	return igt_hook;
 }
