@@ -67,37 +67,12 @@
  * General debugfs helpers
  */
 
-static bool is_mountpoint(const char *path)
-{
-	char buf[strlen(path) + 4];
-	struct stat st;
-	dev_t dev;
-
-	igt_assert_lt(snprintf(buf, sizeof(buf), "%s/.", path), sizeof(buf));
-	if (stat(buf, &st))
-		return false;
-
-	if (!S_ISDIR(st.st_mode))
-		return false;
-
-	dev = st.st_dev;
-
-	igt_assert_lt(snprintf(buf, sizeof(buf), "%s/..", path), sizeof(buf));
-	if (stat(buf, &st))
-		return false;
-
-	if (!S_ISDIR(st.st_mode))
-		return false;
-
-	return dev != st.st_dev;
-}
-
 static const char *__igt_debugfs_mount(void)
 {
-	if (is_mountpoint("/sys/kernel/debug"))
+	if (igt_is_mountpoint("/sys/kernel/debug"))
 		return "/sys/kernel/debug";
 
-	if (is_mountpoint("/debug"))
+	if (igt_is_mountpoint("/debug"))
 		return "/debug";
 
 	if (mount("debug", "/sys/kernel/debug", "debugfs", 0, 0))
@@ -251,6 +226,37 @@ int igt_debugfs_gt_dir(int device, unsigned int gt)
 	return debugfs_gt_dir_fd;
 }
 
+/**
+ * igt_debugfs_tile_dir:
+ * @device: fd of the device
+ * @tile: tile index
+ *
+ * This opens the debugfs directory corresponding to tile for use
+ * with igt_sysfs_get() and related functions.
+ *
+ * Returns:
+ * The directory fd, or -1 on failure.
+ */
+int igt_debugfs_tile_dir(int device, uint8_t tile)
+{
+	int debugfs_tile_dir_fd;
+	char path[PATH_MAX];
+	size_t len;
+	int ret;
+
+	if (!igt_debugfs_path(device, path, sizeof(path)))
+		return -1;
+
+	len = strlen(path);
+	ret = snprintf(path + len, sizeof(path) - len, "/tile%u", tile);
+	igt_assert(ret >= 0 && (size_t)ret < sizeof(path) - len);
+
+	debugfs_tile_dir_fd = open(path, O_RDONLY);
+	igt_debug_on_f(debugfs_tile_dir_fd < 0, "path: %s\n", path);
+
+	return debugfs_tile_dir_fd;
+}
+
 /**
  * igt_debugfs_connector_dir:
  * @device: fd of the device
@@ -376,6 +382,33 @@ igt_debugfs_gt_open(int device, unsigned int gt, const char *filename, int mode)
 	return ret;
 }
 
+/**
+ * igt_debugfs_is_dir:
+ * @drm_fd: fd of the device
+ * @name: name of the debugfs node to check
+ * @gt: gt instance number
+ *
+ * This helps to find the debugfs file is  a directory or not
+ *
+ * Returns:
+ * true if the debugfs is a directory
+ */
+bool igt_debugfs_is_dir(int drm_fd, const char *name, int gt_id)
+{
+	char path[128];
+	struct stat st;
+
+	if (fstat(drm_fd, &st) != 0)
+		return false;
+
+	snprintf(path, sizeof(path), "/sys/kernel/debug/dri/%d/gt%d/%s",
+		 minor(st.st_rdev), gt_id, name);
+	if (!stat(path, &st) && S_ISDIR(st.st_mode))
+		return true;
+
+	return false;
+}
+
 /**
  * igt_debugfs_simple_read:
  * @dir: fd of the debugfs directory
@@ -502,7 +535,7 @@ bool igt_ignore_long_hpd(int drm_fd, bool enable)
 
 static void igt_hpd_storm_exit_handler(int sig)
 {
-	int fd = drm_open_driver(DRIVER_INTEL);
+	int fd = drm_open_driver(DRIVER_INTEL | DRIVER_XE);
 
 	/* Here we assume that only one i915 device will be ever present */
 	igt_hpd_storm_reset(fd);
