@@ -30,6 +30,7 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
+#include <pciaccess.h>
 
 #include "drmtest.h"
 #include "i915/gem.h"
@@ -44,6 +45,7 @@
 #include "intel_reg.h"
 #include "intel_chipset.h"
 #include "igt_dummyload.h"
+#include "xe/xe_gt.h"
 
 /**
  * SECTION:igt_gt
@@ -56,23 +58,28 @@
  * engines.
  */
 
+static int reset_query_once = -1;
+
 static bool has_gpu_reset(int fd)
 {
-	static int once = -1;
-	if (once < 0) {
-		struct drm_i915_getparam gp;
-		int val = 0;
+	if (reset_query_once < 0) {
+		reset_query_once = gem_gpu_reset_type(fd);
 
-		memset(&gp, 0, sizeof(gp));
-		gp.param = 35; /* HAS_GPU_RESET */
-		gp.value = &val;
-
-		if (ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp))
-			once = intel_gen(intel_get_drm_devid(fd)) >= 5;
-		else
-			once = val > 0;
+		/* Very old kernels did not support the query */
+		if (reset_query_once == -1)
+			reset_query_once =
+			      (intel_gen(intel_get_drm_devid(fd)) >= 5) ? 1 : 0;
 	}
-	return once;
+
+	return reset_query_once > 0;
+}
+
+static bool has_engine_reset(int fd)
+{
+	if (reset_query_once < 0)
+		has_gpu_reset(fd);
+
+	return reset_query_once > 1;
 }
 
 static void eat_error_state(int dev)
@@ -117,12 +124,12 @@ static void eat_error_state(int dev)
  * to be done under hang injection.
  * Default: false
  */
-void igt_require_hang_ring(int fd, int ring)
+void igt_require_hang_ring(int fd, uint32_t ctx, int ring)
 {
 	if (!igt_check_boolean_env_var("IGT_HANG", true))
 		igt_skip("hang injection disabled by user [IGT_HANG=0]\n");
 
-	gem_require_ring(fd, ring);
+        igt_require(gem_context_has_engine(fd, ctx, ring));
 	gem_context_require_bannable(fd);
 	if (!igt_check_boolean_env_var("IGT_HANG_WITHOUT_RESET", false))
 		igt_require(has_gpu_reset(fd));
@@ -174,9 +181,19 @@ igt_hang_t igt_allow_hang(int fd, unsigned ctx, unsigned flags)
 	 */
 	if (!igt_check_boolean_env_var("IGT_HANG", true))
 		igt_skip("hang injection disabled by user [IGT_HANG=0]\n");
+
+	if (is_xe_device(fd)) {
+		igt_require(has_gpu_reset(fd));
+		return (struct igt_hang){ 0, ctx, 0, flags };
+	}
+
 	gem_context_require_bannable(fd);
 
-	allow_reset = 1;
+	if (flags & HANG_WANT_ENGINE_RESET)
+		allow_reset = 2;
+	else
+		allow_reset = 1;
+
 	if ((flags & HANG_ALLOW_CAPTURE) == 0) {
 		param.param = I915_CONTEXT_PARAM_NO_ERROR_CAPTURE;
 		param.value = 1;
@@ -187,11 +204,16 @@ igt_hang_t igt_allow_hang(int fd, unsigned ctx, unsigned flags)
 		__gem_context_set_param(fd, &param);
 		allow_reset = INT_MAX; /* any reset method */
 	}
+
 	igt_require(igt_params_set(fd, "reset", "%d", allow_reset));
+	reset_query_once = -1;  /* Re-query after changing param */
 
 	if (!igt_check_boolean_env_var("IGT_HANG_WITHOUT_RESET", false))
 		igt_require(has_gpu_reset(fd));
 
+	if (flags & HANG_WANT_ENGINE_RESET)
+		igt_require(has_engine_reset(fd));
+
 	ban = context_get_ban(fd, ctx);
 	if ((flags & HANG_ALLOW_BAN) == 0)
 		context_set_ban(fd, ctx, 0);
@@ -201,6 +223,9 @@ igt_hang_t igt_allow_hang(int fd, unsigned ctx, unsigned flags)
 
 void igt_disallow_hang(int fd, igt_hang_t arg)
 {
+	if (is_xe_device(fd))
+		return;
+
 	context_set_ban(fd, arg.ctx, arg.ban);
 
 	if ((arg.flags & HANG_ALLOW_CAPTURE) == 0) {
@@ -255,8 +280,8 @@ static bool has_ctx_exec(int fd, unsigned ring, uint32_t ctx)
 
 /**
  * igt_hang_ring_ctx:
- * @fd: open i915 drm file descriptor
- * @ctx: the contxt specifier
+ * @fd: open i915/xe drm file descriptor
+ * @ctx: the context specifier
  * @ring: execbuf ring flag
  * @flags: set of flags to control execution
  * @offset: The resultant gtt offset of the exec obj
@@ -269,13 +294,17 @@ static bool has_ctx_exec(int fd, unsigned ring, uint32_t ctx)
  * Returns:
  * Structure with helper internal state for igt_post_hang_ring().
  */
-igt_hang_t igt_hang_ctx(int fd, uint32_t ctx, int ring, unsigned flags)
+static igt_hang_t __igt_hang_ctx(int fd, uint64_t ahnd, uint32_t ctx, int ring,
+				 unsigned flags)
 {
 	struct drm_i915_gem_context_param param;
 	igt_spin_t *spin;
 	unsigned ban;
 
-	igt_require_hang_ring(fd, ring);
+	if (is_xe_device(fd))
+		return xe_hang_ring(fd, ahnd, ctx, ring, flags);
+
+	igt_require_hang_ring(fd, ctx, ring);
 
 	/* check if non-default ctx submission is allowed */
 	igt_require(ctx == 0 || has_ctx_exec(fd, ring, ctx));
@@ -298,16 +327,28 @@ igt_hang_t igt_hang_ctx(int fd, uint32_t ctx, int ring, unsigned flags)
 		context_set_ban(fd, ctx, 0);
 
 	spin = __igt_spin_new(fd,
-			      .ctx = ctx,
+			      .ahnd = ahnd,
+			      .ctx_id = ctx,
 			      .engine = ring,
 			      .flags = IGT_SPIN_NO_PREEMPTION);
 
 	return (igt_hang_t){ spin, ctx, ban, flags };
 }
 
+igt_hang_t igt_hang_ctx(int fd, uint32_t ctx, int ring, unsigned flags)
+{
+	return __igt_hang_ctx(fd, 0, ctx, ring, flags);
+}
+
+igt_hang_t igt_hang_ctx_with_ahnd(int fd, uint64_t ahnd, uint32_t ctx, int ring,
+				  unsigned flags)
+{
+	return __igt_hang_ctx(fd, ahnd, ctx, ring, flags);
+}
+
 /**
  * igt_hang_ring:
- * @fd: open i915 drm file descriptor
+ * @fd: open i915/xe drm file descriptor
  * @ring: execbuf ring flag
  *
  * This helper function injects a hanging batch into @ring. It returns a
@@ -322,9 +363,15 @@ igt_hang_t igt_hang_ring(int fd, int ring)
 	return igt_hang_ctx(fd, 0, ring, 0);
 }
 
+igt_hang_t igt_hang_ring_with_ahnd(int fd, int ring, uint64_t ahnd)
+{
+	return igt_hang_ctx_with_ahnd(fd, ahnd, 0, ring, 0);
+}
+
+
 /**
  * igt_post_hang_ring:
- * @fd: open i915 drm file descriptor
+ * @fd: open i915/xe drm file descriptor
  * @arg: hang state from igt_hang_ring()
  *
  * This function does the necessary post-processing after a gpu hang injected
@@ -335,6 +382,13 @@ void igt_post_hang_ring(int fd, igt_hang_t arg)
 	if (!arg.spin)
 		return;
 
+	if (is_xe_device(fd)) {
+		igt_spin_free(fd, arg.spin);
+		xe_post_hang_ring(fd, arg);
+
+		return;
+	}
+
 	gem_sync(fd, arg.spin->handle); /* Wait until it hangs */
 	igt_spin_free(fd, arg.spin);
 
@@ -366,6 +420,9 @@ void igt_force_gpu_reset(int drm_fd)
 
 	igt_debug("Triggering GPU reset\n");
 
+	if (is_xe_device(drm_fd))
+		xe_force_gt_reset_all(drm_fd);
+
 	dir = igt_debugfs_dir(drm_fd);
 
 	wedged = 0;
@@ -426,7 +483,7 @@ void igt_fork_hang_helper(void)
  * Stops the child process spawned with igt_fork_hang_helper().
  *
  * In tests with subtests this function can be called outside of failure
- * catching code blocks like #igt_fixture or #igt_subtest.
+ * catching code blocks like #igt_fixture() or #igt_subtest.
  */
 void igt_stop_hang_helper(void)
 {
@@ -436,6 +493,7 @@ void igt_stop_hang_helper(void)
 
 /**
  * igt_open_forcewake_handle:
+ * @fd: drm fd - i915 or xe
  *
  * This functions opens the debugfs forcewake file and so prevents the GT from
  * suspending. The reference is automatically dropped when the is closed.
@@ -445,9 +503,61 @@ void igt_stop_hang_helper(void)
  */
 int igt_open_forcewake_handle(int fd)
 {
+	const char *fn;
+
 	if (getenv("IGT_NO_FORCEWAKE"))
 		return -1;
-	return igt_debugfs_open(fd, "i915_forcewake_user", O_WRONLY);
+
+	if (is_xe_device(fd))
+		fn = "forcewake_all";
+	else if (is_i915_device(fd))
+		fn = "i915_forcewake_user";
+	else
+		return -1;
+
+	return igt_debugfs_open(fd, fn, O_RDONLY);
+}
+
+int igt_open_forcewake_handle_for_pcidev(const struct pci_device *pci_dev)
+{
+	static const char *FORCEWAKE_FILES[] = {
+		"forcewake_all", "i915_forcewake_user",
+		NULL
+	};
+	const char *debugfs_root = igt_debugfs_mount(), **fn;
+	char path[PATH_MAX];
+	int dirlen, ret;
+
+	if (getenv("IGT_NO_FORCEWAKE"))
+		return -1;
+
+	if (!debugfs_root)
+		return -ENOENT;
+
+	ret = snprintf(path, sizeof(path), "%s/dri/%04x:%02x:%02x.%x",
+		       debugfs_root,
+		       pci_dev->domain, pci_dev->bus, pci_dev->dev,
+		       pci_dev->func);
+	if (ret < 0 || ret >= sizeof(path))
+		return -EINVAL;
+
+	dirlen = ret;
+	for (fn = FORCEWAKE_FILES; *fn; fn++) {
+		ret = snprintf(path + dirlen, sizeof(path) - dirlen,
+			       "/%s", *fn);
+		if (ret < 0 || ret >= sizeof(path) - dirlen)
+			return -EINVAL;
+
+		ret = open(path, O_RDONLY);
+		if (ret >= 0)
+			return ret;
+
+		if (errno != ENOENT)
+			return -errno;
+	}
+
+	return -ENOENT;
+
 }
 
 #if defined(__x86_64__) || defined(__i386__)
@@ -548,22 +658,36 @@ unsigned intel_detect_and_clear_missed_interrupts(int fd)
 	return missed;
 }
 
+static bool gem_store_dword_needs_physical(const struct intel_device_info *info)
+{
+	switch (info->graphics_ver) {
+	case 2:
+		return true;
+	case 3:
+		return info->is_grantsdale || info->is_alviso;
+	default:
+		return false;
+	}
+}
+
+/**
+ * gem_class_can_store_dword:
+ * @fd: open i915 drm file descriptor
+ * @class: engine class
+ *
+ * Returns:
+ * True if MI_STORE_DWORD is actually usable.
+ */
 bool gem_class_can_store_dword(int fd, int class)
 {
 	uint16_t devid = intel_get_drm_devid(fd);
 	const struct intel_device_info *info = intel_get_device_info(devid);
-	const int gen = ffs(info->gen);
+	const int ver = info->graphics_ver;
 
-	if (info->gen == 0) /* unknown, assume it just works */
-		return true;
-
-	if (gen <= 2) /* requires physical addresses */
+	if (gem_store_dword_needs_physical(info))
 		return false;
 
-	if (gen == 3 && (info->is_grantsdale || info->is_alviso))
-		return false; /* only supports physical addresses */
-
-	if (gen == 6 && class == I915_ENGINE_CLASS_VIDEO)
+	if (ver == 6 && class == I915_ENGINE_CLASS_VIDEO)
 		return false; /* broken, unbelievably broken */
 
 	if (info->is_broadwater)
@@ -572,18 +696,47 @@ bool gem_class_can_store_dword(int fd, int class)
 	return true;
 }
 
+/**
+ * gem_can_store_dword:
+ * @fd: open i915 drm file descriptor
+ * @engine: engine
+ *
+ * Returns:
+ * True if MI_STORE_DWORD is actually usable.
+ */
 bool gem_can_store_dword(int fd, unsigned int engine)
 {
 	return gem_class_can_store_dword(fd,
 				gem_execbuf_flags_to_engine_class(engine));
 }
 
+/**
+ * gem_store_dword_needs_secure:
+ * @fd: open i915 drm file descriptor
+ *
+ * Returns:
+ * True if the MI_STORE_DWORD needs to be executed from a secure batch.
+ */
+bool gem_store_dword_needs_secure(int fd)
+{
+	const struct intel_device_info *info =
+		intel_get_device_info(intel_get_drm_devid(fd));
+
+	switch (info->graphics_ver) {
+	case 4:
+		return info->is_eaglelake || info->is_cantiga;
+	case 5:
+		return true;
+	default:
+		return false;
+	}
+}
+
 const struct intel_execution_engine2 intel_execution_engines2[] = {
 	{ "rcs0", I915_ENGINE_CLASS_RENDER, 0, I915_EXEC_RENDER },
 	{ "bcs0", I915_ENGINE_CLASS_COPY, 0, I915_EXEC_BLT },
 	{ "vcs0", I915_ENGINE_CLASS_VIDEO, 0, I915_EXEC_BSD | I915_EXEC_BSD_RING1 },
 	{ "vcs1", I915_ENGINE_CLASS_VIDEO, 1, I915_EXEC_BSD | I915_EXEC_BSD_RING2 },
-	{ "vcs2", I915_ENGINE_CLASS_VIDEO, 2, -1 },
 	{ "vecs0", I915_ENGINE_CLASS_VIDEO_ENHANCE, 0, I915_EXEC_VEBOX },
 	{ }
 };
@@ -604,3 +757,18 @@ int gem_execbuf_flags_to_engine_class(unsigned int flags)
 		igt_assert(0);
 	}
 }
+
+/**
+ * gem_engine_can_block_ggtt_binder:
+ * @fd: open i915 drm file descriptor
+ * @engine: engine to be assessed
+ *
+ * Detect if the platform needs blitter based GGTT
+ * updates.
+ */
+bool gem_engine_can_block_ggtt_binder(int fd,
+		const struct intel_execution_engine2 *engine)
+{
+	return IS_METEORLAKE(intel_get_drm_devid(fd)) &&
+		engine->class == I915_ENGINE_CLASS_COPY;
+}
