@@ -40,10 +40,11 @@
 #include <dirent.h>
 
 #include "drmtest.h"
+#include "igt_aux.h"
 #include "igt_device_scan.h"
 #include "igt_kms.h"
+#include "igt_pci.h"
 #include "igt_pm.h"
-#include "igt_aux.h"
 #include "igt_sysfs.h"
 
 /**
@@ -81,6 +82,7 @@ enum {
 #define MAX_POLICY_STRLEN	strlen(MAX_PERFORMANCE_STR)
 /* Root Port bus can have max 32 dev and each dev can have max 8 func */
 #define MAX_PCI_DEVICES		256
+#define PCI_PM_CAP_ID 0x01
 int8_t *__sata_pm_policies;
 int __scsi_host_cnt;
 
@@ -319,6 +321,35 @@ void igt_pm_enable_audio_runtime_pm(void)
 		igt_debug("Failed to enable audio runtime PM! (%d)\n", -err);
 }
 
+static bool lpm_supported_by_sata_host(int host_num)
+{
+	int fd;
+	ssize_t len;
+	char buf[2];
+	char file_name[PATH_MAX];
+
+	snprintf(file_name, PATH_MAX,
+		 "/sys/class/scsi_host/host%d/link_power_management_supported",
+		 host_num);
+
+	fd = open(file_name, O_RDONLY);
+
+	/* assume LPM is supported if sysfs entry is absent. This preserves default behaviour */
+	if (fd < 0)
+		return true;
+
+	len = read(fd, buf, 1);
+
+	close(fd);
+
+	if (len <= 0)
+		return true;
+
+	buf[len] = '\0';
+
+	return buf[0] == '1';
+}
+
 static void __igt_pm_enable_sata_link_power_management(void)
 {
 	int fd, i;
@@ -370,6 +401,9 @@ static void __igt_pm_enable_sata_link_power_management(void)
 	igt_install_exit_handler(__igt_pm_sata_link_pm_exit_handler);
 
 	for (i = 0; i < __scsi_host_cnt; i++) {
+		if (!lpm_supported_by_sata_host(i))
+			continue;
+
 		snprintf(file_name, PATH_MAX,
 			 "/sys/class/scsi_host/host%d/link_power_management_policy",
 			 i);
@@ -410,6 +444,9 @@ static void __igt_pm_restore_sata_link_power_management(void)
 	for (i = 0; i < __scsi_host_cnt; i++) {
 		int8_t policy;
 
+		if (!lpm_supported_by_sata_host(i))
+			continue;
+
 		if (__sata_pm_policies[i] == POLICY_UNKNOWN)
 			continue;
 		else
@@ -1400,7 +1437,7 @@ uint64_t igt_pm_get_runtime_suspended_time(struct pci_device *pci_dev)
 
 	time_fd = igt_pm_get_power_attr_fd_rdonly(pci_dev, "runtime_suspended_time");
 	if (igt_pm_read_power_attr(time_fd, time_str, 64, false)) {
-		igt_assert(sscanf(time_str, "%ld", &time) > 0);
+		igt_assert(sscanf(time_str, "%"PRId64"", &time) > 0);
 
 		igt_debug("runtime suspended time for PCI '%04x:%02x:%02x.%01x' = %" PRIu64 "\n",
 			  pci_dev->domain, pci_dev->bus, pci_dev->dev, pci_dev->func, time);
@@ -1425,7 +1462,7 @@ uint64_t igt_pm_get_runtime_active_time(struct pci_device *pci_dev)
 
 	time_fd = igt_pm_get_power_attr_fd_rdonly(pci_dev, "runtime_active_time");
 	if (igt_pm_read_power_attr(time_fd, time_str, 64, false)) {
-		igt_assert(sscanf(time_str, "%ld", &time) > 0);
+		igt_assert(sscanf(time_str, "%"PRId64"", &time) > 0);
 
 		igt_debug("runtime active time for PCI '%04x:%02x:%02x.%01x' = %" PRIu64 "\n",
 			  pci_dev->domain, pci_dev->bus, pci_dev->dev, pci_dev->func, time);
@@ -1470,3 +1507,39 @@ void igt_pm_ignore_slpc_efficient_freq(int i915, int gtfd, bool val)
 	igt_require(igt_sysfs_has_attr(gtfd, "slpc_ignore_eff_freq"));
 	igt_sysfs_set_u32(gtfd, "slpc_ignore_eff_freq", val);
 }
+
+/**
+ * igt_has_pci_pm_capability:
+ * @pci_dev: PCI device struct
+ *
+ * Returns: true if the device has PCI Power Management capability, false otherwise.
+ */
+bool igt_has_pci_pm_capability(struct pci_device *pci_dev)
+{
+	int offset;
+
+	offset = find_pci_cap_offset(pci_dev, PCI_PM_CAP_ID);
+
+	return (offset > 0);
+}
+
+/**
+ * igt_pm_dpms_toggle:
+ * @output: igt output for which DPMS toggle has to be performed
+ *
+ * Toggles the DPMS state of output to OFF and then back ON.
+ */
+void igt_pm_dpms_toggle(igt_output_t *output)
+{
+	igt_require(igt_setup_runtime_pm(output->display->drm_fd));
+
+	kmstest_set_connector_dpms(output->display->drm_fd,
+				   output->config.connector,
+				   DRM_MODE_DPMS_OFF);
+	igt_require(igt_wait_for_pm_status(IGT_RUNTIME_PM_STATUS_SUSPENDED));
+
+	kmstest_set_connector_dpms(output->display->drm_fd,
+				   output->config.connector,
+				   DRM_MODE_DPMS_ON);
+	igt_assert(igt_wait_for_pm_status(IGT_RUNTIME_PM_STATUS_ACTIVE));
+}
