@@ -10,7 +10,7 @@
 
 /* Tests are not defined here so we can keep most of the functions static */
 
-igt_simple_main
+int igt_simple_main()
 {
 	igt_info("Running igt_facts_test\n");
 
