@@ -41,6 +41,7 @@ struct match_dto {
 	uint64_t *bind_op_seqno;
 };
 
+#define TOKEN_NONE  0
 #define CLIENT_PID  1
 #define CLIENT_RUN  2
 #define CLIENT_FINI 3
@@ -51,6 +52,7 @@ struct match_dto {
 static const char *token_to_str(uint64_t token)
 {
 	static const char * const s[] = {
+		"none",
 		"client pid",
 		"client run",
 		"client fini",
@@ -59,8 +61,12 @@ static const char *token_to_str(uint64_t token)
 		"debugger stage",
 	};
 
-	if (token >= ARRAY_SIZE(s))
+	igt_assert(token);
+
+	if (token >= ARRAY_SIZE(s)) {
+		igt_warn("token outside of bounds %ld\n", token);
 		return "unknown";
+	}
 
 	return s[token];
 }
@@ -254,7 +260,7 @@ static int safe_pipe_read(int pipe[2], void *buf, int nbytes, int timeout_ms)
 {
 	int ret;
 	int t = 0;
-	struct pollfd fd = {
+	struct pollfd r = {
 		.fd = pipe[0],
 		.events = POLLIN,
 		.revents = 0
@@ -266,17 +272,33 @@ static int safe_pipe_read(int pipe[2], void *buf, int nbytes, int timeout_ms)
 	do {
 		const int interval_ms = 1000;
 
-		ret = poll(&fd, 1, interval_ms);
-
+		ret = poll(&r, 1, interval_ms);
 		if (!ret) {
+			igt_debug("poll: timeout\n");
 			catch_child_failure();
 			t += interval_ms;
+		} else if (ret == -1) {
+			if (errno == EINTR) {
+				ret = 0;
+				continue;
+			}
+			return -errno;
+		}
+
+		if (ret == 1) {
+			if (r.revents == POLLIN)
+				return read(pipe[0], buf, nbytes);
+
+			if (r.revents & ~POLLIN) {
+				igt_debug("pipe read failed: %s%s (0x%x)\n",
+					  r.revents & POLLHUP ? "pipe closed" : "",
+					  r.revents & POLLERR ? "poll error" : "",
+					  r.revents);
+				return -EIO;
+			}
 		}
 	} while (!ret && t < timeout_ms);
 
-	if (ret > 0)
-		return read(pipe[0], buf, nbytes);
-
 	return -ETIMEDOUT;
 }
 
@@ -294,38 +316,69 @@ static void pipe_close(int pipe[2])
 		close(pipe[1]);
 }
 
+#define DEAD_CLIENT 0xccccdead
+
 static uint64_t __wait_token(int pipe[2], const uint64_t token, int timeout_ms)
 {
 	uint64_t in;
 	int ret;
 
 	ret = safe_pipe_read(pipe, &in, sizeof(in), timeout_ms);
-	igt_assert_f(ret > 0,
-		     "Pipe read timeout waiting for token '%s:(%" PRId64 ")'\n",
-		     token_to_str(token), token);
+	if (ret < 0) {
+		igt_debug("safe_pipe_read failed with error: %d waiting for token '%s:(%" PRId64 ")'\n",
+			  ret, token_to_str(token), token);
+		return DEAD_CLIENT;
+	} else if (ret == 0) {
+		igt_debug("safe_pipe_read failed: EOF\n");
+		return DEAD_CLIENT;
+	}
 
 	igt_assert_eq(in, token);
 
 	ret = safe_pipe_read(pipe, &in, sizeof(in), timeout_ms);
-	igt_assert_f(ret > 0,
-		     "Pipe read timeout waiting for token value '%s:(%" PRId64 ")'\n",
-		     token_to_str(token), token);
+	if (ret < 0) {
+		igt_debug("safe_pipe_read failed with error: %d waiting for token '%s:(%" PRId64 ")'\n",
+			  ret, token_to_str(token), token);
+		return DEAD_CLIENT;
+	} else if (ret == 0) {
+		igt_debug("safe_pipe_read failed: EOF\n");
+		return DEAD_CLIENT;
+	}
 
 	return in;
 }
 
 static uint64_t client_wait_token(struct xe_eudebug_client *c, const uint64_t token)
 {
-	return __wait_token(c->p_in, token, c->timeout_ms);
+	uint64_t ret = 0;
+
+	igt_debug("client: %d waiting for token '%s'\n", getpid(), token_to_str(token));
+
+	ret = __wait_token(c->p_in, token, c->timeout_ms);
+	if (ret == DEAD_CLIENT)
+		igt_assert(c->allow_dead_client);
+
+	return ret;
 }
 
 static uint64_t wait_from_client(struct xe_eudebug_client *c, const uint64_t token)
 {
-	return __wait_token(c->p_out, token, c->timeout_ms);
+	uint64_t ret = 0;
+
+	igt_debug("debugger: %d waiting for token '%s'\n", getpid(), token_to_str(token));
+
+	ret = __wait_token(c->p_out, token, c->timeout_ms);
+	if (ret == DEAD_CLIENT)
+		igt_assert(c->allow_dead_client);
+
+	return ret;
 }
 
 static void token_signal(int pipe[2], const uint64_t token, const uint64_t value)
 {
+	igt_debug("%d signalling token '%s' with value '%ld'\n",
+		  getpid(), token_to_str(token), value);
+
 	pipe_signal(pipe, token);
 	pipe_signal(pipe, value);
 }
@@ -815,6 +868,9 @@ static void event_log_sort(struct xe_eudebug_event_log *l)
 		events++;
 	}
 
+	if (!events)
+		return;
+
 	tmp = xe_eudebug_event_log_create("tmp", l->max_size);
 
 	for (i = first_seqno; i <= last_seqno; i++) {
@@ -1033,7 +1089,6 @@ static void debugger_run_triggers(struct xe_eudebug_debugger *d,
 	}
 }
 
-#define MAX_EVENT_SIZE (32 * 1024)
 static int
 xe_eudebug_read_event(int fd, struct drm_xe_eudebug_event *event)
 {
@@ -1050,9 +1105,16 @@ xe_eudebug_read_event(int fd, struct drm_xe_eudebug_event *event)
 	return ret;
 }
 
-static void terminate_debugger(int sig)
+static void debugger_signal_handler(int sig, siginfo_t *info, void *context)
 {
-	pthread_exit(NULL);
+	struct xe_eudebug_debugger *d = info->si_ptr;
+
+	igt_assert(d);
+
+	d->received_signal = true;
+
+	if (sig == SIGINT)
+		d->received_sigint = true;
 }
 
 static void *debugger_worker_loop(void *data)
@@ -1070,14 +1132,31 @@ static void *debugger_worker_loop(void *data)
 	igt_assert(d->master_fd >= 0);
 
 	igt_assert_eq(sigaction(SIGINT, NULL, &sa), 0);
-	sa.sa_handler = terminate_debugger;
+	sa.sa_sigaction = debugger_signal_handler;
+	sa.sa_flags |= SA_SIGINFO;
 	igt_assert_eq(sigaction(SIGINT, &sa, NULL), 0);
 
+	igt_assert_eq(sigaction(SIGTERM, NULL, &sa), 0);
+	sa.sa_sigaction = debugger_signal_handler;
+	sa.sa_flags |= SA_SIGINFO;
+	igt_assert_eq(sigaction(SIGTERM, &sa, NULL), 0);
+
 	do {
 		p.fd = d->fd;
 		ret = poll(&p, 1, timeout_ms);
+		if (d->received_sigint) {
+			d->handled_sigint = true;
+			pthread_exit(NULL);
+		}
 
 		if (ret == -1) {
+			if (d->received_signal) {
+				d->received_signal = false;
+
+				if (errno == EINTR)
+					continue;
+			}
+
 			igt_info("poll failed with errno %d\n", errno);
 			break;
 		}
@@ -1146,6 +1225,9 @@ xe_eudebug_debugger_create(int master_fd, uint64_t flags, void *data)
 	d->fd = -1;
 	d->master_fd = master_fd;
 	d->ptr = data;
+	d->received_sigint = false;
+	d->handled_sigint = false;
+	d->received_signal = false;
 
 	return d;
 }
@@ -1168,7 +1250,7 @@ static void debugger_destroy_triggers(struct xe_eudebug_debugger *d)
 void xe_eudebug_debugger_destroy(struct xe_eudebug_debugger *d)
 {
 	if (d->worker_state != DEBUGGER_WORKER_INACTIVE)
-		xe_eudebug_debugger_stop_worker(d, 1);
+		xe_eudebug_debugger_stop_worker(d);
 
 	if (d->target_pid)
 		xe_eudebug_debugger_detach(d);
@@ -1305,9 +1387,9 @@ void xe_eudebug_debugger_start_worker(struct xe_eudebug_debugger *d)
  *
  * Stops the debugger worker. Event log is sorted by seqno after closure.
  */
-void xe_eudebug_debugger_stop_worker(struct xe_eudebug_debugger *d,
-				     int timeout_s)
+void xe_eudebug_debugger_stop_worker(struct xe_eudebug_debugger *d)
 {
+	const int timeout_s = 3;
 	struct timespec t = {};
 	int ret;
 
@@ -1367,6 +1449,19 @@ void xe_eudebug_debugger_wait_stage(struct xe_eudebug_session *s, uint64_t stage
 	igt_assert_eq(stage_in, stage);
 }
 
+/**
+ * xe_eudebug_debugger_kill:
+ * @d: pointer to the debugger
+ * @sig: signal to send
+ *
+ * Sends @sig signal to the debugger thread.
+ * Passes the debugger struct to signal handler.
+ */
+void xe_eudebug_debugger_kill(struct xe_eudebug_debugger *d, int sig)
+{
+	pthread_sigqueue(d->worker_thread, sig, (union sigval){ .sival_ptr = (void*)d });
+}
+
 /**
  * xe_eudebug_client_create:
  * @master_fd: xe client used to open the debugger connection
@@ -1400,6 +1495,7 @@ struct xe_eudebug_client *xe_eudebug_client_create(int master_fd, xe_eudebug_cli
 	c->ptr = data;
 	c->master_fd = master_fd;
 	c->timeout_ms = XE_EUDEBUG_DEFAULT_TIMEOUT_SEC * MSEC_PER_SEC;
+	c->allow_dead_client = false;
 	pthread_mutex_init(&c->lock, NULL);
 
 	igt_fork(child, 1) {
@@ -1417,8 +1513,11 @@ struct xe_eudebug_client *xe_eudebug_client_create(int master_fd, xe_eudebug_cli
 
 		c->pid = client_wait_token(c, CLIENT_RUN);
 		igt_assert_eq(c->pid, mypid);
-		if (work)
-			work(c);
+		igt_assert(work);
+
+		igt_debug("client: work start\n");
+		work(c);
+		igt_debug("client: work end\n");
 
 		client_signal(c, CLIENT_FINI, c->seqno);
 
@@ -1519,9 +1618,9 @@ void xe_eudebug_client_start(struct xe_eudebug_client *c)
 void xe_eudebug_client_wait_done(struct xe_eudebug_client *c)
 {
 	if (!c->done) {
-		c->done = 1;
 		c->seqno = wait_from_client(c, CLIENT_FINI);
 		event_log_read_from_fd(c->log, c->p_out[0]);
+		c->done = 1;
 	}
 }
 
@@ -1614,7 +1713,7 @@ void xe_eudebug_session_run(struct xe_eudebug_session *s)
 	xe_eudebug_client_start(client);
 	xe_eudebug_client_wait_done(client);
 
-	xe_eudebug_debugger_stop_worker(debugger, 1);
+	xe_eudebug_debugger_stop_worker(debugger);
 
 	xe_eudebug_event_log_print(debugger->log, true);
 	xe_eudebug_event_log_print(client->log, true);
