@@ -524,7 +524,7 @@ static const char *find_subtest_begin_limit_limited(struct matches matches,
 	if (begin_idx < first_idx) {
 		/*
 		 * Subtest didn't start, but we have the
-		 * result. Probably because an igt_fixture
+		 * result. Probably because an igt_fixture()
 		 * made it fail/skip.
 		 *
 		 * We go backwards one match from the result match,
@@ -829,23 +829,65 @@ static const char igt_dmesg_whitelist[] =
 static const char igt_piglit_style_dmesg_blacklist[] =
 	"(\\[drm:|drm_|intel_|i915_|\\[drm\\])";
 
-static bool init_regex_whitelist(struct settings* settings, GRegex **re)
+static bool init_dmesg_regex(GRegex **re, const char *regex, const char *msg)
 {
 	GError *err = NULL;
-	const char *regex = settings->piglit_style_dmesg ?
-		igt_piglit_style_dmesg_blacklist :
-		igt_dmesg_whitelist;
 
 	*re = g_regex_new(regex, G_REGEX_OPTIMIZE, 0, &err);
 	if (err) {
-		fprintf(stderr, "Cannot compile dmesg regexp\n");
+		fprintf(stderr, "Cannot compile %s : %s\n",
+			msg, err->message);
 		g_error_free(err);
+
 		return false;
 	}
 
 	return true;
 }
 
+static bool init_regex_whitelist(struct settings *settings, GRegex **re)
+{
+	const char *regex = settings->piglit_style_dmesg ?
+		igt_piglit_style_dmesg_blacklist :
+		igt_dmesg_whitelist;
+	const char *what = settings->piglit_style_dmesg ?
+		"piglit style dmesg blocklist" :
+		"igt dmesg whitelist";
+
+	return init_dmesg_regex(re, regex, what);
+}
+
+static bool not_ignored(GRegex *re, const char *msg)
+{
+	if (!re)
+		return true;
+
+	return !g_regex_match(re, msg, 0, NULL);
+}
+
+static void clean_regex(GRegex **re)
+{
+	if (*re)
+		g_regex_unref(*re);
+
+	*re = NULL;
+}
+
+static void add_ignored_regex(GRegex **re, char *src)
+{
+	char *s;
+
+	s = strchr(src, '\n');
+	if (s)
+		*s = 0;
+
+	if (*re)
+		g_regex_unref(*re);
+
+	init_dmesg_regex(re, src, "ignore match");
+	fprintf(stderr, "igt_resultgen: Added ignore regex '%s'\n", src);
+}
+
 static bool parse_dmesg_line(char* line,
 			     unsigned *flags, unsigned long long *ts_usec,
 			     char *continuation, char **message)
@@ -979,6 +1021,7 @@ static bool fill_from_dmesg(int fd,
 	char dynamic_piglit_name[256];
 	size_t i;
 	GRegex *re;
+	GRegex *re_ignore; /* regex for dynamically ignored dmesg line */
 
 	if (!f) {
 		return false;
@@ -989,12 +1032,13 @@ static bool fill_from_dmesg(int fd,
 		return false;
 	}
 
+	re_ignore = NULL;
 	while (getline(&line, &linelen, f) > 0) {
 		char *formatted;
 		unsigned flags;
 		unsigned long long ts_usec;
 		char continuation;
-		char *message, *subtest, *dynamic_subtest;
+		char *message, *subtest, *dynamic_subtest, *ignore;
 
 		if (!parse_dmesg_line(line, &flags, &ts_usec, &continuation, &message))
 			continue;
@@ -1024,6 +1068,7 @@ static bool fill_from_dmesg(int fd,
 			subtest += strlen(STARTING_SUBTEST_DMESG);
 			generate_piglit_name(binary, subtest, piglit_name, sizeof(piglit_name));
 			current_test = get_or_create_json_object(tests, piglit_name);
+			clean_regex(&re_ignore);
 		}
 
 		if (current_test != NULL &&
@@ -1041,18 +1086,24 @@ static bool fill_from_dmesg(int fd,
 			dynamic_subtest += strlen(STARTING_DYNAMIC_SUBTEST_DMESG);
 			generate_piglit_name_for_dynamic(piglit_name, dynamic_subtest, dynamic_piglit_name, sizeof(dynamic_piglit_name));
 			current_dynamic_test = get_or_create_json_object(tests, dynamic_piglit_name);
+			clean_regex(&re_ignore);
 		}
 
+		if ((ignore = strstr(message, IGT_ADD_IGNORED_REGEX_DMESG)) != NULL)
+			add_ignored_regex(&re_ignore, ignore + strlen(IGT_ADD_IGNORED_REGEX_DMESG));
+
 		if (settings->piglit_style_dmesg) {
 			if ((flags & 0x07) <= settings->dmesg_warn_level && continuation != 'c' &&
-			    g_regex_match(re, message, 0, NULL)) {
+			    g_regex_match(re, message, 0, NULL) &&
+			    not_ignored(re_ignore, message)) {
 				append_line(&warnings, &warningslen, formatted);
 				if (current_test != NULL)
 					append_line(&dynamic_warnings, &dynamic_warnings_len, formatted);
 			}
 		} else {
 			if ((flags & 0x07) <= settings->dmesg_warn_level && continuation != 'c' &&
-			    !g_regex_match(re, message, 0, NULL)) {
+			    !g_regex_match(re, message, 0, NULL) &&
+			    not_ignored(re_ignore, message)) {
 				append_line(&warnings, &warningslen, formatted);
 				if (current_test != NULL)
 					append_line(&dynamic_warnings, &dynamic_warnings_len, formatted);
@@ -1099,6 +1150,7 @@ static bool fill_from_dmesg(int fd,
 	free(dynamic_dmesg);
 	free(warnings);
 	free(dynamic_warnings);
+	clean_regex(&re_ignore);
 	g_regex_unref(re);
 	fclose(f);
 	return true;
@@ -2124,12 +2176,42 @@ static bool parse_test_directory(int dirfd,
 {
 	int fds[_F_LAST];
 	struct subtest_list subtests = {};
-	bool status = true;
 	int commsparsed;
 
-	if (!open_output_files(dirfd, fds, false)) {
-		fprintf(stderr, "Error opening output files\n");
-		return false;
+	if (!open_output_files_rdonly(dirfd, fds)) {
+		struct stat statbuf;
+		size_t fsizes[_F_LAST]; /* checks for both comm and out/err empty */
+		int fst_err;
+
+		for (int i = 0; i < _F_LAST; ++i) {
+			fsizes[i] = 0;
+			if (fds[i] > 0) {
+				fst_err = fstat(fds[i], &statbuf);
+				if (!fst_err)
+					fsizes[i] = statbuf.st_size;
+				else
+					fprintf(stderr, "results: %s: stats failed: %s\n",
+						get_out_filename(i), strerror(fst_err));
+			} else {
+				fprintf(stderr, "results: %s: open failed: %s\n",
+					get_out_filename(i), strerror(-fds[i]));
+			}
+		}
+
+		/* out/err/dmesg.txt should exist, otherwise there was crash */
+		if (fds[_F_OUT] < 0 || fds[_F_ERR] < 0 || fds[_F_DMESG] < 0) {
+			fprintf(stderr, "results: missing output file(s)\n");
+			close_outputs(fds);
+
+			return false;
+		}
+
+		if (!fsizes[_F_SOCKET] && !fsizes[_F_OUT] && !fsizes[_F_ERR]) {
+			if (!fsizes[_F_DMESG])
+				fprintf(stderr, "results: empty output files\n");
+			else
+				fprintf(stderr, "results: comms/err/out empty, using only dmesg\n");
+		}
 	}
 
 	/*
@@ -2139,8 +2221,6 @@ static bool parse_test_directory(int dirfd,
 	commsparsed = fill_from_comms(fds[_F_SOCKET], entry, &subtests, results);
 	if (commsparsed == COMMSPARSE_ERROR) {
 		fprintf(stderr, "Error parsing output files (comms)\n");
-		status = false;
-		goto parse_output_end;
 	}
 
 	if (commsparsed == COMMSPARSE_EMPTY) {
@@ -2148,20 +2228,17 @@ static bool parse_test_directory(int dirfd,
 		 * fill_from_journal fills the subtests struct and
 		 * adds timeout results where applicable.
 		 */
-		fill_from_journal(fds[_F_JOURNAL], entry, &subtests, results);
+		if (fds[_F_JOURNAL] > 0)
+			fill_from_journal(fds[_F_JOURNAL], entry, &subtests, results);
 
 		if (!fill_from_output(fds[_F_OUT], entry->binary, "out", &subtests, results->tests) ||
 		    !fill_from_output(fds[_F_ERR], entry->binary, "err", &subtests, results->tests)) {
 			fprintf(stderr, "Error parsing output files (out.txt, err.txt)\n");
-			status = false;
-			goto parse_output_end;
 		}
 	}
 
 	if (!fill_from_dmesg(fds[_F_DMESG], settings, entry->binary, &subtests, results->tests)) {
 		fprintf(stderr, "Error parsing output files (dmesg.txt)\n");
-		status = false;
-		goto parse_output_end;
 	}
 
 	override_results(entry->binary, &subtests, results->tests);
@@ -2169,11 +2246,10 @@ static bool parse_test_directory(int dirfd,
 
 	add_to_totals(entry->binary, &subtests, results);
 
- parse_output_end:
 	close_outputs(fds);
 	free_subtests(&subtests);
 
-	return status;
+	return true;
 }
 
 static void try_add_notrun_results(const struct job_list_entry *entry,
@@ -2229,7 +2305,7 @@ struct json_object *generate_results_json(int dirfd)
 {
 	struct settings settings;
 	struct job_list job_list;
-	struct json_object *obj, *elapsed;
+	struct json_object *obj, *elapsed, *arr;
 	struct results results;
 	int testdirfd, fd;
 	size_t i;
@@ -2267,6 +2343,11 @@ struct json_object *generate_results_json(int dirfd)
 		close(fd);
 	}
 
+	arr = json_object_new_array();
+	for (i = 0; i < settings.cmdline.argc; i++)
+		json_object_array_add(arr, json_object_new_string(settings.cmdline.argv[i]));
+	json_object_object_add(obj, "cmdline", arr);
+
 	elapsed = json_object_new_object();
 	json_object_object_add(elapsed, "__type__", json_object_new_string("TimeAttribute"));
 	if ((fd = openat(dirfd, "starttime.txt", O_RDONLY)) >= 0) {
@@ -2302,14 +2383,21 @@ struct json_object *generate_results_json(int dirfd)
 		char name[16];
 
 		snprintf(name, 16, "%zd", i);
+		fprintf(stderr, "results: parsing output: %s/ for test: %s\n",
+			name, job_list.entries[i].binary);
 		if ((testdirfd = openat(dirfd, name, O_DIRECTORY | O_RDONLY)) < 0) {
+			if (settings.log_level >= LOG_LEVEL_NORMAL)
+				fprintf(stderr, "results: no output, setting notrun\n");
+
 			try_add_notrun_results(&job_list.entries[i], &settings, &results);
 			continue;
 		}
 
 		if (!parse_test_directory(testdirfd, &job_list.entries[i], &settings, &results)) {
-			close(testdirfd);
-			return NULL;
+			if (settings.log_level >= LOG_LEVEL_NORMAL)
+				fprintf(stderr, "results: no useful output, setting notrun\n");
+
+			try_add_notrun_results(&job_list.entries[i], &settings, &results);
 		}
 		close(testdirfd);
 	}
