@@ -7,6 +7,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <getopt.h>
+#include <inttypes.h>
 #include <libgen.h>
 #include <limits.h>
 #include <stdio.h>
@@ -40,6 +41,8 @@ enum {
 	OPT_INCLUDE = 't',
 	OPT_EXCLUDE = 'x',
 	OPT_ENVIRONMENT = 'e',
+	OPT_FACTS = 'f',
+	OPT_KMEMLEAK = 'k',
 	OPT_SYNC = 's',
 	OPT_LOG_LEVEL = 'l',
 	OPT_OVERWRITE = 'o',
@@ -230,6 +233,17 @@ static const char *usage_str =
 	"                                   environment variable IGT_PING_HOSTNAME does\n"
 	"                                   not respond to ping.\n"
 	"                         all     - abort for all of the above.\n"
+	"  -f, --facts           Enable facts tracking\n"
+	"  -k, -k<option>, --kmemleak, --kmemleak=<option>\n"
+	"                        Enable kmemleak tracking. Each kmemleak scan\n"
+	"                        can take from 5 to 60 seconds, slowing down\n"
+	"                        the run considerably. The default is to scan\n"
+	"                        only once after the last test. It is also\n"
+	"                        possible to scan after each test. Possible\n"
+	"                        options:\n"
+	"                         once - The default is to run one kmemleak\n"
+	"                                scan after the last test\n"
+	"                         each - Run one kmemleak scan after each test\n"
 	"  -s, --sync            Sync results to disk after every test\n"
 	"  -l {quiet,verbose,dummy}, --log-level {quiet,verbose,dummy}\n"
 	"                        Set the logger verbosity level\n"
@@ -279,11 +293,11 @@ static const char *usage_str =
 	"                        results that are removed from the final results set.\n"
 	"                        Possible options:\n"
 	"                         keep-dynamic-subtests  - Remove subtests that have dynamic\n"
-	"                                                  subtests. (default)\n"
+	"                                                  subtests.\n"
 	"                         keep-dynamic           - Alias for the above\n"
 	"                         keep-subtests          - Remove dynamic subtests,\n"
 	"                                                  leaving just the parent subtest.\n"
-	"                         keep-all               - Don't remove anything\n"
+	"                         keep-all               - Don't remove anything (default)\n"
 	"                         keep-requested         - Remove reported results that are\n"
 	"                                                  not in the requested test set.\n"
 	"                                                  Useful when you have a hand-written\n"
@@ -527,6 +541,17 @@ static void free_hook_strs(struct igt_vec *hook_strs)
 	igt_vec_fini(hook_strs);
 }
 
+static void free_array_deep(void **arr, size_t n)
+{
+	if (!arr)
+		return;
+
+	for (size_t i = 0; i < n; i++)
+		free(arr[i]);
+
+	free(arr);
+}
+
 static bool file_exists_at(int dirfd, const char *filename)
 {
 	return faccessat(dirfd, filename, F_OK, 0) == 0;
@@ -565,7 +590,7 @@ static char *_basename(const char *path)
 	return tmpname;
 }
 
-char *absolute_path(char *path)
+char *absolute_path(const char *path)
 {
 	char *result = NULL;
 	char *base, *dir;
@@ -638,11 +663,13 @@ void clear_settings(struct settings *settings)
 	free(settings->name);
 	free(settings->test_root);
 	free(settings->results_path);
+	free(settings->code_coverage_script);
 
 	free_regexes(&settings->include_regexes);
 	free_regexes(&settings->exclude_regexes);
 	free_env_vars(&settings->env_vars);
 	free_hook_strs(&settings->hook_strs);
+	free_array_deep((void **)settings->cmdline.argv, settings->cmdline.argc);
 
 	init_settings(settings);
 }
@@ -665,6 +692,8 @@ bool parse_options(int argc, char **argv,
 		{"environment", required_argument, NULL, OPT_ENVIRONMENT},
 		{"abort-on-monitored-error", optional_argument, NULL, OPT_ABORT_ON_ERROR},
 		{"disk-usage-limit", required_argument, NULL, OPT_DISK_USAGE_LIMIT},
+		{"facts", no_argument, NULL, OPT_FACTS},
+		{"kmemleak", optional_argument, NULL, OPT_KMEMLEAK},
 		{"sync", no_argument, NULL, OPT_SYNC},
 		{"log-level", required_argument, NULL, OPT_LOG_LEVEL},
 		{"test-list", required_argument, NULL, OPT_TEST_LIST},
@@ -693,8 +722,9 @@ bool parse_options(int argc, char **argv,
 	optind = 1;
 
 	settings->dmesg_warn_level = -1;
+	settings->prune_mode = -1;
 
-	while ((c = getopt_long(argc, argv, "hn:dt:x:e:sl:omb:L",
+	while ((c = getopt_long(argc, argv, "hn:dt:x:e:fk::sl:omb:L",
 				long_options, NULL)) != -1) {
 		switch (c) {
 		case OPT_VERSION:
@@ -735,6 +765,22 @@ bool parse_options(int argc, char **argv,
 				goto error;
 			}
 			break;
+		case OPT_FACTS:
+			settings->facts = true;
+			break;
+		case OPT_KMEMLEAK:
+			/* The default is once */
+			settings->kmemleak = true;
+			settings->kmemleak_each = false;
+			if (optarg) {
+				if (strcmp(optarg, "each") == 0) {
+					settings->kmemleak_each = true;
+				/* "once" is the default. No action needed */
+				} else if (strcmp(optarg, "once") != 0) {
+					usage(stderr, "Invalid kmemleak option");
+					goto error;
+				}
+			}
 		case OPT_SYNC:
 			settings->sync = true;
 			break;
@@ -818,6 +864,9 @@ bool parse_options(int argc, char **argv,
 	if (settings->dmesg_warn_level < 0)
 		settings->dmesg_warn_level = 4; /* KERN_WARN */
 
+	if (settings->prune_mode < 0)
+		settings->prune_mode = PRUNE_KEEP_ALL;
+
 	if (settings->list_all) { /* --list-all doesn't require results path */
 		switch (argc - optind) {
 		case 1:
@@ -864,6 +913,16 @@ bool parse_options(int argc, char **argv,
 		goto error;
 	}
 
+	settings->cmdline.argv = calloc(argc, sizeof(*settings->cmdline.argv));
+	if (!settings->cmdline.argv)
+		goto error;
+
+	settings->cmdline.argc = argc;
+	for (int i = 0; i < argc; i++) {
+		settings->cmdline.argv[i] = strdup(argv[i]);
+		if (!settings->cmdline.argv[i])
+			goto error;
+	}
 
 	return true;
 
@@ -1041,10 +1100,91 @@ static bool serialize_hook_strs(struct settings *settings, int dirfd)
 	return true;
 }
 
+/*
+ * Serialize @s to @f, escaping '\' and '\n'. See unescape_str()
+ */
+static void escape_str(const char *s, FILE *f)
+{
+	while (*s) {
+		size_t len = strcspn(s, "\\\n");
+
+		if (len > 0) {
+			fwrite(s, len, 1, f);
+			s += len;
+		}
+
+		if (*s) {
+			fprintf(f, "\\x%xh", *s);
+			s++;
+		}
+	}
+}
+
+/*
+ * Unescape a '\' and '\n': undo escape_str
+ *
+ * Escape chars using the form '\x<hex>h' so they don't interfere with the line
+ * parser.
+ *
+ * Return the number of chars saved in buf and optionally
+ * the number of chars scanned in @n_src if that is non-nul.
+ */
+static ssize_t unescape_str(char *buf, size_t *n_src)
+{
+	size_t dst_len = 0;
+	char *s = buf;
+
+	while (*s) {
+		char next = *(s + 1);
+
+		if (*s != '\\') {
+			buf[dst_len++] = *s++;
+		} else if (next == 'x') {
+			unsigned long num;
+
+			s += 2;
+
+			num = strtoul(s, &s, 16);
+			/* cover both error due to overflow or invalid char */
+			if (num > UINT8_MAX || *s != 'h')
+				return -EINVAL;
+
+			buf[dst_len++] = num;
+			s++;
+		} else {
+			return -EINVAL;
+		}
+	}
+
+	buf[dst_len] = '\0';
+
+	if (n_src)
+		*n_src = s - buf;
+
+	return dst_len;
+}
+
+#define SERIALIZE_LINE(f, s, name, fmt) fprintf(f, "%s : " fmt "\n", #name, s->name)
+#define SERIALIZE_INT(f, s, name) SERIALIZE_LINE(f, s, name, "%d")
+#define SERIALIZE_UL(f, s, name) SERIALIZE_LINE(f, s, name, "%zu")
+#define SERIALIZE_STR(f, s, name) do {		\
+		if (s->name) {			\
+			fputs(#name " : ", f);	\
+			escape_str(s->name, f);	\
+			fputc('\n', f);		\
+		}				\
+	} while (0)
+#define SERIALIZE_STR_ARRAY(f, s, name, name_len)		\
+	do {							\
+		SERIALIZE_INT(f, s, name_len);			\
+		for (int _i = 0; _i < s->name_len; _i++) {	\
+			fprintf(f, #name "[%d] : ", _i);	\
+			escape_str(s->name[_i], f);		\
+			fputc('\n', f);				\
+		}						\
+	} while (0)
 bool serialize_settings(struct settings *settings)
 {
-#define SERIALIZE_LINE(f, s, name, format) fprintf(f, "%s : " format "\n", #name, s->name)
-
 	FILE *f;
 	int dirfd, covfd;
 	char path[PATH_MAX];
@@ -1086,30 +1226,34 @@ bool serialize_settings(struct settings *settings)
 		return false;
 	}
 
-	SERIALIZE_LINE(f, settings, abort_mask, "%d");
-	SERIALIZE_LINE(f, settings, disk_usage_limit, "%zd");
+	SERIALIZE_INT(f, settings, abort_mask);
+	SERIALIZE_UL(f, settings, disk_usage_limit);
 	if (settings->test_list)
-		SERIALIZE_LINE(f, settings, test_list, "%s");
+		SERIALIZE_STR(f, settings, test_list);
 	if (settings->name)
-		SERIALIZE_LINE(f, settings, name, "%s");
-	SERIALIZE_LINE(f, settings, dry_run, "%d");
-	SERIALIZE_LINE(f, settings, allow_non_root, "%d");
-	SERIALIZE_LINE(f, settings, sync, "%d");
-	SERIALIZE_LINE(f, settings, log_level, "%d");
-	SERIALIZE_LINE(f, settings, overwrite, "%d");
-	SERIALIZE_LINE(f, settings, multiple_mode, "%d");
-	SERIALIZE_LINE(f, settings, inactivity_timeout, "%d");
-	SERIALIZE_LINE(f, settings, per_test_timeout, "%d");
-	SERIALIZE_LINE(f, settings, overall_timeout, "%d");
-	SERIALIZE_LINE(f, settings, use_watchdog, "%d");
-	SERIALIZE_LINE(f, settings, piglit_style_dmesg, "%d");
-	SERIALIZE_LINE(f, settings, dmesg_warn_level, "%d");
-	SERIALIZE_LINE(f, settings, prune_mode, "%d");
-	SERIALIZE_LINE(f, settings, test_root, "%s");
-	SERIALIZE_LINE(f, settings, results_path, "%s");
-	SERIALIZE_LINE(f, settings, enable_code_coverage, "%d");
-	SERIALIZE_LINE(f, settings, cov_results_per_test, "%d");
-	SERIALIZE_LINE(f, settings, code_coverage_script, "%s");
+		SERIALIZE_STR(f, settings, name);
+	SERIALIZE_INT(f, settings, dry_run);
+	SERIALIZE_INT(f, settings, allow_non_root);
+	SERIALIZE_INT(f, settings, facts);
+	SERIALIZE_INT(f, settings, kmemleak);
+	SERIALIZE_INT(f, settings, kmemleak_each);
+	SERIALIZE_INT(f, settings, sync);
+	SERIALIZE_INT(f, settings, log_level);
+	SERIALIZE_INT(f, settings, overwrite);
+	SERIALIZE_INT(f, settings, multiple_mode);
+	SERIALIZE_INT(f, settings, inactivity_timeout);
+	SERIALIZE_INT(f, settings, per_test_timeout);
+	SERIALIZE_INT(f, settings, overall_timeout);
+	SERIALIZE_INT(f, settings, use_watchdog);
+	SERIALIZE_INT(f, settings, piglit_style_dmesg);
+	SERIALIZE_INT(f, settings, dmesg_warn_level);
+	SERIALIZE_INT(f, settings, prune_mode);
+	SERIALIZE_STR(f, settings, test_root);
+	SERIALIZE_STR(f, settings, results_path);
+	SERIALIZE_INT(f, settings, enable_code_coverage);
+	SERIALIZE_INT(f, settings, cov_results_per_test);
+	SERIALIZE_STR(f, settings, code_coverage_script);
+	SERIALIZE_STR_ARRAY(f, settings, cmdline.argv, cmdline.argc);
 
 	if (settings->sync) {
 		fflush(f);
@@ -1137,52 +1281,102 @@ bool serialize_settings(struct settings *settings)
 
 	close(dirfd);
 	return true;
-
+}
+#undef SERIALIZE_STR_ARRAY
+#undef SERIALIZE_STR
+#undef SERIALIZE_UL
+#undef SERIALIZE_INT
 #undef SERIALIZE_LINE
+
+static int parse_int(char **val)
+{
+	return atoi(*val);
 }
 
-bool read_settings_from_file(struct settings *settings, FILE *f)
+static unsigned long parse_ul(char **val)
 {
-#define PARSE_LINE(s, name, val, field, write) \
-	if (!strcmp(name, #field)) {	       \
-		s->field = write;	       \
-		free(name);		       \
-		free(val);		       \
-		name = val = NULL;	       \
-		continue;		       \
+	return strtoul(*val, NULL, 10);
+}
+
+static char *parse_str(char **val)
+{
+	char *ret = *val;
+
+	/*
+	 * Unescaping a string is guaranteed to produce a string that is
+	 * smaller or of the same size. Just modify it in place and leak the
+	 * buffer
+	 */
+	if (unescape_str(ret, NULL) >= 0) {
+		*val = NULL;
+		return ret;
 	}
 
+	return NULL;
+}
+
+#define PARSE_LINE(s, name, val, field, _f)	\
+	if (!strcmp(name, #field)) {		\
+		s->field = _f(val);		\
+		goto cleanup;			\
+	}
+#define PARSE_LINE_ARRAY(s, name, val, field, field_len, _f)		\
+	do {								\
+		int idx;						\
+		if (!strcmp(name, #field_len)) {			\
+			s->field_len = parse_int(val);			\
+			s->field = calloc(s->field_len,			\
+					  sizeof(*s->field));		\
+			goto cleanup;					\
+		} else if (sscanf(name, #field "[%u]", &idx) == 1 &&	\
+			   idx < s->field_len) {			\
+			s->field[idx] = _f(val);			\
+			goto cleanup;					\
+		}							\
+	} while (0)
+#define PARSE_INT(s, name, val, field) PARSE_LINE(s, name, &val, field, parse_int)
+#define PARSE_UL(s, name, val, field)  PARSE_LINE(s, name, &val, field, parse_ul)
+#define PARSE_STR(s, name, val, field) PARSE_LINE(s, name, &val, field, parse_str)
+#define PARSE_STR_ARRAY(s, name, val, field, field_len) \
+	PARSE_LINE_ARRAY(s, name, &val, field, field_len, parse_str)
+bool read_settings_from_file(struct settings *settings, FILE *f)
+{
 	char *name = NULL, *val = NULL;
 
 	settings->dmesg_warn_level = -1;
 
 	while (fscanf(f, "%ms : %m[^\n]", &name, &val) == 2) {
-		int numval = atoi(val);
-		PARSE_LINE(settings, name, val, abort_mask, numval);
-		PARSE_LINE(settings, name, val, disk_usage_limit, strtoul(val, NULL, 10));
-		PARSE_LINE(settings, name, val, test_list, val ? strdup(val) : NULL);
-		PARSE_LINE(settings, name, val, name, val ? strdup(val) : NULL);
-		PARSE_LINE(settings, name, val, dry_run, numval);
-		PARSE_LINE(settings, name, val, allow_non_root, numval);
-		PARSE_LINE(settings, name, val, sync, numval);
-		PARSE_LINE(settings, name, val, log_level, numval);
-		PARSE_LINE(settings, name, val, overwrite, numval);
-		PARSE_LINE(settings, name, val, multiple_mode, numval);
-		PARSE_LINE(settings, name, val, inactivity_timeout, numval);
-		PARSE_LINE(settings, name, val, per_test_timeout, numval);
-		PARSE_LINE(settings, name, val, overall_timeout, numval);
-		PARSE_LINE(settings, name, val, use_watchdog, numval);
-		PARSE_LINE(settings, name, val, piglit_style_dmesg, numval);
-		PARSE_LINE(settings, name, val, dmesg_warn_level, numval);
-		PARSE_LINE(settings, name, val, prune_mode, numval);
-		PARSE_LINE(settings, name, val, test_root, val ? strdup(val) : NULL);
-		PARSE_LINE(settings, name, val, results_path, val ? strdup(val) : NULL);
-		PARSE_LINE(settings, name, val, enable_code_coverage, numval);
-		PARSE_LINE(settings, name, val, cov_results_per_test, numval);
-		PARSE_LINE(settings, name, val, code_coverage_script, val ? strdup(val) : NULL);
+		PARSE_INT(settings, name, val, abort_mask);
+		PARSE_UL(settings, name, val, disk_usage_limit);
+		PARSE_STR(settings, name, val, test_list);
+		PARSE_STR(settings, name, val, name);
+		PARSE_INT(settings, name, val, dry_run);
+		PARSE_INT(settings, name, val, allow_non_root);
+		PARSE_INT(settings, name, val, facts);
+		PARSE_INT(settings, name, val, kmemleak);
+		PARSE_INT(settings, name, val, kmemleak_each);
+		PARSE_INT(settings, name, val, sync);
+		PARSE_INT(settings, name, val, log_level);
+		PARSE_INT(settings, name, val, overwrite);
+		PARSE_INT(settings, name, val, multiple_mode);
+		PARSE_INT(settings, name, val, inactivity_timeout);
+		PARSE_INT(settings, name, val, per_test_timeout);
+		PARSE_INT(settings, name, val, overall_timeout);
+		PARSE_INT(settings, name, val, use_watchdog);
+		PARSE_INT(settings, name, val, piglit_style_dmesg);
+		PARSE_INT(settings, name, val, dmesg_warn_level);
+		PARSE_INT(settings, name, val, prune_mode);
+		PARSE_STR(settings, name, val, test_root);
+		PARSE_STR(settings, name, val, results_path);
+		PARSE_INT(settings, name, val, enable_code_coverage);
+		PARSE_INT(settings, name, val, cov_results_per_test);
+		PARSE_STR(settings, name, val, code_coverage_script);
+		PARSE_STR_ARRAY(settings, name, val, cmdline.argv, cmdline.argc);
 
 		printf("Warning: Unknown field in settings file: %s = %s\n",
 		       name, val);
+
+cleanup:
 		free(name);
 		free(val);
 		name = val = NULL;
@@ -1199,9 +1393,13 @@ bool read_settings_from_file(struct settings *settings, FILE *f)
 	free(val);
 
 	return true;
-
-#undef PARSE_LINE
 }
+#undef PARSE_STR_ARRAY
+#undef PARSE_STR
+#undef PARSE_UL
+#undef PARSE_INT
+#undef PARSE_LINE_ARRAY
+#undef PARSE_LINE
 
 /**
  * read_env_vars_from_file() - load env vars from a file
@@ -1260,7 +1458,7 @@ static bool read_hook_strs_from_file(struct igt_vec *hook_strs, FILE *f)
 
 	buf = malloc(buf_capacity);
 
-	while ((line_length = getline(&line, &line_size, f) != -1)) {
+	while ((line_length = getline(&line, &line_size, f)) != -1) {
 		char *s = line;
 
 		if (buf_len == 0) {
