@@ -1,6 +1,6 @@
 #include "igt.h"
 
-igt_main
+int igt_main()
 {
 	igt_subtest_with_dynamic("dynamic-subtest") {
 		igt_dynamic("failing")
