@@ -1,8 +1,8 @@
 #include "igt.h"
 
-igt_main
+int igt_main()
 {
-	igt_fixture {
+	igt_fixture() {
 		igt_require_f(false, "Skipping from fixture\n");
 	}
 
