@@ -24,7 +24,7 @@
 #include "lib/amdgpu/amd_memory.h"
 #include "lib/amdgpu/amd_command_submission.h"
 #include "lib/amdgpu/amd_deadlock_helpers.h"
-#include "lib/amdgpu/amd_dispatch.h"
+#include "lib/amdgpu/compute_utils/amd_dispatch.h"
 #include "lib/amdgpu/amdgpu_asic_addr.h"
 
 
@@ -32,8 +32,7 @@
 
 #define SHARED_CHILD_DESCRIPTOR 3
 
-#define SHARED_MEM_NAME  "/queue_reset_shm"
-#define TEST_TIMEOUT 100 //100 seconds
+#define TEST_TIMEOUT 20 //100 seconds
 
 enum  process_type {
 	PROCESS_UNKNOWN,
@@ -45,6 +44,7 @@ struct job_struct {
 	unsigned int error;
 	enum amd_ip_block_type ip;
 	unsigned int ring_id;
+	int reset_err_result;
 	/* additional data if necessary */
 };
 
@@ -80,6 +80,52 @@ struct shmbuf {
 
 };
 
+// Local memory structure for communication between child process and worker thread
+struct localbuf {
+	pthread_mutex_t mutex;		// Mutex for condition variable
+	pthread_cond_t cond_var;	// Condition variable to signal context availability
+	int context_ready;			// Flag to indicate if the context is ready
+	amdgpu_context_handle amdgpu_context;// Pointer to amdgpu context
+};
+
+struct thread_params {
+	amdgpu_device_handle device;
+	struct shmbuf *sh_mem;
+	int num_of_tests;
+	struct localbuf local_mem;
+
+};
+
+static struct thread_params*
+allocate_thread_param(amdgpu_device_handle device, struct shmbuf *sh_mem,
+					int num_of_tests)
+{
+	struct thread_params	*param = NULL;
+
+	param = malloc(sizeof(struct thread_params));
+	memset(param, 0, sizeof(struct thread_params));
+
+	param->device = device;
+	param->sh_mem = sh_mem;
+	param->num_of_tests = num_of_tests;
+	pthread_mutex_init(&param->local_mem.mutex, NULL);
+	pthread_cond_init(&param->local_mem.cond_var, NULL);
+	param->local_mem.amdgpu_context = NULL;
+	param->local_mem.context_ready = 0;
+
+	return param;
+}
+
+static void
+free_thread_param(struct thread_params *param)
+{
+	if (param) {
+		pthread_cond_destroy(&param->local_mem.cond_var);
+		pthread_mutex_destroy(&param->local_mem.mutex);
+		free(param);
+	}
+}
+
 static inline
 void set_bit(int nr, uint32_t *addr)
 {
@@ -307,7 +353,8 @@ static void wait_for_complete_iteration(struct shmbuf *sh_mem)
 
 static void set_next_test_to_run(struct shmbuf *sh_mem, unsigned int error,
 		enum amd_ip_block_type ip_good, enum amd_ip_block_type ip_bad,
-		unsigned int ring_id_good, unsigned int ring_id_bad)
+		unsigned int ring_id_good, unsigned int ring_id_bad,
+		const struct reset_err_result *result)
 {
 	char error_str[128];
 	char ip_good_str[64];
@@ -325,8 +372,16 @@ static void set_next_test_to_run(struct shmbuf *sh_mem, unsigned int error,
 	sh_mem->bad_job.error = error;
 	sh_mem->bad_job.ip = ip_bad;
 	sh_mem->bad_job.ring_id = ring_id_bad;
+	if (ip_bad == AMD_IP_GFX)
+		sh_mem->bad_job.reset_err_result = result->gfx_reset_result;
+	else if (ip_bad == AMD_IP_COMPUTE)
+		sh_mem->bad_job.reset_err_result = result->compute_reset_result;
+	else
+		sh_mem->bad_job.reset_err_result = result->sdma_reset_result;
+
 	sh_mem->good_job.error = CMD_STREAM_EXEC_SUCCESS;
 	sh_mem->good_job.ip = ip_good;
+	sh_mem->good_job.reset_err_result = 0;
 	sh_mem->good_job.ring_id = ring_id_good;
 	sh_mem->sub_test_is_skipped = false;
 	sh_mem->sub_test_is_existed = true;
@@ -349,7 +404,7 @@ static void set_next_test_to_skip(struct shmbuf *sh_mem)
 }
 
 static int
-shared_mem_destroy(struct shmbuf *shmp, int shm_fd, bool unmap)
+shared_mem_destroy(struct shmbuf *shmp, int shm_fd, bool unmap, char shm_name[256])
 {
 	int ret = 0;
 
@@ -363,20 +418,20 @@ shared_mem_destroy(struct shmbuf *shmp, int shm_fd, bool unmap)
 	if (shm_fd > 0)
 		close(shm_fd);
 
-	shm_unlink(SHARED_MEM_NAME);
+	shm_unlink(shm_name);
 
 	return ret;
 }
 
 static int
-shared_mem_create(struct shmbuf **ppbuf)
+shared_mem_create(struct shmbuf **ppbuf, char shm_name[256])
 {
 	int shm_fd = -1;
 	struct shmbuf *shmp = NULL;
 	bool unmap = false;
 
 	// Create a shared memory object
-	shm_fd = shm_open(SHARED_MEM_NAME, O_CREAT | O_RDWR, 0666);
+	shm_fd = shm_open(shm_name, O_CREAT | O_RDWR, 0666);
 	if (shm_fd == -1)
 		goto error;
 
@@ -414,7 +469,7 @@ shared_mem_create(struct shmbuf **ppbuf)
 	return shm_fd;
 
 error:
-	shared_mem_destroy(shmp,  shm_fd,  unmap);
+	shared_mem_destroy(shmp,  shm_fd,  unmap, shm_name);
 	return shm_fd;
 }
 
@@ -449,41 +504,10 @@ is_queue_reset_tests_enable(const struct amdgpu_gpu_info *gpu_info, uint32_t ver
 	return enable;
 }
 
-static bool
-is_sub_test_queue_reset_enable(const struct amdgpu_gpu_info *gpu_info,
-		struct asic_id_filter exclude_filter[_MAX_NUM_ASIC_ID_EXCLUDE_FILTER],
-		const struct dynamic_test *it)
-{
-	int i;
-	bool enable = true;
-	int chip_id;
-	char error_str[128];
-	bool is_dispatch;
-
-	for (i = 0; i < _MAX_NUM_ASIC_ID_EXCLUDE_FILTER; i++) {
-		if (gpu_info->family_id == exclude_filter[i].family_id) {
-			chip_id = gpu_info->chip_external_rev - gpu_info->chip_rev;
-			if (chip_id >= exclude_filter[i].chip_id_begin &&
-				chip_id < exclude_filter[i].chip_id_end) {
-				enable = false;
-				is_dispatch_shader_test(it->test, error_str, &is_dispatch);
-				igt_info("PID %d SKIP subtest %s CHIP family (%s) %d chip %d, begin end [%d %d] excluded\n",
-						getpid(), error_str, g_pChip->name,
-						gpu_info->family_id, chip_id,
-						exclude_filter[i].chip_id_begin,
-						exclude_filter[i].chip_id_end);
-				break;
-			}
-		}
-	}
-
-	return enable;
-}
-
 static int
 amdgpu_write_linear(amdgpu_device_handle device, amdgpu_context_handle context_handle,
 		const struct amdgpu_ip_block_version *ip_block,
-		const struct job_struct *job)
+		const struct job_struct *job, struct amdgpu_cs_err_codes *err_codes)
 {
 	const int pm4_dw = 256;
 	struct amdgpu_ring_context *ring_context;
@@ -528,6 +552,8 @@ amdgpu_write_linear(amdgpu_device_handle device, amdgpu_context_handle context_h
 
 	r = amdgpu_test_exec_cs_helper(device, ip_block->type, ring_context,
 			expect_failure);
+	err_codes->err_code_cs_submit = ring_context->err_codes.err_code_cs_submit;
+	err_codes->err_code_wait_for_fence = ring_context->err_codes.err_code_wait_for_fence;
 
 	amdgpu_bo_unmap_and_free(ring_context->bo, ring_context->va_handle,
 			ring_context->bo_mc, ring_context->write_length * sizeof(uint32_t));
@@ -537,8 +563,8 @@ amdgpu_write_linear(amdgpu_device_handle device, amdgpu_context_handle context_h
 }
 
 static int
-run_monitor_child(amdgpu_device_handle device, amdgpu_context_handle *arr_context,
-			   struct shmbuf *sh_mem, int num_of_tests)
+run_monitor_thread(amdgpu_device_handle device, struct shmbuf *sh_mem,
+		int num_of_tests, struct thread_params *param)
 {
 	int ret;
 	int test_counter = 0;
@@ -550,27 +576,38 @@ run_monitor_child(amdgpu_device_handle device, amdgpu_context_handle *arr_contex
 	int64_t cnt = 0;
 	time_t start, end;
 	double elapsed = 0;
+	amdgpu_context_handle local_context = NULL;
 
 	after_reset_state = after_reset_hangs = 0;
 	init_flags = in_process_flags = 0;
 
-	ret = amdgpu_cs_query_reset_state2(arr_context[0], &init_flags);
-	if (init_flags & AMDGPU_CTX_QUERY2_FLAGS_RESET_IN_PROGRESS)
-		igt_assert_eq(init_flags & AMDGPU_CTX_QUERY2_FLAGS_RESET_IN_PROGRESS, 0);
-
 	while (num_of_tests > 0) {
 		sync_point_enter(sh_mem);
+		if (is_subtest_skipped(sh_mem)) {
+			sync_point_exit(sh_mem);
+			num_of_tests--;
+			test_counter++;
+			continue;
+		}
 		state_machine = 0;
 		error_code = 0;
 		flags = 0;
 		set_reset_state(sh_mem, false, ALL_RESET_BITS);
+
+		pthread_mutex_lock(&param->local_mem.mutex);
+		while (!param->local_mem.context_ready)
+			pthread_cond_wait(&param->local_mem.cond_var, &param->local_mem.mutex);
+
+		local_context = param->local_mem.amdgpu_context;
+		pthread_mutex_unlock(&param->local_mem.mutex);
+
 		time(&start);
 		while (1) {
 			if (is_subtest_skipped(sh_mem))
 				break;
 
 			if (state_machine == 0) {
-				amdgpu_cs_query_reset_state2(arr_context[test_counter], &init_flags);
+				amdgpu_cs_query_reset_state2(local_context, &init_flags);
 
 				if (init_flags & AMDGPU_CTX_QUERY2_FLAGS_RESET)
 					state_machine = 1;
@@ -579,9 +616,9 @@ run_monitor_child(amdgpu_device_handle device, amdgpu_context_handle *arr_contex
 					state_machine = 2;
 
 			} else if (state_machine == 1) {
-				amdgpu_cs_query_reset_state(arr_context[test_counter],
+				amdgpu_cs_query_reset_state(local_context,
 						&after_reset_state, &after_reset_hangs);
-				amdgpu_cs_query_reset_state2(arr_context[test_counter],
+				amdgpu_cs_query_reset_state2(local_context,
 						&in_process_flags);
 
 				//TODO refactor this block !
@@ -602,9 +639,9 @@ run_monitor_child(amdgpu_device_handle device, amdgpu_context_handle *arr_contex
 					}
 				}
 			} else if (state_machine == 2) {
-				amdgpu_cs_query_reset_state(arr_context[test_counter],
+				amdgpu_cs_query_reset_state(local_context,
 						&after_reset_state, &after_reset_hangs);
-				amdgpu_cs_query_reset_state2(arr_context[test_counter],
+				amdgpu_cs_query_reset_state2(local_context,
 						&in_process_flags);
 				/* here we should start timer and wait for some time until
 				 * the flag AMDGPU_CTX_QUERY2_FLAGS_RESET disappear
@@ -632,13 +669,22 @@ run_monitor_child(amdgpu_device_handle device, amdgpu_context_handle *arr_contex
 	return ret;
 }
 
+static void *
+worker_thread(void *arg)
+{
+	struct thread_params *param = (struct thread_params *)arg;
+
+	run_monitor_thread(param->device, param->sh_mem, param->num_of_tests, param);
+
+	return NULL;
+}
 
 
 static int
-run_test_child(amdgpu_device_handle device, amdgpu_context_handle *arr_context,
-				struct shmbuf *sh_mem, int num_of_tests, uint32_t version)
+run_test_child(amdgpu_device_handle device, struct shmbuf *sh_mem,
+		int num_of_tests, uint32_t version, struct thread_params *param)
 {
-	int ret;
+	int ret, r;
 	bool bool_ret;
 	int test_counter = 0;
 	char error_str[128];
@@ -647,6 +693,8 @@ run_test_child(amdgpu_device_handle device, amdgpu_context_handle *arr_context,
 
 	struct job_struct job;
 	const struct amdgpu_ip_block_version *ip_block_test = NULL;
+	struct amdgpu_cs_err_codes err_codes;
+	amdgpu_context_handle local_context = NULL;
 
 	while (num_of_tests > 0) {
 		sync_point_enter(sh_mem);
@@ -661,24 +709,50 @@ run_test_child(amdgpu_device_handle device, amdgpu_context_handle *arr_context,
 		bool_ret = is_dispatch_shader_test(job.error,  error_str, &is_dispatch);
 		igt_assert_eq(bool_ret, 1);
 		ip_block_test = get_ip_block(device, job.ip);
+		err_codes.err_code_cs_submit = 0;
+		err_codes.err_code_wait_for_fence = 0;
+
+		r = amdgpu_cs_ctx_create(device, &local_context);
+		igt_assert_eq(r, 0);
+		pthread_mutex_lock(&param->local_mem.mutex);
+		param->local_mem.amdgpu_context = local_context;
+		param->local_mem.context_ready = 1; // Mark the context as ready
+		pthread_cond_signal(&param->local_mem.cond_var); // Notify the worker thread
+		pthread_mutex_unlock(&param->local_mem.mutex);
+
 		if (is_dispatch) {
-			ret = amdgpu_memcpy_dispatch_test(device, job.ip, job.ring_id, version,
-					job.error);
+			ret = amdgpu_memcpy_dispatch_test(device, local_context, job.ip, job.ring_id, 0,version,
+					job.error, &err_codes, false);
 		} else {
-			ret = amdgpu_write_linear(device, arr_context[test_counter],
-					ip_block_test, &job);
+			ret = amdgpu_write_linear(device, local_context,
+					ip_block_test, &job, &err_codes);
 		}
 
 		num_of_tests--;
 		set_test_state(sh_mem, true, ret, ERROR_CODE_SET_BIT);
+
 		while (1) {
 			/*we may have GPU reset vs queue reset */
 			if (get_reset_state(sh_mem, &reset_flags))
 				break;
 			sleep(1);
 		}
+
+
 		sync_point_exit(sh_mem);
+
+		pthread_mutex_lock(&param->local_mem.mutex);
+		param->local_mem.context_ready = 0;
+		param->local_mem.amdgpu_context = NULL;
+		pthread_mutex_unlock(&param->local_mem.mutex);
+
+		amdgpu_cs_ctx_free(local_context);
 		test_counter++;
+		igt_assert_eq(err_codes.err_code_cs_submit, 0);
+		//igt_assert_eq(err_codes.err_code_wait_for_fence, job.reset_err_result);
+		//igt_info("====Test %s ip %s wait_for_fence %d==============\n", error_str,
+		//		job.ip == AMD_IP_GFX ? "AMD_IP_GFX" : "AMD_IP_COMPUTE",
+		//		err_codes.err_code_wait_for_fence);
 	}
 	return ret;
 }
@@ -698,10 +772,7 @@ run_background(amdgpu_device_handle device, struct shmbuf *sh_mem,
 	const struct amdgpu_ip_block_version *ip_block_test = NULL;
 	int error_code;
 	unsigned int flags;
-
-	r = amdgpu_cs_ctx_create(device, &context_handle);
-	igt_assert_eq(r, 0);
-
+	struct amdgpu_cs_err_codes err_codes;
 
 	while (num_of_tests > 0) {
 		sync_point_enter(sh_mem);
@@ -713,8 +784,10 @@ run_background(amdgpu_device_handle device, struct shmbuf *sh_mem,
 		read_next_job(sh_mem, &job, true);
 		ip_block_test = get_ip_block(device, job.ip);
 		is_dispatch_shader_test(job.error,  error_str, &is_dispatch);
+		r = amdgpu_cs_ctx_create(device, &context_handle);
+		igt_assert_eq(r, 0);
 		while (1) {
-			r = amdgpu_write_linear(device, context_handle,  ip_block_test, &job);
+			r = amdgpu_write_linear(device, context_handle,  ip_block_test, &job, &err_codes);
 
 			if (counter > NUM_ITERATION && counter % NUM_ITERATION == 0)
 				igt_debug("+++BACKGROUND++ amdgpu_write_linear for %s ring_id %d ret %d counter %d\n",
@@ -727,46 +800,40 @@ run_background(amdgpu_device_handle device, struct shmbuf *sh_mem,
 			}
 			if (r != -ECANCELED && r != -ETIME && r != -ENODATA)
 				igt_assert_eq(r, 0);
-			/*
-			 * TODO we have issue during gpu reset the return code assert we put after we check the
-			 * test is completed otherwise the job is failed due to
-			 * amdgpu_job_run Skip job if VRAM is lost
-			 * if (job->generation != amdgpu_vm_generation(adev, job->vm)
-			 */
+			//igt_assert_eq(err_codes.err_code_wait_for_fence, job.reset_err_result);
 			counter++;
 
 		}
+		r = amdgpu_cs_ctx_free(context_handle);
 		sync_point_exit(sh_mem);
 		num_of_tests--;
 	}
-	r = amdgpu_cs_ctx_free(context_handle);
 	return r;
 }
 
 
-
-
 static int
-run_all(amdgpu_device_handle device, amdgpu_context_handle *arr_context_handle,
-		enum process_type process, struct shmbuf *sh_mem,  int num_of_tests,
-		uint32_t version, pid_t *monitor_child, pid_t *test_child)
+run_all(amdgpu_device_handle device, enum process_type process,
+		struct shmbuf *sh_mem,  int num_of_tests, uint32_t version,
+		pid_t *monitor_child, pid_t *test_child)
 {
+	struct thread_params *param = NULL;
+	pthread_t worker;
+
 	if (process == PROCESS_TEST) {
-		*monitor_child = fork();
-		if (*monitor_child == -1) {
-			igt_fail(IGT_EXIT_FAILURE);
-		} else if (*monitor_child == 0) {
-			*monitor_child = getppid();
-			run_monitor_child(device, arr_context_handle, sh_mem, num_of_tests);
-				igt_success();
-				igt_exit();
-		}
 		*test_child = fork();
 		if (*test_child == -1) {
 			igt_fail(IGT_EXIT_FAILURE);
 		} else if (*test_child == 0) {
 			*test_child = getppid();
-			run_test_child(device, arr_context_handle, sh_mem, num_of_tests, version);
+			param = allocate_thread_param(device, sh_mem, num_of_tests);
+			if (pthread_create(&worker, NULL, worker_thread, param) != 0) {
+				igt_info("********Failed to create worker thread");
+				igt_fail(IGT_EXIT_FAILURE);
+			}
+			run_test_child(device, sh_mem, num_of_tests, version, param);
+			pthread_join(worker, NULL);
+			free_thread_param(param);
 			igt_success();
 			igt_exit();
 
@@ -877,6 +944,29 @@ is_run_subtest_parameter_found(int argc, char **argv)
 	return ret;
 }
 
+#define ONDEVICE	"--device"
+static int
+is_run_device_parameter_found(int argc, char **argv)
+{
+	int i;
+	int res = 0;
+	char *p = NULL;
+
+	for (i = 1; i < argc; i++) {
+		if (strcmp(ONDEVICE, argv[i]) == 0) {
+			/* Get the sum for a specific device as a unique identifier */
+			p = argv[i+1];
+			while (*p) {
+				res += *p;
+				p++;
+			}
+			break;
+		}
+	}
+
+	return res;
+}
+
 
 static bool
 add_background_parameter(int *pargc, char **argv)
@@ -924,35 +1014,6 @@ launch_background_process(int argc, char **argv, char *path, pid_t *ppid, int sh
 	return status;
 }
 
-static void
-create_contexts(amdgpu_device_handle device, amdgpu_context_handle **pp_contexts,
-		int num_of_contexts)
-{
-	amdgpu_context_handle *p_contexts = NULL;
-	int i, r;
-
-	p_contexts = (amdgpu_context_handle *)malloc(sizeof(amdgpu_context_handle)
-			*num_of_contexts);
-
-	for (i = 0; i < num_of_contexts; i++) {
-		r = amdgpu_cs_ctx_create(device, &p_contexts[i]);
-		igt_assert_eq(r, 0);
-	}
-	*pp_contexts = p_contexts;
-
-}
-static void
-free_contexts(amdgpu_device_handle device, amdgpu_context_handle *p_contexts,
-		int num_of_contexts)
-{
-	int i;
-
-	if (p_contexts) {
-		for (i = 0; i < num_of_contexts; i++)
-			amdgpu_cs_ctx_free(p_contexts[i]);
-	}
-}
-
 static bool
 adjust_begin_index(unsigned int *ring_begin, unsigned int available_good_rings)
 {
@@ -1022,7 +1083,24 @@ reset_rings_numbers(unsigned int *ring_id_good, unsigned int *ring_id_bad,
 	*ring_id_job_bad = 0;
 }
 
-igt_main
+static int
+get_num_of_tests(struct dynamic_test *arr_err, enum amd_ip_block_type *ip_tests, int num_ip)
+{
+	int i, cnt=0;
+
+	for (i = 0; i < num_ip; i++) {
+		for (struct dynamic_test *it = arr_err; it->name; it++) {
+			if(*ip_tests == AMD_IP_DMA && (!it->support_sdma))
+				continue;
+			cnt++;
+		}
+		ip_tests++;
+	}
+
+	return cnt;
+}
+
+int igt_main()
 {
 	char cmdline[2048];
 	int argc = 0;
@@ -1031,38 +1109,36 @@ igt_main
 	enum  process_type process = PROCESS_UNKNOWN;
 	pid_t pid_background = 0;
 	pid_t monitor_child = 0, test_child = 0;
-	int testExitMethod, monitorExitMethod, backgrounExitMethod;
+	int testExitMethod, backgrounExitMethod;
 	posix_spawn_file_actions_t action;
 	amdgpu_device_handle device;
 	struct amdgpu_gpu_info gpu_info = {0};
-	struct drm_amdgpu_info_hw_ip info[2] = {0};
 	int fd = -1;
 	int fd_shm = -1;
 	struct shmbuf *sh_mem = NULL;
 
 	int r;
+	char shm_name[256] = {0};
 	bool arr_cap[AMD_IP_MAX] = {0};
+	uint32_t reset;
 	unsigned int ring_id_good;
 	unsigned int ring_id_bad;
 	unsigned int ring_id_job_good;
 	unsigned int ring_id_job_bad;
+	int expect_error;
+	struct pci_addr pci ;
 
-	enum amd_ip_block_type ip_tests[2] = {AMD_IP_COMPUTE/*keep first*/, AMD_IP_GFX};
+	enum amd_ip_block_type ip_tests[3] = {AMD_IP_COMPUTE/*keep first*/, AMD_IP_GFX, AMD_IP_DMA};
 	enum amd_ip_block_type ip_background = AMD_IP_COMPUTE;
+	struct drm_amdgpu_info_hw_ip info[ARRAY_SIZE(ip_tests)] = {0};
 
-	amdgpu_context_handle *arr_context_handle = NULL;
-
-	/* TODO remove this , it is used only to create array of contexts
-	 * which are shared between child processes ( test/monitor/main and
-	 *  separate for background
-	 */
 	struct dynamic_test arr_err[] = {
 			{CMD_STREAM_EXEC_INVALID_PACKET_LENGTH, "CMD_STREAM_EXEC_INVALID_PACKET_LENGTH",
 				"Stressful-and-multiple-cs-of-bad and good length-operations-using-multiple-processes",
-				{ {FAMILY_UNKNOWN, 0x1, 0x10 }, {FAMILY_AI, 0x32, 0x3C }, {FAMILY_AI, 0x3C, 0xFF } } },
+				{}, {-ECANCELED, -ECANCELED, -ECANCELED }, true},
 			{CMD_STREAM_EXEC_INVALID_OPCODE, "CMD_STREAM_EXEC_INVALID_OPCODE",
 				"Stressful-and-multiple-cs-of-bad and good opcode-operations-using-multiple-processes",
-				{ {FAMILY_UNKNOWN, -1, -1 }, {FAMILY_UNKNOWN, -1, -1 }, {FAMILY_UNKNOWN, -1, -1 } } },
+				{}, {-ECANCELED, -ECANCELED, -ECANCELED }, true },
 			//TODO  not job timeout, debug why for n31.
 			//{CMD_STREAM_TRANS_BAD_MEM_ADDRESS_BY_SYNC,"CMD_STREAM_TRANS_BAD_MEM_ADDRESS_BY_SYNC",
 			//	"Stressful-and-multiple-cs-of-bad and good mem-sync-operations-using-multiple-processes"},
@@ -1071,22 +1147,22 @@ igt_main
 			//	"Stressful-and-multiple-cs-of-bad and good reg-operations-using-multiple-processes"},
 			{BACKEND_SE_GC_SHADER_INVALID_PROGRAM_ADDR, "BACKEND_SE_GC_SHADER_INVALID_PROGRAM_ADDR",
 				"Stressful-and-multiple-cs-of-bad and good shader-operations-using-multiple-processes",
-				{ {FAMILY_UNKNOWN, 0x1, 0x10 }, {FAMILY_AI, 0x32, 0x3C }, {FAMILY_AI, 0x3C, 0xFF } } },
+				{}, {-ENODATA, -ENODATA, -ENODATA }, false },
 			//TODO  KGQ cannot recover by queue reset, it maybe need a fw bugfix on naiv31
 			//{BACKEND_SE_GC_SHADER_INVALID_PROGRAM_SETTING,"BACKEND_SE_GC_SHADER_INVALID_PROGRAM_SETTING",
 			//	"Stressful-and-multiple-cs-of-bad and good shader-operations-using-multiple-processes"},
 			{BACKEND_SE_GC_SHADER_INVALID_USER_DATA, "BACKEND_SE_GC_SHADER_INVALID_USER_DATA",
 				"Stressful-and-multiple-cs-of-bad and good shader-operations-using-multiple-processes",
-				{ {FAMILY_UNKNOWN, -1, -1 }, {FAMILY_AI, 0x32, 0x3C }, {FAMILY_AI, 0x3C, 0xFF } } },
+				{}, {-ENODATA, -ENODATA, -ENODATA }, false },
 			{BACKEND_SE_GC_SHADER_INVALID_SHADER, "BACKEND_SE_GC_SHADER_INVALID_SHADER",
 				"Stressful-and-multiple-cs-of-bad and good shader-operations-using-multiple-processes",
-				{ {FAMILY_UNKNOWN, 0x1, 0x10 }, {FAMILY_AI, 0x32, 0x3C }, {FAMILY_AI, 0x3C, 0xFF } } },
+				{}, {-ENODATA, -ENODATA, -ENODATA }, false },
 			{}
 	};
 
 	int const_num_of_tests;
 
-	igt_fixture {
+	igt_fixture() {
 		uint32_t major, minor;
 		int err;
 
@@ -1098,8 +1174,10 @@ igt_main
 		if (is_run_subtest_parameter_found(argc, argv))
 			const_num_of_tests = 1;
 		else
-			const_num_of_tests = (sizeof(arr_err)/sizeof(struct dynamic_test) - 1) * ARRAY_SIZE(ip_tests);
+			const_num_of_tests = get_num_of_tests(&arr_err[0], &ip_tests[0], ARRAY_SIZE(ip_tests));
 
+		r = is_run_device_parameter_found(argc, argv);
+		snprintf(shm_name, sizeof(shm_name), "/queue_reset_shm_%d", r);
 		fd = drm_open_driver(DRIVER_AMDGPU);
 
 		err = amdgpu_device_initialize(fd, &major, &minor, &device);
@@ -1121,35 +1199,52 @@ igt_main
 
 		if (!is_background_parameter_found(argc, argv)) {
 			add_background_parameter(&argc, argv);
-			fd_shm = shared_mem_create(&sh_mem);
+			fd_shm = shared_mem_create(&sh_mem, shm_name);
 			igt_require(fd_shm != -1);
 			launch_background_process(argc, argv, path, &pid_background, fd_shm);
 			process = PROCESS_TEST;
+			igt_skip_on(get_pci_addr_from_fd(fd, &pci));
+			igt_info("PCI Address: domain %04x, bus %02x, device %02x, function %02x\n",
+							pci.domain, pci.bus, pci.device, pci.function);
 		} else {
 			process = PROCESS_BACKGROUND;
 		}
-		if (process == PROCESS_TEST)
-			create_contexts(device, &arr_context_handle, const_num_of_tests);
-		else if (process == PROCESS_BACKGROUND)
+		if (process == PROCESS_BACKGROUND)
 			fd_shm = shared_mem_open(&sh_mem);
+
 		igt_require(fd_shm != -1);
 		igt_require(sh_mem != NULL);
 
-		run_all(device, arr_context_handle,
-			process, sh_mem, const_num_of_tests, info[0].hw_ip_version_major,
-			&monitor_child, &test_child);
+		run_all(device, process, sh_mem, const_num_of_tests,
+				info[0].hw_ip_version_major, &monitor_child, &test_child);
 	}
+	//print expect error table for each test
+	if (!argc) {// --list-subtests arg is 0
+		for (int i = 0; i < ARRAY_SIZE(ip_tests); i++)
+			for (struct dynamic_test *it = &arr_err[0]; it->name; it++) {
+				expect_error = ip_tests[i] == AMD_IP_COMPUTE ?
+						it->result.compute_reset_result : it->result.gfx_reset_result;
+				igt_info("test ip: %s,  test: %s,  expected error:%d \n",
+					ip_tests[i] == AMD_IP_COMPUTE ? "COMPUTE" : "GFX", it->name, expect_error);
+			}
+	}
+
 	for (int i = 0; i < ARRAY_SIZE(ip_tests); i++) {
 		reset_rings_numbers(&ring_id_good, &ring_id_bad, &ring_id_job_good, &ring_id_job_bad);
 		for (struct dynamic_test *it = &arr_err[0]; it->name; it++) {
+			if(ip_tests[i] == AMD_IP_DMA && (!it->support_sdma))
+				continue;
 			igt_describe("Stressful-and-multiple-cs-of-bad-and-good-length-operations-using-multiple-processes");
-			igt_subtest_with_dynamic_f("amdgpu-%s-%s", ip_tests[i] == AMD_IP_COMPUTE ? "COMPUTE":"GFX", it->name) {
-				if (arr_cap[ip_tests[i]] && is_sub_test_queue_reset_enable(&gpu_info, it->exclude_filter, it) &&
+			igt_subtest_with_dynamic_f("amdgpu-%s-%s", ip_tests[i] == AMD_IP_COMPUTE ? "COMPUTE":
+					ip_tests[i] == AMD_IP_GFX ? "GFX" : "SDMA", it->name) {
+				reset = AMDGPU_RESET_TYPE_PER_QUEUE;
+				if (arr_cap[ip_tests[i]] && is_reset_enable(ip_tests[i], reset, &pci) &&
 						get_next_rings(&ring_id_good, &ring_id_bad, info[0].available_rings,
 						info[i].available_rings, ip_background != ip_tests[i], &ring_id_job_good, &ring_id_job_bad)) {
 					igt_dynamic_f("amdgpu-%s-ring-good-%d-bad-%d-%s", it->name, ring_id_job_good, ring_id_job_bad,
-							ip_tests[i] == AMD_IP_COMPUTE ? "COMPUTE":"GFX")
-					set_next_test_to_run(sh_mem, it->test, ip_background, ip_tests[i], ring_id_job_good, ring_id_job_bad);
+							ip_tests[i] == AMD_IP_COMPUTE ? "COMPUTE":
+							ip_tests[i] == AMD_IP_GFX ? "GFX" : "SDMA")
+					set_next_test_to_run(sh_mem, it->test, ip_background, ip_tests[i], ring_id_job_good, ring_id_job_bad, &it->result);
 				} else {
 					set_next_test_to_skip(sh_mem);
 				}
@@ -1160,16 +1255,13 @@ igt_main
 	if (sh_mem && (!sh_mem->sub_test_is_existed))
 		set_next_test_to_skip(sh_mem);
 
-	igt_fixture {
-		if (process == PROCESS_TEST) {
-			waitpid(monitor_child, &monitorExitMethod, 0);
+	igt_fixture() {
+		if (process == PROCESS_TEST)
 			waitpid(test_child, &testExitMethod, 0);
-		}
 		waitpid(pid_background, &backgrounExitMethod, 0);
-		free_contexts(device, arr_context_handle, const_num_of_tests);
 		amdgpu_device_deinitialize(device);
 		drm_close_driver(fd);
-		shared_mem_destroy(sh_mem, fd_shm, true);
+		shared_mem_destroy(sh_mem, fd_shm, true, shm_name);
 		posix_spawn_file_actions_destroy(&action);
 
 		free_command_line(argc, argv, path);
