@@ -460,19 +460,19 @@ static void test_hpd_storm_disable(chamelium_data_t *data,
 }
 
 IGT_TEST_DESCRIPTION("Testing HPD with a Chamelium board");
-igt_main
+int igt_main()
 {
 	chamelium_data_t data;
 	struct chamelium_port *port;
 	int p;
 
-	igt_fixture {
+	igt_fixture() {
 		chamelium_init_test(&data);
 	}
 
 	igt_describe("DisplayPort tests");
-	igt_subtest_group {
-		igt_fixture {
+	igt_subtest_group() {
+		igt_fixture() {
 			chamelium_require_connector_present(
 				data.ports, DRM_MODE_CONNECTOR_DisplayPort,
 				data.port_count, 1);
@@ -523,8 +523,8 @@ igt_main
 	}
 
 	igt_describe("HDMI tests");
-	igt_subtest_group {
-		igt_fixture {
+	igt_subtest_group() {
+		igt_fixture() {
 			chamelium_require_connector_present(
 				data.ports, DRM_MODE_CONNECTOR_HDMIA,
 				data.port_count, 1);
@@ -575,8 +575,8 @@ igt_main
 	}
 
 	igt_describe("VGA tests");
-	igt_subtest_group {
-		igt_fixture {
+	igt_subtest_group() {
+		igt_fixture() {
 			chamelium_require_connector_present(
 				data.ports, DRM_MODE_CONNECTOR_VGA,
 				data.port_count, 1);
@@ -616,8 +616,8 @@ igt_main
 	}
 
 	igt_describe("Tests that operate on all connectors");
-	igt_subtest_group {
-		igt_fixture {
+	igt_subtest_group() {
+		igt_fixture() {
 			igt_require(data.port_count);
 		}
 
@@ -637,7 +637,7 @@ igt_main
 	connector_subtest("vga-hpd-for-each-pipe", VGA)
 		test_hotplug_for_each_pipe(&data, port);
 
-	igt_fixture {
+	igt_fixture() {
 		igt_display_fini(&data.display);
 		drm_close_driver(data.drm_fd);
 	}
