@@ -1029,6 +1029,9 @@ test_saturated_hostile_all(int i915, const intel_ctx_t *base_ctx,
 	igt_spin_busywait_until_started(spin);
 	intel_ctx_destroy(i915, ctx);
 
+	/* Give the kernel some time to terminate the context */
+	usleep(reset_timeout_ms * 1000);
+
 	/* Hostile request requires a GPU reset to terminate */
 	igt_assert_eq(wait_for_status(spin->out_fence, reset_timeout_ms), -EIO);
 
@@ -1173,6 +1176,9 @@ static void __smoker(int i915, const intel_ctx_cfg_t *cfg,
 	drm_close_driver(fd);
 	flush_delayed_fput(i915);
 
+	/* Give the kernel some time to terminate the context */
+	usleep(reset_timeout_ms * 1000);
+
 	igt_spin_end(spin);
 
 	igt_assert_eq(wait_for_status(spin->out_fence, timeout), expected);
@@ -1325,7 +1331,7 @@ static void exit_handler(int sig)
 	enable_hangcheck(i915);
 }
 
-igt_main
+int igt_main()
 {
 	const intel_ctx_cfg_t empty_cfg = {};
 	struct {
@@ -1345,7 +1351,7 @@ igt_main
 	};
 	const intel_ctx_t *ctx;
 
-	igt_fixture {
+	igt_fixture() {
 		i915 = drm_open_driver(DRIVER_INTEL);
 		igt_require_gem(i915);
 
@@ -1394,8 +1400,8 @@ igt_main
 	igt_subtest("heartbeat-hostile")
 		test_noheartbeat_close(i915, IGT_SPIN_NO_PREEMPTION);
 
-	igt_subtest_group {
-		igt_fixture
+	igt_subtest_group() {
+		igt_fixture()
 			gem_require_contexts(i915);
 
 		for (test = tests; test->name; test++) {
@@ -1414,10 +1420,10 @@ igt_main
 
 	/* New way of selecting engines. */
 
-	igt_subtest_group {
+	igt_subtest_group() {
 		const struct intel_execution_engine2 *e;
 
-		igt_fixture
+		igt_fixture()
 			gem_require_contexts(i915);
 
 		for (test = tests; test->name; test++) {
@@ -1450,8 +1456,8 @@ igt_main
 			many_contexts(i915, &ctx->cfg);
 	}
 
-	igt_subtest_group {
-		igt_fixture {
+	igt_subtest_group() {
+		igt_fixture() {
 			gem_require_contexts(i915);
 			intel_allocator_multiprocess_start();
 		}
@@ -1459,13 +1465,13 @@ igt_main
 		igt_subtest("smoketest")
 			smoketest(i915, &ctx->cfg);
 
-		igt_fixture {
+		igt_fixture() {
 			intel_allocator_multiprocess_stop();
 		}
 
 	}
 
-	igt_fixture {
+	igt_fixture() {
 		drm_close_driver(i915);
 	}
 }
