@@ -248,7 +248,7 @@ static void test_batch_first(int fd)
 	memset(&execbuf, 0, sizeof(execbuf));
 	execbuf.buffers_ptr = to_user_pointer(obj);
 	execbuf.buffer_count = ARRAY_SIZE(obj);
-	if (gen > 3 && gen < 6)
+	if (gem_store_dword_needs_secure(fd))
 		execbuf.flags |= I915_EXEC_SECURE;
 
 	/* Normal mode */
@@ -449,9 +449,9 @@ uint32_t batch[2] = {MI_BATCH_BUFFER_END};
 uint32_t handle, devid;
 int fd;
 
-igt_main
+int igt_main()
 {
-	igt_fixture {
+	igt_fixture() {
 		fd = drm_open_driver(DRIVER_INTEL);
 		igt_require_gem(fd);
 
@@ -720,7 +720,7 @@ igt_main
 
 #undef RUN_FAIL
 
-	igt_fixture {
+	igt_fixture() {
 		gem_close(fd, handle);
 
 		drm_close_driver(fd);
