@@ -903,7 +903,7 @@ static void active(int fd, const intel_ctx_t *ctx, unsigned engine)
 	memset(&execbuf, 0, sizeof(execbuf));
 	execbuf.buffers_ptr = to_user_pointer(obj);
 	execbuf.buffer_count = 2;
-	if (gen < 6)
+	if (gem_store_dword_needs_secure(fd))
 		execbuf.flags |= I915_EXEC_SECURE;
 	execbuf.rsvd1 = ctx->id;
 
@@ -1310,14 +1310,13 @@ static void concurrent_child(int i915, const intel_ctx_t *ctx,
 			     uint32_t *common, int num_common,
 			     int in, int out)
 {
-	const unsigned int gen = intel_gen(intel_get_drm_devid(i915));
 	int idx = flags_to_index(e);
 	uint64_t relocs = concurrent_relocs(i915, idx, CONCURRENT);
 	struct drm_i915_gem_exec_object2 obj[num_common + 2];
 	struct drm_i915_gem_execbuffer2 execbuf = {
 		.buffers_ptr = to_user_pointer(obj),
 		.buffer_count = ARRAY_SIZE(obj),
-		.flags = e->flags | I915_EXEC_HANDLE_LUT | (gen < 6 ? I915_EXEC_SECURE : 0),
+		.flags = e->flags | I915_EXEC_HANDLE_LUT,
 		.rsvd1 = ctx->id,
 	};
 	uint32_t *batch = &obj[num_common + 1].handle;
@@ -1325,6 +1324,9 @@ static void concurrent_child(int i915, const intel_ctx_t *ctx,
 	uint32_t *x;
 	int err = 0;
 
+	if (gem_store_dword_needs_secure(i915))
+		execbuf.flags |= I915_EXEC_SECURE;
+
 	memset(obj, 0, sizeof(obj));
 	obj[0].handle = gem_create(i915, 64 * CONCURRENT * 4);
 
@@ -1623,7 +1625,7 @@ static void invalid_domains(int fd)
 	gem_close(fd, obj[0].handle);
 }
 
-igt_main
+int igt_main()
 {
 	const intel_ctx_t *ctx;
 	const struct intel_execution_engine2 *e;
@@ -1664,7 +1666,7 @@ igt_main
 	uint64_t size;
 	int fd = -1;
 
-	igt_fixture {
+	igt_fixture() {
 		fd = drm_open_driver_master(DRIVER_INTEL);
 		igt_require_gem(fd);
 		/* Check if relocations supported by platform */
@@ -1675,8 +1677,8 @@ igt_main
 	for (f = flags; f->name; f++) {
 		igt_hang_t hang;
 
-		igt_subtest_group {
-			igt_fixture {
+		igt_subtest_group() {
+			igt_fixture() {
 				if (f->flags & HANG)
 					hang = igt_allow_hang(fd, 0, 0);
 			}
@@ -1701,7 +1703,7 @@ igt_main
 				}
 			}
 
-			igt_fixture {
+			igt_fixture() {
 				if (f->flags & HANG)
 					igt_disallow_hang(fd, hang);
 			}
@@ -1750,13 +1752,13 @@ igt_main
 	igt_subtest("invalid-domains")
 		invalid_domains(fd);
 
-	igt_subtest_group {
+	igt_subtest_group() {
 		igt_display_t display = {
 			.drm_fd = fd,
 			.n_pipes = IGT_MAX_PIPES
 		};
 
-		igt_fixture {
+		igt_fixture() {
 			igt_device_set_master(fd);
 			kmstest_set_vt_graphics_mode();
 			igt_display_require(&display, fd);
@@ -1769,10 +1771,10 @@ igt_main
 			}
 		}
 
-		igt_fixture
+		igt_fixture()
 			igt_display_fini(&display);
 	}
 
-	igt_fixture
+	igt_fixture()
 		drm_close_driver(fd);
 }
