@@ -187,7 +187,7 @@ static uint32_t __store_dword(int fd, uint64_t ahnd, const intel_ctx_t *ctx,
 	execbuf.buffers_ptr = to_user_pointer(obj + !cork);
 	execbuf.buffer_count = 2 + !!cork;
 	execbuf.flags = ring;
-	if (gen < 6)
+	if (gem_store_dword_needs_secure(fd))
 		execbuf.flags |= I915_EXEC_SECURE;
 	execbuf.rsvd1 = ctx->id;
 
@@ -2342,7 +2342,7 @@ static void reorder_wide(int fd, const intel_ctx_cfg_t *cfg, unsigned ring)
 	execbuf.buffers_ptr = to_user_pointer(obj);
 	execbuf.buffer_count = ARRAY_SIZE(obj);
 	execbuf.flags = ring;
-	if (gen < 6)
+	if (gem_store_dword_needs_secure(fd))
 		execbuf.flags |= I915_EXEC_SECURE;
 
 	execbuf.flags |= I915_EXEC_FENCE_IN;
@@ -2447,6 +2447,7 @@ static void test_pi_ringfull(int fd, const intel_ctx_cfg_t *cfg,
 	struct itimerval itv;
 	IGT_CORK_HANDLE(c);
 	bool *result;
+	bool *sync;
 
 	/*
 	 * We start simple. A low priority client should never prevent a high
@@ -2469,6 +2470,8 @@ static void test_pi_ringfull(int fd, const intel_ctx_cfg_t *cfg,
 	result = mmap(NULL, 4096, PROT_WRITE, MAP_SHARED | MAP_ANON, -1, 0);
 	igt_assert(result != MAP_FAILED);
 
+	sync = result + 0x100;
+
 	memset(&execbuf, 0, sizeof(execbuf));
 	memset(&obj, 0, sizeof(obj));
 
@@ -2528,10 +2531,15 @@ static void test_pi_ringfull(int fd, const intel_ctx_cfg_t *cfg,
 
 	/* both parent + child on the same cpu, only parent is RT */
 	bind_to_cpu(0);
+	*sync = 0;
 
 	igt_fork(child, 1) {
 		int err;
 
+		/* We want the parent to always go first */
+		while (!*sync)
+			sched_yield();
+
 		/* Replace our batch to avoid conflicts over shared resources? */
 		if (!(flags & SHARED)) {
 			obj[1].handle = gem_create(fd, 4096);
@@ -2542,7 +2550,6 @@ static void test_pi_ringfull(int fd, const intel_ctx_cfg_t *cfg,
 
 		igt_debug("Waking parent\n");
 		kill(getppid(), SIGALRM);
-		sched_yield();
 		result[1] = true;
 
 		sigaction(SIGALRM, &sa, NULL);
@@ -2566,17 +2573,19 @@ static void test_pi_ringfull(int fd, const intel_ctx_cfg_t *cfg,
 
 		if (!(flags & SHARED))
 			gem_close(fd, obj[1].handle);
+
+		/* Interrupt parent's execbuf now that the child is done */
+		kill(getppid(), SIGALRM);
 	}
 
+	*sync = 1;
+
 	/* Relinquish CPU just to allow child to create a context */
-	sleep(1);
+	pause();
 	igt_assert_f(result[0], "HP context (child) not created\n");
 	igt_assert_f(!result[1], "Child released too early!\n");
 
 	/* Parent sleeps waiting for ringspace, releasing child */
-	itv.it_value.tv_sec = 0;
-	itv.it_value.tv_usec = 50000;
-	setitimer(ITIMER_REAL, &itv, NULL);
 	igt_debug("LP parent executing\n");
 	igt_assert_eq(__execbuf(fd, &execbuf), -EINTR);
 	igt_assert_f(result[1], "Child was not released!\n");
@@ -3193,12 +3202,12 @@ static void fairslice(int i915, const intel_ctx_cfg_t *cfg,
 			    !gem_engine_can_block_ggtt_binder(i915, e)) \
 		igt_dynamic_f("%s", e->name)
 
-igt_main
+int igt_main()
 {
 	int fd = -1;
 	const intel_ctx_t *ctx = NULL;
 
-	igt_fixture {
+	igt_fixture() {
 		igt_require_sw_sync();
 
 		fd = drm_open_driver_master(DRIVER_INTEL);
@@ -3213,7 +3222,7 @@ igt_main
 		igt_fork_hang_detector(fd);
 	}
 
-	igt_subtest_group {
+	igt_subtest_group() {
 		const struct intel_execution_engine2 *e;
 
 		test_each_ggtt_binder_nonblocking_engine("fifo", fd, ctx, e)
@@ -3235,10 +3244,10 @@ igt_main
 			independent(fd, ctx, e->flags, IGT_SPIN_USERPTR);
 	}
 
-	igt_subtest_group {
+	igt_subtest_group() {
 		const struct intel_execution_engine2 *e;
 
-		igt_fixture {
+		igt_fixture() {
 			igt_require(gem_scheduler_enabled(fd));
 			igt_require(gem_scheduler_has_ctx_priority(fd));
 		}
@@ -3257,8 +3266,8 @@ igt_main
 		test_each_engine("u-lateslice", fd, ctx, e)
 			lateslice(fd, &ctx->cfg, e->flags, IGT_SPIN_USERPTR);
 
-		igt_subtest_group {
-			igt_fixture {
+		igt_subtest_group() {
+			igt_fixture() {
 				igt_require(gem_scheduler_has_timeslicing(fd));
 				igt_require(intel_gen(intel_get_drm_devid(fd)) >= 8);
 			}
@@ -3269,7 +3278,7 @@ igt_main
 			test_each_engine("u-fairslice", fd, ctx, e)
 				fairslice(fd, &ctx->cfg, e, IGT_SPIN_USERPTR, 2);
 
-			igt_fixture {
+			igt_fixture() {
 				intel_allocator_multiprocess_start();
 			}
 			igt_subtest("fairslice-all")  {
@@ -3288,7 +3297,7 @@ igt_main
 				}
 				igt_waitchildren();
 			}
-			igt_fixture {
+			igt_fixture() {
 				intel_allocator_multiprocess_stop();
 			}
 		}
@@ -3336,8 +3345,8 @@ igt_main
 		test_each_ggtt_binder_nonblocking_engine("promotion", fd, ctx, e)
 			promotion(fd, &ctx->cfg, e->flags);
 
-		igt_subtest_group {
-			igt_fixture {
+		igt_subtest_group() {
+			igt_fixture() {
 				igt_require(gem_scheduler_has_preemption(fd));
 			}
 
@@ -3362,8 +3371,8 @@ igt_main
 			test_each_engine_store("preempt-engines", fd, ctx, e)
 				preempt_engines(fd, e, 0);
 
-			igt_subtest_group {
-				igt_fixture {
+			igt_subtest_group() {
+				igt_fixture() {
 					igt_require(!gem_scheduler_has_static_priority(fd));
 				}
 
@@ -3379,10 +3388,10 @@ igt_main
 					preempt_queue(fd, &ctx->cfg, e->flags, CONTEXTS | CHAIN);
 			}
 
-			igt_subtest_group {
+			igt_subtest_group() {
 				igt_hang_t hang;
 
-				igt_fixture {
+				igt_fixture() {
 					igt_stop_hang_detector();
 					hang = igt_allow_hang(fd, ctx->id, 0);
 				}
@@ -3393,7 +3402,7 @@ igt_main
 				test_each_engine_store("preemptive-hang", fd, ctx, e)
 					preemptive_hang(fd, &ctx->cfg, e);
 
-				igt_fixture {
+				igt_fixture() {
 					igt_disallow_hang(fd, hang);
 					igt_fork_hang_detector(fd);
 				}
@@ -3422,8 +3431,8 @@ igt_main
 		test_each_engine_store("smoketest", fd, ctx, e)
 			smoketest(fd, &ctx->cfg, e->flags, 5);
 
-		igt_subtest_group {
-			igt_fixture {
+		igt_subtest_group() {
+			igt_fixture() {
 				igt_require(!gem_scheduler_has_static_priority(fd));
 			}
 
@@ -3432,10 +3441,10 @@ igt_main
 		}
 	}
 
-	igt_subtest_group {
+	igt_subtest_group() {
 		const struct intel_execution_engine2 *e;
 
-		igt_fixture {
+		igt_fixture() {
 			igt_require(gem_scheduler_enabled(fd));
 			igt_require(gem_scheduler_has_ctx_priority(fd));
 			igt_require(gem_scheduler_has_preemption(fd));
@@ -3457,8 +3466,8 @@ igt_main
 			test_pi_iova(fd, &ctx->cfg, e->flags, SHARED);
 	}
 
-	igt_subtest_group {
-		igt_fixture {
+	igt_subtest_group() {
+		igt_fixture() {
 			igt_require(gem_scheduler_enabled(fd));
 			igt_require(gem_scheduler_has_semaphores(fd));
 		}
@@ -3467,7 +3476,7 @@ igt_main
 			measure_semaphore_power(fd, ctx);
 	}
 
-	igt_fixture {
+	igt_fixture() {
 		igt_stop_hang_detector();
 		intel_ctx_destroy(fd, ctx);
 		drm_close_driver(fd);
