@@ -720,7 +720,7 @@ store_ring(int fd, const intel_ctx_t *ctx, unsigned ring,
 		execbuf.flags = ied_flags(&ied, child);
 		execbuf.flags |= I915_EXEC_NO_RELOC;
 		execbuf.flags |= I915_EXEC_HANDLE_LUT;
-		if (gen < 6)
+		if (gem_store_dword_needs_secure(fd))
 			execbuf.flags |= I915_EXEC_SECURE;
 		execbuf.rsvd1 = ctx->id;
 
@@ -829,7 +829,7 @@ switch_ring(int fd, const intel_ctx_t *ctx, unsigned ring,
 			c->execbuf.flags = ied_flags(&ied, child);
 			c->execbuf.flags |= I915_EXEC_NO_RELOC;
 			c->execbuf.flags |= I915_EXEC_HANDLE_LUT;
-			if (gen < 6)
+			if (gem_store_dword_needs_secure(fd))
 				c->execbuf.flags |= I915_EXEC_SECURE;
 
 			c->ctx = intel_ctx_create(fd, &ctx->cfg);
@@ -997,7 +997,7 @@ __store_many(int fd, const intel_ctx_t *ctx, unsigned ring,
 	execbuf.flags = ring;
 	execbuf.flags |= I915_EXEC_NO_RELOC;
 	execbuf.flags |= I915_EXEC_HANDLE_LUT;
-	if (gen < 6)
+	if (gem_store_dword_needs_secure(fd))
 		execbuf.flags |= I915_EXEC_SECURE;
 	execbuf.rsvd1 = ctx->id;
 
@@ -1212,7 +1212,7 @@ store_all(int fd, const intel_ctx_t *ctx, int num_children, int timeout)
 		execbuf.buffers_ptr = to_user_pointer(object);
 		execbuf.flags |= I915_EXEC_NO_RELOC;
 		execbuf.flags |= I915_EXEC_HANDLE_LUT;
-		if (gen < 6)
+		if (gem_store_dword_needs_secure(fd))
 			execbuf.flags |= I915_EXEC_SECURE;
 		execbuf.rsvd1 = ctx->id;
 
@@ -1365,7 +1365,7 @@ preempt(int fd, const intel_ctx_t *ctx, unsigned ring,
 	intel_ctx_destroy(fd, tmp_ctx[0]);
 }
 
-igt_main
+int igt_main()
 {
 	const int ncpus = sysconf(_SC_NPROCESSORS_ONLN);
 	const struct {
@@ -1436,7 +1436,7 @@ igt_main
 	const intel_ctx_t *ctx;
 	int fd = -1;
 
-	igt_fixture {
+	igt_fixture() {
 		fd = drm_open_driver(DRIVER_INTEL);
 		igt_require_gem(fd);
 		gem_submission_print_method(fd);
@@ -1502,8 +1502,8 @@ igt_main
 		}
 	}
 
-	igt_subtest_group {
-		igt_fixture {
+	igt_subtest_group() {
+		igt_fixture() {
 			gem_require_contexts(fd);
 			igt_require(gem_scheduler_has_ctx_priority(fd));
 			igt_require(gem_scheduler_has_preemption(fd));
@@ -1525,7 +1525,7 @@ igt_main
 		}
 	}
 
-	igt_fixture {
+	igt_fixture() {
 		intel_allocator_multiprocess_stop();
 		igt_stop_hang_detector();
 		intel_ctx_destroy(fd, ctx);
