@@ -69,171 +69,65 @@
 #include "intel_blt.h"
 /**
  * TEST: gem userptr blits
- * Run type: FULL
+ * Category: Core
+ * Mega feature: Compute
+ * Sub-category: Compute tests
+ * Functionality: userptr blits
+ * Feature: userptr
+ * Test category: GEM_Legacy
  *
  * SUBTEST: access-control
- * Feature: userptr
- *
  * SUBTEST: coherency-sync
- * Feature: userptr
- *
  * SUBTEST: coherency-unsync
- * Feature: userptr
- *
  * SUBTEST: create-destroy-sync
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: create-destroy-unsync
- * Feature: userptr
- *
  * SUBTEST: dmabuf-sync
- * Feature: prime, userptr
- *
  * SUBTEST: dmabuf-unsync
- * Feature: prime, userptr
- *
  * SUBTEST: forbidden-operations
- * Feature: userptr
- *
  * SUBTEST: forked-access
- * Feature: userptr
- *
  * SUBTEST: forked-sync-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-sync-mempressure-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-sync-mempressure-normal
- * Feature: userptr
- *
  * SUBTEST: forked-sync-multifd-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-sync-multifd-mempressure-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-sync-multifd-mempressure-normal
- * Feature: userptr
- *
  * SUBTEST: forked-sync-multifd-normal
- * Feature: userptr
- *
  * SUBTEST: forked-sync-normal
- * Feature: userptr
- *
  * SUBTEST: forked-sync-swapping-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-sync-swapping-mempressure-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-sync-swapping-mempressure-normal
- * Feature: userptr
- *
  * SUBTEST: forked-sync-swapping-multifd-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-sync-swapping-multifd-mempressure-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-sync-swapping-multifd-mempressure-normal
- * Feature: userptr
- *
  * SUBTEST: forked-sync-swapping-multifd-normal
- * Feature: userptr
- *
  * SUBTEST: forked-sync-swapping-normal
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-mempressure-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-mempressure-normal
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-multifd-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-multifd-mempressure-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-multifd-mempressure-normal
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-multifd-normal
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-normal
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-swapping-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-swapping-mempressure-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-swapping-mempressure-normal
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-swapping-multifd-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-swapping-multifd-mempressure-interruptible
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-swapping-multifd-mempressure-normal
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-swapping-multifd-normal
- * Feature: userptr
- *
  * SUBTEST: forked-unsync-swapping-normal
- * Feature: userptr
- *
  * SUBTEST: huge-split
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: input-checking
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
  *
  * SUBTEST: invalid-mmap-offset-unsync
  * Description: Verify unsynchronized userptr on mmap-offset mappings fails
- * Feature: userptr
  *
  * SUBTEST: invalid-null-pointer
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: major-normal-sync
- * Feature: userptr
- *
  * SUBTEST: major-sync-interruptible
- * Feature: userptr
- *
  * SUBTEST: major-unsync-interruptible
- * Feature: userptr
- *
  * SUBTEST: major-unsync-normal
- * Feature: userptr
  *
  * SUBTEST: map-fixed-invalidate
  * Description: Try to anger lockdep with MMU notifier still active after MAP_FIXED remap
@@ -252,176 +146,45 @@
  * Feature: gtt, userptr
  *
  * SUBTEST: minor-normal-sync
- * Feature: userptr
- *
  * SUBTEST: minor-sync-interruptible
- * Feature: userptr
- *
  * SUBTEST: minor-unsync-interruptible
- * Feature: userptr
- *
  * SUBTEST: minor-unsync-normal
- * Feature: userptr
- *
  * SUBTEST: mlocked-normal-sync
- * Feature: userptr
- *
  * SUBTEST: mlocked-sync-interruptible
- * Feature: userptr
- *
  * SUBTEST: mlocked-unsync-interruptible
- * Feature: userptr
- *
  * SUBTEST: mlocked-unsync-normal
- * Feature: userptr
  *
  * SUBTEST: mmap-offset-banned
- * Category: Server
  * Description: Verify mmap_offset to userptr is banned
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
  *
  * SUBTEST: nohangcheck
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: probe
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: process-exit
- * Feature: userptr
- *
  * SUBTEST: process-exit-busy
- * Feature: userptr
- *
  * SUBTEST: readonly-pwrite-unsync
- * Feature: userptr
- *
  * SUBTEST: readonly-unsync
- * Feature: userptr
- *
  * SUBTEST: relocations
- * Feature: userptr
- *
  * SUBTEST: sd-probe
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: set-cache-level
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: stress-mm
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: stress-mm-invalidate-close
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: stress-mm-invalidate-close-overlap
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: stress-purge
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: swapping-normal-sync
- * Feature: userptr
- *
  * SUBTEST: swapping-sync-interruptible
- * Feature: userptr
- *
  * SUBTEST: swapping-unsync-interruptible
- * Feature: userptr
- *
  * SUBTEST: swapping-unsync-normal
- * Feature: userptr
- *
  * SUBTEST: sync-overlap
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: sync-unmap
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: sync-unmap-after-close
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: sync-unmap-cycles
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: unsync-overlap
- * Feature: userptr
- *
  * SUBTEST: unsync-unmap
- * Feature: userptr
- *
  * SUBTEST: unsync-unmap-after-close
- * Feature: userptr
- *
  * SUBTEST: unsync-unmap-cycles
- * Feature: userptr
- *
  * SUBTEST: usage-restrictions
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
- *
  * SUBTEST: userfault
- * Feature: userptr
- *
  * SUBTEST: vma-merge
- * Category: Server
- * Feature: userptr
- * Functionality: buffer management
- * Sub-category: Compute
- * Test category: GEM_Legacy
  */
 
 #ifndef PAGE_SIZE
@@ -700,7 +463,7 @@ static void store_dword(int fd, uint32_t target,
 	execbuf.buffers_ptr = to_user_pointer(obj);
 	execbuf.buffer_count = ARRAY_SIZE(obj);
 	execbuf.flags = 0;
-	if (gen < 6)
+	if (gem_store_dword_needs_secure(fd))
 		execbuf.flags |= I915_EXEC_SECURE;
 
 	memset(obj, 0, sizeof(obj));
@@ -1051,7 +814,7 @@ static void test_nohangcheck_hostile(int i915)
 						IGT_SPIN_FENCE_OUT));
 
 		new = fill_ring(i915, &spin->execbuf);
-		igt_assert(new != -1);
+		igt_assert_neq(new, -1);
 		spin->out_fence = -1;
 
 		if (fence < 0) {
@@ -1068,7 +831,7 @@ static void test_nohangcheck_hostile(int i915)
 	}
 	intel_ctx_destroy(i915, ctx);
 	put_ahnd(ahnd);
-	igt_assert(fence != -1);
+	igt_assert_neq(fence, -1);
 
 	if (sync_fence_wait(fence, MSEC_PER_SEC)) { /* 640ms preempt-timeout */
 		igt_debugfs_dump(i915, "i915_engine_info");
@@ -1668,7 +1431,7 @@ static void store_dword_rand(int i915, const intel_ctx_t *ctx,
 	batchsz = count * 16 + 4;
 	batchsz = ALIGN(batchsz, 4096);
 
-	reloc = calloc(sizeof(*reloc), count);
+	reloc = calloc(count, sizeof(*reloc));
 
 	memset(obj, 0, sizeof(obj));
 	obj[0].handle = target;
@@ -1683,7 +1446,7 @@ static void store_dword_rand(int i915, const intel_ctx_t *ctx,
 	exec.buffer_count = 2;
 	exec.buffers_ptr = to_user_pointer(obj);
 	exec.flags = engine;
-	if (gen < 6)
+	if (gem_store_dword_needs_secure(i915))
 		exec.flags |= I915_EXEC_SECURE;
 	exec.rsvd1 = ctx->id;
 
@@ -2734,12 +2497,12 @@ static int opt_handler(int opt, int opt_index, void *data)
 
 const char *help_str = "  -c\tBuffer count\n";
 
-igt_main_args("c:", NULL, help_str, opt_handler, NULL)
+int igt_main_args("c:", NULL, help_str, opt_handler, NULL)
 {
 	int size = sizeof(linear);
 	igt_fd_t(fd);
 
-	igt_fixture {
+	igt_fixture() {
 		unsigned int mmo_max = 0;
 
 		fd = drm_open_driver(DRIVER_INTEL);
@@ -2769,8 +2532,8 @@ igt_main_args("c:", NULL, help_str, opt_handler, NULL)
 		}
 	}
 
-	igt_subtest_group {
-		igt_fixture {
+	igt_subtest_group() {
+		igt_fixture() {
 			/* Either mode will do for parameter checking */
 			gem_userptr_test_synchronized();
 			if (!has_userptr(fd))
@@ -2814,10 +2577,10 @@ igt_main_args("c:", NULL, help_str, opt_handler, NULL)
 		}
 	}
 
-	igt_subtest_group {
+	igt_subtest_group() {
 		gem_userptr_test_unsynchronized();
 
-		igt_fixture {
+		igt_fixture() {
 			igt_require(has_userptr(fd));
 		}
 
@@ -2881,7 +2644,7 @@ igt_main_args("c:", NULL, help_str, opt_handler, NULL)
 			test_major_evictions(fd, size, count);
 		}
 
-		igt_fixture {
+		igt_fixture() {
 			size = sizeof(linear);
 			count = 2 * gem_aperture_size(fd) / (1024*1024) / 3;
 			if (count > total_ram * 3 / 4)
@@ -2908,10 +2671,10 @@ igt_main_args("c:", NULL, help_str, opt_handler, NULL)
 		igt_stop_signal_helper();
 	}
 
-	igt_subtest_group {
+	igt_subtest_group() {
 		gem_userptr_test_synchronized();
 
-		igt_fixture {
+		igt_fixture() {
 			igt_require(has_userptr(fd));
 			size = sizeof(linear);
 			count = 2 * gem_aperture_size(fd) / (1024*1024) / 3;
@@ -3007,7 +2770,7 @@ igt_main_args("c:", NULL, help_str, opt_handler, NULL)
 			test_major_evictions(fd, size, count);
 		}
 
-		igt_fixture {
+		igt_fixture() {
 			size = 1024 * 1024;
 			count = 2 * gem_aperture_size(fd) / (1024*1024) / 3;
 			if (count > total_ram * 3 / 4)
@@ -3034,8 +2797,8 @@ igt_main_args("c:", NULL, help_str, opt_handler, NULL)
 		igt_stop_signal_helper();
 	}
 
-	igt_subtest_group {
-		igt_fixture {
+	igt_subtest_group() {
+		igt_fixture() {
 			gem_userptr_test_synchronized();
 			if (!has_userptr(fd))
 				gem_userptr_test_unsynchronized();
