@@ -436,11 +436,11 @@ static void fbc_dirty_rectangle_test(data_t *data, void (*test_func)(data_t *))
 	cleanup(data);
 }
 
-igt_main
+int igt_main()
 {
 	data_t data = {0};
 
-	igt_fixture {
+	igt_fixture() {
 		data.drm_fd = drm_open_driver_master(DRIVER_XE);
 		igt_require(data.drm_fd >= 0);
 		data.debugfs_fd = igt_debugfs_dir(data.drm_fd);
@@ -530,7 +530,7 @@ igt_main
 		}
 	}
 
-	igt_fixture {
+	igt_fixture() {
 		igt_display_fini(&data.display);
 		close(data.drm_fd);
 	}
