@@ -40,7 +40,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include "i915/intel_fbc.h"
 
 /**
  * SUBTEST: fbc
@@ -183,6 +182,13 @@ static bool fbc_wait_until_update(struct drm_info *drm)
 	if (intel_gen(intel_get_drm_devid(drm->fd)) >= 9) {
 		if (!fbc_wait_until_enabled(drm->debugfs_fd))
 			return false;
+		/*
+		 * Skip cursor blinking check when running in simulation mode.
+		 * CRC operations are significantly slower in simulation,
+		 * so CRC checks are avoided to prevent false negatives.
+		 */
+		if (igt_run_in_simulation())
+			return true;
 
 		return fbc_check_cursor_blinking(drm);
 	} else {
@@ -445,18 +451,17 @@ static void teardown_environment(struct drm_info *drm)
 	kmstest_restore_vt_mode();
 }
 
-igt_main
+int igt_main()
 {
 	struct drm_info drm = { .fd = -1 };
 
-	igt_fixture
+	igt_fixture()
 		setup_environment(&drm);
 
 	igt_describe("Test the relationship between fbcon and the frontbuffer "
 		     "tracking infrastructure with fbc enabled.");
 	igt_subtest("fbc") {
-		/* FBC disabled: Wa_16023588340 */
-		igt_skip_on_f(intel_is_fbc_disabled_by_wa(drm.fd), "WA has disabled FBC on BMG\n");
+		igt_require_f(!IS_BATTLEMAGE(drm.devid), "FBC isn't supported on BMG\n");
 		subtest(&drm, &fbc, false);
 	}
 
@@ -468,8 +473,7 @@ igt_main
 	igt_describe("Suspend test to validate  the relationship between fbcon and the frontbuffer "
 		     "tracking infrastructure with fbc enabled.");
 	igt_subtest("fbc-suspend") {
-		/* FBC disabled: Wa_16023588340 */
-		igt_skip_on_f(intel_is_fbc_disabled_by_wa(drm.fd), "WA has disabled FBC on BMG\n");
+		igt_require_f(!IS_BATTLEMAGE(drm.devid), "FBC isn't supported on BMG\n");
 		subtest(&drm, &fbc, true);
 	}
 
@@ -478,6 +482,6 @@ igt_main
 	igt_subtest("psr-suspend")
 		subtest(&drm, &psr, true);
 
-	igt_fixture
+	igt_fixture()
 		teardown_environment(&drm);
 }
