@@ -48,7 +48,6 @@ struct batch_data {
 	uint32_t expected_data;
 };
 
-#define VAL_ATOMIC_EXPECTED  56
 #define WRITE_VALUE(data__, i__)	({			\
 	if (!(data__)->expected_data)				\
 		(data__)->expected_data = rand() << 12 | (i__);	\
@@ -71,7 +70,7 @@ static void write_dword(struct test_exec_data *data, uint64_t sdi_addr, uint32_t
 	uint32_t *batch = data->batch;
 
 	if (atomic) {
-		data->data = VAL_ATOMIC_EXPECTED - 1;
+		data->data = wdata - 1;
 		batch[(*idx)++] = MI_ATOMIC | MI_ATOMIC_INC;
 		batch[(*idx)++] = sdi_addr;
 		batch[(*idx)++] = sdi_addr >> 32;
@@ -715,6 +714,7 @@ processes_evict(int fd, uint64_t alloc_size, uint64_t stride,
 #define CPU_FAULT	(0x1 << 0)
 #define REMAP		(0x1 << 1)
 #define MIDDLE		(0x1 << 2)
+#define ATOMIC_ACCESS	(0x1 << 3)
 
 /**
  * SUBTEST: partial-munmap-cpu-fault
@@ -748,6 +748,38 @@ processes_evict(int fd, uint64_t alloc_size, uint64_t stride,
  * SUBTEST: partial-middle-remap-no-cpu-fault
  * Description: remap middle with no cpu access in between
  * Test category: functionality test
+ *
+ * SUBTEST: partial-atomic-munmap-cpu-fault
+ * Description: munmap partially with cpu access in between, GPU uses atomics
+ * Test category: functionality test
+ *
+ * SUBTEST: partial-atomic-munmap-no-cpu-fault
+ * Description: munmap partially with no cpu access in between, GPU uses atomics
+ * Test category: functionality test
+ *
+ * SUBTEST: partial-atomic-remap-cpu-fault
+ * Description: remap partially with cpu access in between, GPU uses atomics
+ * Test category: functionality test
+ *
+ * SUBTEST: partial-atomic-remap-no-cpu-fault
+ * Description: remap partially with no cpu access in between, GPU uses atomics
+ * Test category: functionality test
+ *
+ * SUBTEST: partial-atomic-middle-munmap-cpu-fault
+ * Description: munmap middle with cpu access in between, GPU uses atomics
+ * Test category: functionality test
+ *
+ * SUBTEST: partial-atomic-middle-munmap-no-cpu-fault
+ * Description: munmap middle with no cpu access in between, GPU uses atomics
+ * Test category: functionality test
+ *
+ * SUBTEST: partial-atomic-middle-remap-cpu-fault
+ * Description: remap middle with cpu access in between, GPU uses atomics
+ * Test category: functionality test
+ *
+ * SUBTEST: partial-atomic-middle-remap-no-cpu-fault
+ * Description: remap middle with no cpu access in between, GPU uses atomics
+ * Test category: functionality test
  */
 
 static void
@@ -762,20 +794,14 @@ partial(int fd, struct drm_xe_engine_class_instance *eci, unsigned int flags)
 		.num_syncs = 1,
 		.syncs = to_user_pointer(sync),
 	};
-	struct {
-		uint32_t batch[16];
-		uint64_t pad;
-		uint64_t vm_sync;
-		uint64_t exec_sync;
-		uint32_t data;
-		uint32_t expected_data;
-	} *data;
+	struct test_exec_data *data;
 	size_t bo_size = SZ_2M, unmap_offset = 0;
 	uint32_t vm, exec_queue;
 	u64 *exec_ufence = NULL;
 	int i;
 	void *old, *new = NULL;
 	struct aligned_alloc_type alloc;
+	bool atomic = flags & ATOMIC_ACCESS;
 
 	if (flags & MIDDLE)
 		unmap_offset = bo_size / 4;
@@ -811,8 +837,8 @@ partial(int fd, struct drm_xe_engine_class_instance *eci, unsigned int flags)
 		uint64_t sdi_addr = addr + sdi_offset;
 		int b = 0;
 
-		write_dword((struct test_exec_data *)&data[i], sdi_addr, WRITE_VALUE(&data[i], i),
-			    &b, false);
+		write_dword(&data[i], sdi_addr, WRITE_VALUE(&data[i], i),
+			    &b, atomic);
 		igt_assert(b <= ARRAY_SIZE(data[i].batch));
 
 		if (!i)
@@ -834,8 +860,9 @@ partial(int fd, struct drm_xe_engine_class_instance *eci, unsigned int flags)
 
 		xe_wait_ufence(fd, new ?: exec_ufence, USER_FENCE_VALUE,
 			       exec_queue, FIVE_SEC);
-		if (i || (flags & CPU_FAULT))
+		if (i || (flags & CPU_FAULT)) {
 			igt_assert_eq(data[i].data, READ_VALUE(&data[i]));
+		}
 		exec_ufence[0] = 0;
 
 		if (!i) {
@@ -1224,17 +1251,30 @@ xe_vm_parse_execute_madvise(int fd, uint32_t vm, struct test_exec_data *data,
 	}
 
 	if (flags & MADVISE_SPLIT_VMA) {
+		uint16_t dev_id = intel_get_drm_devid(fd);
+		uint64_t split_addr, split_size;
+		uint64_t alignment = SZ_4K;
+
+		if (IS_PONTEVECCHIO(dev_id))
+			alignment = SZ_64K;
+
 		if (bo_size)
-			bo_size = ALIGN(bo_size, SZ_4K);
+			bo_size = ALIGN(bo_size, alignment);
+
+		split_addr = to_user_pointer(data) + bo_size/2;
+		split_addr = ALIGN(split_addr, alignment);
+		split_size = bo_size / 2;
+		split_size = ALIGN(split_size, alignment);
 
 		bo_flags = DRM_XE_GEM_CREATE_FLAG_NEEDS_VISIBLE_VRAM;
 		bo = xe_bo_create(fd, vm, bo_size, vram_if_possible(fd, eci->gt_id),
 				  bo_flags);
-		xe_vm_bind_async(fd, vm, 0, bo, 0, to_user_pointer(data) + bo_size / 2,
-				 bo_size / 2, 0, 0);
 
-		__xe_vm_bind_assert(fd, vm, 0, 0, 0, to_user_pointer(data) + bo_size / 2,
-				    bo_size / 2, DRM_XE_VM_BIND_OP_MAP,
+		xe_vm_bind_async(fd, vm, 0, bo, 0, split_addr,
+				 split_size, 0, 0);
+
+		__xe_vm_bind_assert(fd, vm, 0, 0, 0, split_addr,
+				    split_size, DRM_XE_VM_BIND_OP_MAP,
 				    DRM_XE_VM_BIND_FLAG_CPU_ADDR_MIRROR, sync,
 				    1, 0, 0);
 		xe_wait_ufence(fd, &data[0].vm_sync, USER_FENCE_VALUE, 0, FIVE_SEC);
@@ -1242,7 +1282,7 @@ xe_vm_parse_execute_madvise(int fd, uint32_t vm, struct test_exec_data *data,
 		gem_close(fd, bo);
 		bo = 0;
 
-		xe_vm_madvise_atomic_attr(fd, vm, to_user_pointer(data), bo_size / 2,
+		xe_vm_madvise_atomic_attr(fd, vm, split_addr, split_size,
 					  DRM_XE_ATOMIC_GLOBAL);
 	}
 
@@ -1304,6 +1344,7 @@ madvise_prefetch_op(int fd, uint32_t vm, uint64_t addr, size_t bo_size,
 			return;
 		}
 
+		free(mem_attrs);
 	} else if (flags & PREFETCH_SAME_ATTR) {
 		xe_vm_madvise_atomic_attr(fd, vm, to_user_pointer(data), bo_size,
 					  DRM_XE_ATOMIC_GLOBAL);
@@ -1319,6 +1360,7 @@ madvise_prefetch_op(int fd, uint32_t vm, uint64_t addr, size_t bo_size,
 
 		xe_vm_madvise_atomic_attr(fd, vm, to_user_pointer(data), bo_size / 2,
 					  DRM_XE_ATOMIC_GLOBAL);
+		free(mem_attrs);
 	} else if (flags & PREFETCH_CHANGE_ATTR) {
 		xe_vm_madvise_atomic_attr(fd, vm, to_user_pointer(data), bo_size,
 					  DRM_XE_ATOMIC_GLOBAL);
@@ -1334,8 +1376,8 @@ madvise_prefetch_op(int fd, uint32_t vm, uint64_t addr, size_t bo_size,
 		xe_vm_madvise_atomic_attr(fd, vm, to_user_pointer(data), bo_size,
 					  DRM_XE_ATOMIC_DEVICE);
 		}
+		free(mem_attrs);
 	}
-	free(mem_attrs);
 }
 
 static void
@@ -1740,25 +1782,17 @@ test_exec(int fd, struct drm_xe_engine_class_instance *eci,
 				if (flags & FORK_READ) {
 					igt_fork(child, 1)
 						igt_assert_eq(data[idx].data,
-							      flags & ATOMIC_BATCH
-							      ? VAL_ATOMIC_EXPECTED
-							      : READ_VALUE(&data[idx]));
+							      READ_VALUE(&data[idx]));
 					if (!(flags & FORK_READ_AFTER))
 						igt_assert_eq(data[idx].data,
-							      flags & ATOMIC_BATCH
-							      ? VAL_ATOMIC_EXPECTED
-							      : READ_VALUE(&data[idx]));
+							      READ_VALUE(&data[idx]));
 					igt_waitchildren();
 					if (flags & FORK_READ_AFTER)
 						igt_assert_eq(data[idx].data,
-							      flags & ATOMIC_BATCH
-							      ? VAL_ATOMIC_EXPECTED
-							      : READ_VALUE(&data[idx]));
+							      READ_VALUE(&data[idx]));
 				} else {
 					igt_assert_eq(data[idx].data,
-						      flags & ATOMIC_BATCH
-						      ? VAL_ATOMIC_EXPECTED
-						      : READ_VALUE(&data[idx]));
+						      READ_VALUE(&data[idx]));
 					if (flags & PREFETCH_SYS_BENCHMARK) {
 						struct timespec tv = {};
 						u64 start, end;
@@ -1785,17 +1819,13 @@ test_exec(int fd, struct drm_xe_engine_class_instance *eci,
 								((void *)data) + j * orig_size;
 
 							igt_assert_eq(__data[idx].data,
-								      flags & ATOMIC_BATCH
-								      ? VAL_ATOMIC_EXPECTED
-								      : READ_VALUE(&data[idx]));
+								      READ_VALUE(&data[idx]));
 						}
 					}
 				}
 				if (flags & EVERY_OTHER_CHECK)
 					igt_assert_eq(data[prev_idx].data,
-						      flags & ATOMIC_BATCH
-						      ? VAL_ATOMIC_EXPECTED
-						      : READ_VALUE(&data[prev_idx]));
+						      READ_VALUE(&data[prev_idx]));
 			}
 		}
 
@@ -2199,7 +2229,7 @@ struct section {
 	uint8_t (*fn)(int pat);
 };
 
-igt_main
+int igt_main()
 {
 	struct drm_xe_engine_class_instance *hwe;
 	const struct section sections[] = {
@@ -2296,6 +2326,14 @@ igt_main
 		{ "middle-munmap-no-cpu-fault", MIDDLE },
 		{ "middle-remap-cpu-fault", MIDDLE | CPU_FAULT | REMAP },
 		{ "middle-remap-no-cpu-fault", MIDDLE | REMAP },
+		{ "atomic-munmap-cpu-fault", ATOMIC_ACCESS | CPU_FAULT },
+		{ "atomic-munmap-no-cpu-fault", ATOMIC_ACCESS },
+		{ "atomic-remap-cpu-fault", ATOMIC_ACCESS | CPU_FAULT | REMAP },
+		{ "atomic-remap-no-cpu-fault", ATOMIC_ACCESS | REMAP },
+		{ "atomic-middle-munmap-cpu-fault", ATOMIC_ACCESS | MIDDLE | CPU_FAULT },
+		{ "atomic-middle-munmap-no-cpu-fault", ATOMIC_ACCESS | MIDDLE },
+		{ "atomic-middle-remap-cpu-fault", ATOMIC_ACCESS | MIDDLE | CPU_FAULT | REMAP },
+		{ "atomic-middle-remap-no-cpu-fault", ATOMIC_ACCESS | MIDDLE | REMAP },
 		{ NULL },
 	};
 	const struct section esections[] = {
@@ -2356,7 +2394,7 @@ igt_main
 
 	int fd;
 
-	igt_fixture {
+	igt_fixture() {
 		struct xe_device *xe;
 
 		fd = drm_open_driver(DRIVER_XE);
@@ -2574,7 +2612,7 @@ igt_main
 		xe_for_each_engine(fd, hwe)
 			test_compute(fd, hwe, SZ_2M);
 
-	igt_fixture {
+	igt_fixture() {
 		xe_device_put(fd);
 		drm_close_driver(fd);
 		close_sync_file();
