@@ -6,7 +6,11 @@
 #include <cairo.h>
 #include <errno.h>
 #include <fcntl.h>
+#ifdef ANDROID
+#include "android/glib.h"
+#else
 #include <glib.h>
+#endif
 #include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -18,13 +22,17 @@
 
 #include "igt.h"
 #include "igt_crc.h"
+#include "intel_blt.h"
 #include "intel_bufops.h"
+#include "intel_mocs.h"
+#include "intel_pat.h"
 #include "xe/xe_ioctl.h"
 #include "xe/xe_query.h"
 
 /**
  * TEST: Basic tests for intel-bb xe functionality
- * Category: Software building block
+ * Category: Core
+ * Mega feature: General Core features
  * Sub-category: IGT Lib
  * Functionality: intel-bb
  * Test category: functionality test
@@ -36,6 +44,7 @@
 #define HEIGHT	64
 #define STRIDE	(WIDTH * 4)
 #define SIZE	(HEIGHT * STRIDE)
+#define RENDER_WIDTH 256
 
 #define COLOR_00	0x00
 #define COLOR_33	0x33
@@ -49,15 +58,6 @@ static bool write_png;
 static bool buf_info;
 static bool print_base64;
 
-static void *alloc_aligned(uint64_t size)
-{
-	void *p;
-
-	igt_assert_eq(posix_memalign(&p, 16, size), 0);
-
-	return p;
-}
-
 static void fill_buf(struct intel_buf *buf, uint8_t color)
 {
 	uint8_t *ptr;
@@ -191,8 +191,9 @@ static void simple_bb(struct buf_ops *bops, bool new_context)
 	intel_bb_reset(ibb, true);
 
 	if (new_context) {
-		vm = xe_vm_create(xe, DRM_XE_VM_CREATE_ASYNC_BIND_OPS, 0);
-		ctx = xe_exec_queue_create(xe, vm, xe_hw_engine(xe, 0), 0);
+		vm = xe_vm_create(xe, 0, 0);
+		ctx = xe_exec_queue_create(xe, vm, &xe_engine(xe, 0)->instance,
+					   0);
 		intel_bb_destroy(ibb);
 		ibb = intel_bb_create_with_context(xe, ctx, vm, NULL, PAGE_SIZE);
 		intel_bb_out(ibb, MI_BATCH_BUFFER_END);
@@ -396,11 +397,12 @@ static void create_in_region(struct buf_ops *bops, uint64_t region)
 		intel_bb_set_debug(ibb, true);
 
 	size = xe_min_page_size(xe, system_memory(xe));
-	handle = xe_bo_create_flags(xe, 0, size, system_memory(xe));
+	handle = xe_bo_create(xe, 0, size, system_memory(xe), 0);
 	intel_buf_init_full(bops, handle, &buf,
 			    width/4, height, 32, 0,
 			    I915_TILING_NONE, 0,
-			    size, 0, region);
+			    size, 0, region, DEFAULT_PAT_INDEX,
+			    DEFAULT_MOCS_INDEX);
 	intel_buf_set_ownership(&buf, true);
 
 	intel_bb_add_intel_buf(ibb, &buf, false);
@@ -519,63 +521,6 @@ static void scratch_buf_init(struct buf_ops *bops,
 	igt_assert(intel_buf_height(buf) == height);
 }
 
-static void scratch_buf_draw_pattern(struct buf_ops *bops,
-				     struct intel_buf *buf,
-				     int x, int y, int w, int h,
-				     int cx, int cy, int cw, int ch,
-				     bool use_alternate_colors)
-{
-	cairo_surface_t *surface;
-	cairo_pattern_t *pat;
-	cairo_t *cr;
-	void *linear;
-
-	linear = alloc_aligned(buf->surface[0].size);
-
-	surface = cairo_image_surface_create_for_data(linear,
-						      CAIRO_FORMAT_RGB24,
-						      intel_buf_width(buf),
-						      intel_buf_height(buf),
-						      buf->surface[0].stride);
-
-	cr = cairo_create(surface);
-
-	cairo_rectangle(cr, cx, cy, cw, ch);
-	cairo_clip(cr);
-
-	pat = cairo_pattern_create_mesh();
-	cairo_mesh_pattern_begin_patch(pat);
-	cairo_mesh_pattern_move_to(pat, x,   y);
-	cairo_mesh_pattern_line_to(pat, x+w, y);
-	cairo_mesh_pattern_line_to(pat, x+w, y+h);
-	cairo_mesh_pattern_line_to(pat, x,   y+h);
-	if (use_alternate_colors) {
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 0, 0.0, 1.0, 1.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 1, 1.0, 0.0, 1.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 2, 1.0, 1.0, 0.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 3, 0.0, 0.0, 0.0);
-	} else {
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 0, 1.0, 0.0, 0.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 1, 0.0, 1.0, 0.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 2, 0.0, 0.0, 1.0);
-		cairo_mesh_pattern_set_corner_color_rgb(pat, 3, 1.0, 1.0, 1.0);
-	}
-	cairo_mesh_pattern_end_patch(pat);
-
-	cairo_rectangle(cr, x, y, w, h);
-	cairo_set_source(cr, pat);
-	cairo_fill(cr);
-	cairo_pattern_destroy(pat);
-
-	cairo_destroy(cr);
-
-	cairo_surface_destroy(surface);
-
-	linear_to_intel_buf(bops, buf, linear);
-
-	free(linear);
-}
-
 #define GROUP_SIZE 4096
 static int compare_detail(const uint32_t *ptr1, uint32_t *ptr2,
 			  uint32_t size)
@@ -654,7 +599,8 @@ static int dump_base64(const char *name, struct intel_buf *buf)
 
 		igt_info("--- %s ---\n", name);
 		pos = str = g_base64_encode(destbuf, outsize);
-		outsize = strlen(str);
+		outsize = str ? strlen(str) : 0;
+
 		while (pos) {
 			char line[LINELEN + 1];
 			int to_copy = min(LINELEN, outsize);
@@ -702,9 +648,9 @@ static int __do_intel_bb_blit(struct buf_ops *bops, uint32_t tiling)
 		intel_buf_print(&dst);
 	}
 
-	scratch_buf_draw_pattern(bops, &src,
-				 0, 0, width, height,
-				 0, 0, width, height, 0);
+	intel_buf_draw_pattern(bops, &src,
+			       0, 0, width, height,
+			       0, 0, width, height, 0);
 
 	intel_bb_blt_copy(ibb,
 			  &src, 0, 0, src.surface[0].stride,
@@ -733,11 +679,11 @@ static int __do_intel_bb_blit(struct buf_ops *bops, uint32_t tiling)
 
 	/* We'll fail on src <-> final compare so just warn */
 	if (tiling == I915_TILING_NONE) {
-		if (compare_bufs(&src, &dst, false) > 0)
-			igt_warn("none->none blit failed!");
+		igt_warn_on_f(compare_bufs(&src, &dst, false) > 0,
+			      "none->none blit failed!");
 	} else {
-		if (compare_bufs(&src, &dst, false) == 0)
-			igt_warn("none->tiled blit failed!");
+		igt_warn_on_f(compare_bufs(&src, &dst, false) == 0,
+			      "none->tiled blit failed!");
 	}
 
 	fails = compare_bufs(&src, &final, true);
@@ -952,11 +898,11 @@ static int render(struct buf_ops *bops, uint32_t tiling,
 	scratch_buf_init(bops, &final, width, height, I915_TILING_NONE,
 			 I915_COMPRESSION_NONE);
 
-	scratch_buf_draw_pattern(bops, &src,
-				 0, 0, width, height,
-				 0, 0, width, height, 0);
+	intel_buf_draw_pattern(bops, &src,
+			       0, 0, width, height,
+			       0, 0, width, height, 0);
 
-	render_copy = igt_get_render_copyfunc(devid);
+	render_copy = igt_get_render_copyfunc(xe);
 	igt_assert(render_copy);
 
 	render_copy(ibb,
@@ -977,18 +923,18 @@ static int render(struct buf_ops *bops, uint32_t tiling,
 	if (write_png) {
 		snprintf(name, sizeof(name) - 1,
 			 "render_dst_tiling_%d.png", tiling);
-		intel_buf_write_to_png(&src, "render_src_tiling_none.png");
-		intel_buf_write_to_png(&dst, name);
-		intel_buf_write_to_png(&final, "render_final_tiling_none.png");
+		intel_buf_raw_write_to_png(&src, "render_src_tiling_none.png");
+		intel_buf_raw_write_to_png(&dst, name);
+		intel_buf_raw_write_to_png(&final, "render_final_tiling_none.png");
 	}
 
 	/* We'll fail on src <-> final compare so just warn */
 	if (tiling == I915_TILING_NONE) {
-		if (compare_bufs(&src, &dst, false) > 0)
-			igt_warn("%s: none->none failed!\n", __func__);
+		igt_warn_on_f(compare_bufs(&src, &dst, false) > 0,
+			      "%s: none->none failed!\n", __func__);
 	} else {
-		if (compare_bufs(&src, &dst, false) == 0)
-			igt_warn("%s: none->tiled failed!\n", __func__);
+		igt_warn_on_f(compare_bufs(&src, &dst, false) == 0,
+			      "%s: none->tiled failed!\n", __func__);
 	}
 
 	fails = compare_bufs(&src, &final, true);
@@ -1038,22 +984,13 @@ const char *help_str =
 	"  -b\tDump to base64 (bb and images)\n"
 	;
 
-igt_main_args("dpib", NULL, help_str, opt_handler, NULL)
+int igt_main_args("dpib", NULL, help_str, opt_handler, NULL)
 {
-	int xe, i;
+	int xe;
 	struct buf_ops *bops;
 	uint32_t width;
 
-	struct test {
-		uint32_t tiling;
-		const char *tiling_name;
-	} tests[] = {
-		{ I915_TILING_NONE, "none" },
-		{ I915_TILING_X, "x" },
-		{ I915_TILING_Y, "y" },
-	};
-
-	igt_fixture {
+	igt_fixture() {
 		xe = drm_open_driver(DRIVER_XE);
 		bops = buf_ops_create(xe);
 	}
@@ -1115,18 +1052,23 @@ igt_main_args("dpib", NULL, help_str, opt_handler, NULL)
 		delta_check(bops);
 
 	igt_subtest_with_dynamic("render") {
+		int tiling;
+
 		igt_require(xe_has_engine_class(xe, DRM_XE_ENGINE_CLASS_RENDER));
 
-		for (i = 0; i < ARRAY_SIZE(tests); i++) {
-			const struct test *t = &tests[i];
+		for_each_tiling(tiling) {
+			if (!render_supports_tiling(xe, tiling, false))
+				continue;
 
-			for (width = 512; width <= 1024; width += 512)
-				igt_dynamic_f("render-%s-%u", t->tiling_name, width)
-					render(bops, t->tiling, width, width);
+			width = RENDER_WIDTH;
+			igt_dynamic_f("render-%s-%u",
+				      blt_tiling_name(tiling), width)
+				render(bops, blt_tile_to_i915_tile(tiling),
+				       width, width);
 		}
 	}
 
-	igt_fixture {
+	igt_fixture() {
 		buf_ops_destroy(bops);
 		drm_close_driver(xe);
 	}
