@@ -730,7 +730,7 @@ static void throughput_ratio(int pf_fd, int num_vfs, const struct subm_opts *opt
 	/* cleanup */
 	subm_set_fini(set);
 	set_vfs_scheduling_params(pf_fd, num_vfs, &(struct vf_sched_params){});
-	igt_sriov_disable_vfs(pf_fd);
+	xe_sriov_disable_vfs_restore_auto_provisioning(pf_fd);
 }
 
 /**
@@ -816,7 +816,7 @@ static void nonpreempt_engine_resets(int pf_fd, int num_vfs,
 	/* cleanup */
 	subm_set_fini(set);
 	set_vfs_scheduling_params(pf_fd, num_vfs, &(struct vf_sched_params){});
-	igt_sriov_disable_vfs(pf_fd);
+	xe_sriov_disable_vfs_restore_auto_provisioning(pf_fd);
 }
 
 static struct subm_opts subm_opts = {
@@ -884,12 +884,12 @@ static const char help_str[] =
 	"  --pt_us\tpreempt_timeout_us\n"
 	"  --inflight\tNumber of submissions kept in flight per VF (0=auto)\n";
 
-igt_main_args("", long_opts, help_str, subm_opts_handler, NULL)
+int igt_main_args("", long_opts, help_str, subm_opts_handler, NULL)
 {
 	int pf_fd;
 	bool autoprobe;
 
-	igt_fixture {
+	igt_fixture() {
 		pf_fd = drm_open_driver(DRIVER_XE);
 		igt_require(igt_sriov_is_pf(pf_fd));
 		igt_require(igt_sriov_get_enabled_vfs(pf_fd) == 0);
@@ -922,10 +922,10 @@ igt_main_args("", long_opts, help_str, subm_opts_handler, NULL)
 				nonpreempt_engine_resets(pf_fd, vf, &subm_opts);
 	}
 
-	igt_fixture {
+	igt_fixture() {
 		set_vfs_scheduling_params(pf_fd, igt_sriov_get_total_vfs(pf_fd),
 					  &(struct vf_sched_params){});
-		igt_sriov_disable_vfs(pf_fd);
+		xe_sriov_disable_vfs_restore_auto_provisioning(pf_fd);
 		/* abort to avoid execution of next tests with enabled VFs */
 		igt_abort_on_f(igt_sriov_get_enabled_vfs(pf_fd) > 0,
 			       "Failed to disable VF(s)");
