@@ -21,10 +21,21 @@
  * IN THE SOFTWARE.
  */
 
+/**
+ * TEST: kms atomic transition
+ * Category: Display
+ * Description: This is a stress test, to ensure that all combinations of
+ * 		atomic transitions work correctly. For i915/xe this will mainly be a
+ * 		stress test on watermark calculations.
+ * Driver requirement: i915, xe
+ * Mega feature: General Display Features
+ */
+
 #include "igt.h"
 #include "igt_rand.h"
 #include "drmtest.h"
 #include "sw_sync.h"
+#include "igt_sysfs.h"
 #include <errno.h>
 #include <pthread.h>
 #include <stdbool.h>
@@ -33,6 +44,67 @@
 #include <time.h>
 #include <poll.h>
 
+/**
+ * SUBTEST: plane-primary-toggle-with-vblank-wait
+ * Description: Check toggling of primary plane with vblank
+ *
+ * SUBTEST: plane-all-modeset-%s
+ * Description: Modeset test for all plane combinations %arg[1]
+ *
+ * arg[1]:
+ *
+ * @transition:                           default
+ * @transition-fencing:                   with fencing commit
+ */
+
+/**
+ * SUBTEST: plane-all-modeset-%s
+ * Description: Modeset test for all plane combinations %arg[1]
+ *
+ * arg[1]:
+ *
+ * @transition-fencing-internal-panels:   on internal panels with fencing commit
+ * @transition-internal-panels:           on internal panels
+ */
+
+/**
+ * SUBTEST: plane-all-%s
+ * Description: Transition test for all plane combinations %arg[1]
+ *
+ * arg[1]:
+ *
+ * @transition:                           default
+ * @transition-fencing:                   with fencing commit
+ * @transition-nonblocking:               with non-blocking commit
+ * @transition-nonblocking-fencing:       with non-blocking & fencing commit
+ */
+
+/**
+ * SUBTEST: plane-toggle-modeset-transition
+ * Description: Check toggling and modeset transition on plane
+ *
+ * SUBTEST: plane-use-after-nonblocking-%s
+ * Description: Transition test with non %arg[1] and make sure commit of disabled
+ *              plane has to complete before atomic commit on that plane
+ *
+ * arg[1]:
+ *
+ * @unbind:           blocking commit
+ * @unbind-fencing:   blocking commit with fencing
+ */
+
+/**
+ * SUBTEST: modeset-%s
+ * Description: Modeset transition tests for combinations of %arg[1]
+ *
+ * arg[1]:
+ *
+ * @transition:                     crtc enabled
+ * @transition-fencing:             crtc enabled with fencing commit
+ * @transition-nonblocking:         crtc enabled with nonblocking commit
+ * @transition-nonblocking-fencing: crtc enabled with nonblocking & fencing commit
+ */
+
 #ifndef DRM_CAP_CURSOR_WIDTH
 #define DRM_CAP_CURSOR_WIDTH 0x8
 #endif
@@ -47,9 +119,10 @@ struct plane_parms {
 
 typedef struct {
 	int drm_fd;
-	struct igt_fb fb, argb_fb, sprite_fb;
+	struct igt_fb fbs[2], argb_fb, sprite_fb;
 	igt_display_t display;
 	bool extended;
+	igt_pipe_crc_t *pipe_crcs[IGT_MAX_PIPES];
 } data_t;
 
 /* globals for fence support */
@@ -62,10 +135,15 @@ run_primary_test(data_t *data, enum pipe pipe, igt_output_t *output)
 {
 	drmModeModeInfo *mode;
 	igt_plane_t *primary;
-	igt_fb_t fb;
+	igt_fb_t *fb = &data->fbs[0];
 	int i, ret;
 	unsigned flags = DRM_MODE_ATOMIC_TEST_ONLY | DRM_MODE_ATOMIC_ALLOW_MODESET;
 
+	igt_display_reset(&data->display);
+
+	igt_info("Using (pipe %s + %s) to run the subtest.\n",
+		 kmstest_pipe_name(pipe), igt_output_name(output));
+
 	igt_output_set_pipe(output, pipe);
 	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
 
@@ -76,9 +154,9 @@ run_primary_test(data_t *data, enum pipe pipe, igt_output_t *output)
 	igt_skip_on_f(ret == -EINVAL, "Primary plane cannot be disabled separately from output\n");
 
 	igt_create_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
-		      DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE, &fb);
+		      DRM_FORMAT_XRGB8888, DRM_FORMAT_MOD_LINEAR, fb);
 
-	igt_plane_set_fb(primary, &fb);
+	igt_plane_set_fb(primary, fb);
 
 	for (i = 0; i < 4; i++) {
 		igt_display_commit2(&data->display, COMMIT_ATOMIC);
@@ -87,19 +165,15 @@ run_primary_test(data_t *data, enum pipe pipe, igt_output_t *output)
 			igt_wait_for_vblank(data->drm_fd,
 					data->display.pipes[pipe].crtc_offset);
 
-		igt_plane_set_fb(primary, (i & 1) ? &fb : NULL);
+		igt_plane_set_fb(primary, (i & 1) ? fb : NULL);
 		igt_display_commit2(&data->display, COMMIT_ATOMIC);
 
 		if (i & 1)
 			igt_wait_for_vblank(data->drm_fd,
 					data->display.pipes[pipe].crtc_offset);
 
-		igt_plane_set_fb(primary, (i & 1) ? NULL : &fb);
+		igt_plane_set_fb(primary, (i & 1) ? NULL : fb);
 	}
-
-	igt_plane_set_fb(primary, NULL);
-	igt_output_set_pipe(output, PIPE_NONE);
-	igt_remove_fb(data->drm_fd, &fb);
 }
 
 static void *fence_inc_thread(void *arg)
@@ -134,13 +208,13 @@ static bool skip_plane(data_t *data, igt_plane_t *plane)
 	if (data->extended)
 		return false;
 
-	if (!is_i915_device(data->drm_fd))
+	if (!is_intel_device(data->drm_fd))
 		return false;
 
 	if (plane->type == DRM_PLANE_TYPE_CURSOR)
 		return false;
 
-	if (intel_gen(intel_get_drm_devid(data->drm_fd)) < 11)
+	if (intel_display_ver(intel_get_drm_devid(data->drm_fd)) < 11)
 		return false;
 
 	/*
@@ -173,7 +247,7 @@ wm_setup_plane(data_t *data, enum pipe pipe,
 			continue;
 
 		if (!mask || !(parms[i].mask & mask)) {
-			if (plane->values[IGT_PLANE_FB_ID]) {
+			if (plane->values[IGT_PLANE_FB_ID] && plane->type != DRM_PLANE_TYPE_PRIMARY) {
 				igt_plane_set_fb(plane, NULL);
 				planes_set_up++;
 			}
@@ -233,7 +307,7 @@ static void set_sprite_wh(data_t *data, enum pipe pipe,
 	igt_remove_fb(data->drm_fd, sprite_fb);
 	igt_create_fb(data->drm_fd, w, h,
 		      alpha ? DRM_FORMAT_ARGB8888 : DRM_FORMAT_XRGB8888,
-		      LOCAL_DRM_FORMAT_MOD_NONE, sprite_fb);
+		      DRM_FORMAT_MOD_LINEAR, sprite_fb);
 }
 
 #define is_atomic_check_failure_errno(errno) \
@@ -282,7 +356,7 @@ static void setup_parms(data_t *data, enum pipe pipe,
 		} else {
 			if (!n_overlays)
 				alpha = igt_plane_has_format_mod(plane,
-					DRM_FORMAT_ARGB8888, LOCAL_DRM_FORMAT_MOD_NONE);
+					DRM_FORMAT_ARGB8888, DRM_FORMAT_MOD_LINEAR);
 			parms[i].fb = sprite_fb;
 			parms[i].mask = 1 << 2;
 
@@ -310,10 +384,10 @@ static void setup_parms(data_t *data, enum pipe pipe,
 	}
 
 	igt_create_fb(data->drm_fd, cursor_width, cursor_height,
-		      DRM_FORMAT_ARGB8888, LOCAL_DRM_FORMAT_MOD_NONE, argb_fb);
+		      DRM_FORMAT_ARGB8888, DRM_FORMAT_MOD_LINEAR, argb_fb);
 
 	igt_create_fb(data->drm_fd, cursor_width, cursor_height,
-		      DRM_FORMAT_ARGB8888, LOCAL_DRM_FORMAT_MOD_NONE, sprite_fb);
+		      DRM_FORMAT_ARGB8888, DRM_FORMAT_MOD_LINEAR, sprite_fb);
 
 	*iter_max = iter_mask + 1;
 	if (!n_overlays)
@@ -406,11 +480,11 @@ static void prepare_fencing(data_t *data, enum pipe pipe)
 	igt_require_sw_sync();
 
 	n_planes = data->display.pipes[pipe].n_planes;
-	timeline = calloc(sizeof(*timeline), n_planes);
+	timeline = calloc(n_planes, sizeof(*timeline));
 	igt_assert_f(timeline != NULL, "Failed to allocate memory for timelines\n");
-	thread = calloc(sizeof(*thread), n_planes);
+	thread = calloc(n_planes, sizeof(*thread));
 	igt_assert_f(thread != NULL, "Failed to allocate memory for thread\n");
-	seqno = calloc(sizeof(*seqno), n_planes);
+	seqno = calloc(n_planes, sizeof(*seqno));
 	igt_assert_f(seqno != NULL, "Failed to allocate memory for seqno\n");
 
 	for_each_plane_on_pipe(&data->display, pipe, plane)
@@ -447,7 +521,7 @@ static int fd_completed(int fd)
 	int ret;
 
 	ret = poll(&pfd, 1, 0);
-	igt_assert(ret >= 0);
+	igt_assert_lte(0, ret);
 	return ret;
 }
 
@@ -489,6 +563,9 @@ run_transition_test(data_t *data, enum pipe pipe, igt_output_t *output,
 	unsigned flags = 0;
 	int ret;
 
+	igt_info("Using (pipe %s + %s) to run the subtest.\n",
+		 kmstest_pipe_name(pipe), igt_output_name(output));
+
 	if (fencing)
 		prepare_fencing(data, pipe);
 	else
@@ -506,7 +583,7 @@ run_transition_test(data_t *data, enum pipe pipe, igt_output_t *output,
 	override_mode.flags ^= DRM_MODE_FLAG_PHSYNC | DRM_MODE_FLAG_NHSYNC;
 
 	igt_create_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
-		      DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE, &data->fb);
+		      DRM_FORMAT_XRGB8888, DRM_FORMAT_MOD_LINEAR, &data->fbs[0]);
 
 	igt_output_set_pipe(output, pipe);
 
@@ -520,9 +597,9 @@ run_transition_test(data_t *data, enum pipe pipe, igt_output_t *output,
 		igt_output_set_pipe(output, pipe);
 	}
 
-	igt_display_commit2(&data->display, COMMIT_ATOMIC);
+	setup_parms(data, pipe, mode, &data->fbs[0], &data->argb_fb, &data->sprite_fb, parms, &iter_max);
 
-	setup_parms(data, pipe, mode, &data->fb, &data->argb_fb, &data->sprite_fb, parms, &iter_max);
+	igt_display_commit2(&data->display, COMMIT_ATOMIC);
 
 	/*
 	 * In some configurations the tests may not run to completion with all
@@ -555,8 +632,8 @@ run_transition_test(data_t *data, enum pipe pipe, igt_output_t *output,
 			break;
 		}
 
-		if (!ret)
-			igt_skip("Cannot run tests without proper size sprite planes\n");
+		igt_skip_on_f(!ret,
+			      "Cannot run tests without proper size sprite planes\n");
 	}
 
 	igt_display_commit2(&data->display, COMMIT_ATOMIC);
@@ -662,7 +739,8 @@ static void test_cleanup(data_t *data, enum pipe pipe, igt_output_t *output, boo
 
 	igt_display_commit2(&data->display, COMMIT_ATOMIC);
 
-	igt_remove_fb(data->drm_fd, &data->fb);
+	igt_remove_fb(data->drm_fd, &data->fbs[0]);
+	igt_remove_fb(data->drm_fd, &data->fbs[1]);
 	igt_remove_fb(data->drm_fd, &data->argb_fb);
 	igt_remove_fb(data->drm_fd, &data->sprite_fb);
 }
@@ -677,7 +755,11 @@ static void commit_display(data_t *data, unsigned event_mask, bool nonblocking)
 	if (nonblocking)
 		flags |= DRM_MODE_ATOMIC_NONBLOCK;
 
-	igt_display_commit_atomic(&data->display, flags, NULL);
+	do {
+		ret = igt_display_try_commit_atomic(&data->display, flags, NULL);
+	} while (ret == -EBUSY);
+
+	igt_assert_eq(ret, 0);
 
 	igt_debug("Event mask: %x, waiting for %i events\n", event_mask, num_events);
 
@@ -705,15 +787,40 @@ static void commit_display(data_t *data, unsigned event_mask, bool nonblocking)
 	igt_reset_timeout();
 }
 
+static void unset_output_pipe(igt_display_t *display)
+{
+	int i;
+
+	for (i = 0; i < display->n_outputs; i++)
+		igt_output_set_pipe(&display->outputs[i], PIPE_NONE);
+}
+
 static unsigned set_combinations(data_t *data, unsigned mask, struct igt_fb *fb)
 {
 	igt_output_t *output;
 	enum pipe pipe;
 	unsigned event_mask = 0;
-	int i;
 
-	for (i = 0; i < data->display.n_outputs; i++)
-		igt_output_set_pipe(&data->display.outputs[i], PIPE_NONE);
+	unset_output_pipe(&data->display);
+
+	for_each_pipe(&data->display, pipe) {
+		igt_plane_t *plane = igt_pipe_get_plane_type(&data->display.pipes[pipe],
+			DRM_PLANE_TYPE_PRIMARY);
+
+		enum pipe old_pipe = plane->ref->pipe->pipe;
+
+		/*
+		 * If a plane is being shared by multiple pipes, we must disable the pipe that
+		 * currently is holding the plane
+		 */
+		if (old_pipe != pipe) {
+			igt_plane_t *old_plane = igt_pipe_get_plane_type(&data->display.pipes[old_pipe],
+				DRM_PLANE_TYPE_PRIMARY);
+
+			igt_plane_set_fb(old_plane, NULL);
+			igt_display_commit2(&data->display, COMMIT_ATOMIC);
+		}
+	}
 
 	for_each_pipe(&data->display, pipe) {
 		igt_plane_t *plane = igt_pipe_get_plane_type(&data->display.pipes[pipe],
@@ -735,8 +842,13 @@ static unsigned set_combinations(data_t *data, unsigned mask, struct igt_fb *fb)
 			if (output->pending_pipe != PIPE_NONE)
 				continue;
 
-			mode = igt_output_get_mode(output);
-			break;
+			igt_output_set_pipe(output, pipe);
+			if (intel_pipe_output_combo_valid(&data->display)) {
+				mode = igt_output_get_mode(output);
+				break;
+			} else {
+				igt_output_set_pipe(output, PIPE_NONE);
+			}
 		}
 
 		if (!mode)
@@ -783,16 +895,16 @@ static void collect_crcs_mask(igt_pipe_crc_t **pipe_crcs, unsigned mask, igt_crc
 
 static void run_modeset_tests(data_t *data, int howmany, bool nonblocking, bool fencing)
 {
-	struct igt_fb fbs[2];
-	int i, j = 0;
+	int i, j;
 	unsigned iter_max;
-	igt_pipe_crc_t *pipe_crcs[IGT_MAX_PIPES] = { 0 };
 	igt_output_t *output;
-	unsigned width = 0, height = 0;
+	uint16_t width = 0, height = 0;
+	enum pipe pipe_enum;
 
-	for (i = 0; i < data->display.n_outputs; i++)
-		igt_output_set_pipe(&data->display.outputs[i], PIPE_NONE);
+retry:
+	unset_output_pipe(&data->display);
 
+	j = 0;
 	for_each_connected_output(&data->display, output) {
 		drmModeModeInfo *mode = igt_output_get_mode(output);
 
@@ -801,9 +913,9 @@ static void run_modeset_tests(data_t *data, int howmany, bool nonblocking, bool
 	}
 
 	igt_create_pattern_fb(data->drm_fd, width, height,
-				   DRM_FORMAT_XRGB8888, 0, &fbs[0]);
+				   DRM_FORMAT_XRGB8888, DRM_FORMAT_MOD_LINEAR, &data->fbs[0]);
 	igt_create_color_pattern_fb(data->drm_fd, width, height,
-				    DRM_FORMAT_XRGB8888, 0, .5, .5, .5, &fbs[1]);
+				    DRM_FORMAT_XRGB8888, DRM_FORMAT_MOD_LINEAR, .5, .5, .5, &data->fbs[1]);
 
 	for_each_pipe(&data->display, i) {
 		igt_pipe_t *pipe = &data->display.pipes[i];
@@ -813,21 +925,31 @@ static void run_modeset_tests(data_t *data, int howmany, bool nonblocking, bool
 		/* count enable pipes to set max iteration */
 		j += 1;
 
-		if (is_i915_device(data->drm_fd))
-			pipe_crcs[i] = igt_pipe_crc_new(data->drm_fd, i, INTEL_PIPE_CRC_SOURCE_AUTO);
+		if (is_intel_device(data->drm_fd))
+			data->pipe_crcs[i] = igt_pipe_crc_new(data->drm_fd, i,
+							      IGT_PIPE_CRC_SOURCE_AUTO);
 
 		for_each_valid_output_on_pipe(&data->display, i, output) {
 			if (output->pending_pipe != PIPE_NONE)
 				continue;
 
 			igt_output_set_pipe(output, i);
-			mode = igt_output_get_mode(output);
-			break;
+			if (intel_pipe_output_combo_valid(&data->display)) {
+				mode = igt_output_get_mode(output);
+
+				igt_info("(pipe %s + %s), mode:",
+					 kmstest_pipe_name(i), igt_output_name(output));
+				kmstest_dump_mode(mode);
+
+				break;
+			} else {
+				igt_output_set_pipe(output, PIPE_NONE);
+			}
 		}
 
 		if (mode) {
-			igt_plane_set_fb(plane, &fbs[1]);
-			igt_fb_set_size(&fbs[1], plane, mode->hdisplay, mode->vdisplay);
+			igt_plane_set_fb(plane, &data->fbs[1]);
+			igt_fb_set_size(&data->fbs[1], plane, mode->hdisplay, mode->vdisplay);
 			igt_plane_set_size(plane, mode->hdisplay, mode->vdisplay);
 
 			if (fencing)
@@ -837,8 +959,20 @@ static void run_modeset_tests(data_t *data, int howmany, bool nonblocking, bool
 		}
 	}
 
-	iter_max = 1 << j;
+	iter_max = (j > 0) ? (j << 1) : 1;
 
+	if (igt_run_in_simulation() && iter_max > 1)
+		iter_max = iter_max >> 2;
+
+	if (igt_display_try_commit_atomic(&data->display,
+				DRM_MODE_ATOMIC_TEST_ONLY |
+				DRM_MODE_ATOMIC_ALLOW_MODESET,
+				NULL) != 0) {
+		bool found = igt_override_all_active_output_modes_to_fit_bw(&data->display);
+		igt_require_f(found, "No valid mode combo found.\n");
+
+		goto retry;
+	}
 	igt_display_commit2(&data->display, COMMIT_ATOMIC);
 
 	for (i = 0; i < iter_max; i++) {
@@ -848,13 +982,13 @@ static void run_modeset_tests(data_t *data, int howmany, bool nonblocking, bool
 		if (igt_hweight(i) > howmany)
 			continue;
 
-		event_mask = set_combinations(data, i, &fbs[0]);
+		event_mask = set_combinations(data, i, &data->fbs[0]);
 		if (!event_mask && i)
 			continue;
 
 		commit_display(data, event_mask, nonblocking);
 
-		collect_crcs_mask(pipe_crcs, i, crcs[0]);
+		collect_crcs_mask(data->pipe_crcs, i, crcs[0]);
 
 		for (j = iter_max - 1; j > i + 1; j--) {
 			if (igt_hweight(j) > howmany)
@@ -863,30 +997,30 @@ static void run_modeset_tests(data_t *data, int howmany, bool nonblocking, bool
 			if (igt_hweight(i) < howmany && igt_hweight(j) < howmany)
 				continue;
 
-			event_mask = set_combinations(data, j, &fbs[1]);
+			event_mask = set_combinations(data, j, &data->fbs[1]);
 			if (!event_mask)
 				continue;
 
 			commit_display(data, event_mask, nonblocking);
 
-			collect_crcs_mask(pipe_crcs, j, crcs[1]);
+			collect_crcs_mask(data->pipe_crcs, j, crcs[1]);
 
 			refresh_primaries(data, j);
 			commit_display(data, j, nonblocking);
-			collect_crcs_mask(pipe_crcs, j, crcs[2]);
+			collect_crcs_mask(data->pipe_crcs, j, crcs[2]);
 
-			event_mask = set_combinations(data, i, &fbs[0]);
+			event_mask = set_combinations(data, i, &data->fbs[0]);
 			if (!event_mask)
 				continue;
 
 			commit_display(data, event_mask, nonblocking);
-			collect_crcs_mask(pipe_crcs, i, crcs[3]);
+			collect_crcs_mask(data->pipe_crcs, i, crcs[3]);
 
 			refresh_primaries(data, i);
 			commit_display(data, i, nonblocking);
-			collect_crcs_mask(pipe_crcs, i, crcs[4]);
+			collect_crcs_mask(data->pipe_crcs, i, crcs[4]);
 
-			if (!is_i915_device(data->drm_fd))
+			if (!is_intel_device(data->drm_fd))
 				continue;
 
 			for (int k = 0; k < IGT_MAX_PIPES; k++) {
@@ -901,16 +1035,17 @@ static void run_modeset_tests(data_t *data, int howmany, bool nonblocking, bool
 		}
 	}
 
-	set_combinations(data, 0, NULL);
+	/* Cleanup */
+	unset_output_pipe(&data->display);
 	igt_display_commit2(&data->display, COMMIT_ATOMIC);
 
-	if (is_i915_device(data->drm_fd)) {
-		for_each_pipe(&data->display, i)
-			igt_pipe_crc_free(pipe_crcs[i]);
+	if (is_intel_device(data->drm_fd)) {
+		for_each_pipe(&data->display, pipe_enum)
+			igt_pipe_crc_free(data->pipe_crcs[pipe_enum]);
 	}
 
-	igt_remove_fb(data->drm_fd, &fbs[1]);
-	igt_remove_fb(data->drm_fd, &fbs[0]);
+	igt_remove_fb(data->drm_fd, &data->fbs[0]);
+	igt_remove_fb(data->drm_fd, &data->fbs[1]);
 }
 
 static void run_modeset_transition(data_t *data, int requested_outputs, bool nonblocking, bool fencing)
@@ -937,24 +1072,29 @@ static void run_modeset_transition(data_t *data, int requested_outputs, bool non
 		}
 	}
 
-	igt_require_f(num_outputs >= requested_outputs,
-		      "Should have at least %i outputs, found %i\n",
-		      requested_outputs, num_outputs);
+	if (num_outputs < requested_outputs) {
+		igt_debug("Should have at least %i outputs, found %i\n",
+			  requested_outputs, num_outputs);
+		return;
+	}
 
-	run_modeset_tests(data, requested_outputs, nonblocking, fencing);
+	igt_dynamic_f("%ix-outputs", requested_outputs)
+		run_modeset_tests(data, requested_outputs, nonblocking, fencing);
 }
 
-static bool output_is_internal_panel(igt_output_t *output)
+static bool pipe_output_combo_valid(igt_display_t *display,
+				    enum pipe pipe, igt_output_t *output)
 {
-	switch (output->config.connector->connector_type) {
-	case DRM_MODE_CONNECTOR_LVDS:
-	case DRM_MODE_CONNECTOR_eDP:
-	case DRM_MODE_CONNECTOR_DSI:
-	case DRM_MODE_CONNECTOR_DPI:
-		return true;
-	default:
-		return false;
-	}
+	bool ret = true;
+
+	igt_display_reset(display);
+
+	igt_output_set_pipe(output, pipe);
+	if (!intel_pipe_output_combo_valid(display))
+		ret = false;
+	igt_output_set_pipe(output, PIPE_NONE);
+
+	return ret;
 }
 
 static int opt_handler(int opt, int opt_index, void *_data)
@@ -980,14 +1120,62 @@ static const char help_str[] =
 
 static data_t data;
 
-igt_main_args("", long_opts, help_str, opt_handler, &data)
+int igt_main_args("", long_opts, help_str, opt_handler, &data)
 {
 	igt_output_t *output;
 	enum pipe pipe;
-	int i, count = 0;
+	struct {
+		const char *name;
+		enum transition_type type;
+		bool nonblocking;
+		bool fencing;
+		const char *desc;
+	} transition_tests[] = {
+		{ "plane-all-transition", TRANSITION_PLANES, false, false,
+		  "Transition test for all plane combinations" },
+		{ "plane-all-transition-fencing", TRANSITION_PLANES, false, true,
+		  "Transition test for all plane combinations with fencing commit" },
+		{ "plane-all-transition-nonblocking", TRANSITION_PLANES, true, false,
+		  "Transition test for all plane combinations with nonblocking commit" },
+		{ "plane-all-transition-nonblocking-fencing", TRANSITION_PLANES, true, true,
+		  "Transition test for all plane combinations with nonblocking and fencing commit" },
+		{ "plane-use-after-nonblocking-unbind", TRANSITION_AFTER_FREE, true, false,
+		  "Transition test with non blocking commit and make sure commit of disabled plane has "
+		       "to complete before atomic commit on that plane" },
+		{ "plane-use-after-nonblocking-unbind-fencing", TRANSITION_AFTER_FREE, true, true,
+		  "Transition test with non blocking and fencing commit and make sure commit of "
+		       "disabled plane has to complete before atomic commit on that plane" },
+		{ "plane-all-modeset-transition", TRANSITION_MODESET, false, false,
+		  "Modeset test for all plane combinations" },
+		{ "plane-all-modeset-transition-fencing", TRANSITION_MODESET, false, true,
+		  "Modeset test for all plane combinations with fencing commit" },
+		{ "plane-all-modeset-transition-internal-panels", TRANSITION_MODESET_FAST, false, false,
+		  "Modeset test for all plane combinations on internal panels" },
+		{ "plane-all-modeset-transition-fencing-internal-panels", TRANSITION_MODESET_FAST, false, true,
+		  "Modeset test for all plane combinations on internal panels with fencing commit" },
+		{ "plane-toggle-modeset-transition", TRANSITION_MODESET_DISABLE, false, false,
+		  "Check toggling and modeset transition on plane" },
+	};
+	struct {
+		const char *name;
+		bool nonblocking;
+		bool fencing;
+		const char *desc;
+	} modeset_tests[] = {
+		{ "modeset-transition", false, false,
+		  "Modeset transition tests for combinations of crtc enabled" },
+		{ "modeset-transition-fencing", false, true,
+		  "Modeset transition tests for combinations of crtc enabled with fencing commit" },
+		{ "modeset-transition-nonblocking", true, false,
+		  "Modeset transition tests for combinations of crtc enabled with nonblocking commit" },
+		{ "modeset-transition-nonblocking-fencing", true, true,
+		  "Modeset transition tests for combinations of crtc enabled with nonblocking and fencing commit" },
+	};
+	int i, j, count = 0;
 	int pipe_count = 0;
 
-	igt_fixture {
+	igt_fixture() {
+		unsigned int debug_mask_if_ci = DRM_UT_KMS;
 		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
 
 		kmstest_set_vt_graphics_mode();
@@ -999,215 +1187,88 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 
 		for_each_connected_output(&data.display, output)
 			count++;
+
+		igt_install_exit_handler(igt_drm_debug_mask_reset_exit_handler);
+		update_debug_mask_if_ci(debug_mask_if_ci);
 	}
 
 	igt_describe("Check toggling of primary plane with vblank");
-	igt_subtest("plane-primary-toggle-with-vblank-wait") {
-		for_each_pipe_with_valid_output(&data.display, pipe, output) {
-			if (pipe_count == 2 * count && !data.extended)
-				break;
-			pipe_count++;
-			run_primary_test(&data, pipe, output);
-
-		}
+	igt_subtest_with_dynamic("plane-primary-toggle-with-vblank-wait") {
 		pipe_count = 0;
-	}
 
-	igt_describe("Transition test for all plane combinations");
-	igt_subtest_with_dynamic("plane-all-transition") {
 		for_each_pipe_with_valid_output(&data.display, pipe, output) {
 			if (pipe_count == 2 * count && !data.extended)
 				break;
-			pipe_count++;
-			igt_dynamic_f("%s-pipe-%s", igt_output_name(output), kmstest_pipe_name(pipe))
-				run_transition_test(&data, pipe, output, TRANSITION_PLANES, false, false);
-			test_cleanup(&data, pipe, output, false);
-		}
-		pipe_count = 0;
-	}
 
-	igt_describe("Transition test for all plane combinations with fencing commit");
-	igt_subtest_with_dynamic("plane-all-transition-fencing") {
-		for_each_pipe_with_valid_output(&data.display, pipe, output) {
-			if (pipe_count == 2 * count && !data.extended)
-				break;
-			pipe_count++;
-			igt_dynamic_f("%s-pipe-%s", igt_output_name(output), kmstest_pipe_name(pipe))
-				run_transition_test(&data, pipe, output, TRANSITION_PLANES, false, true);
-			test_cleanup(&data, pipe, output, true);
-		}
-		pipe_count = 0;
-	}
-
-	igt_describe("Transition test for all plane combinations with nonblocking commit");
-	igt_subtest_with_dynamic("plane-all-transition-nonblocking") {
-		for_each_pipe_with_valid_output(&data.display, pipe, output) {
-			if (pipe_count == 2 * count && !data.extended)
-				break;
-			pipe_count++;
-			igt_dynamic_f("%s-pipe-%s", igt_output_name(output), kmstest_pipe_name(pipe))
-				run_transition_test(&data, pipe, output, TRANSITION_PLANES, true, false);
-			test_cleanup(&data, pipe, output, false);
-		}
-		pipe_count = 0;
-	}
-
-	igt_describe("Transition test for all plane combinations with nonblocking and fencing commit");
-	igt_subtest_with_dynamic("plane-all-transition-nonblocking-fencing") {
-		for_each_pipe_with_valid_output(&data.display, pipe, output) {
-			if (pipe_count == 2 * count && !data.extended)
-				break;
-			pipe_count++;
-			igt_dynamic_f("%s-pipe-%s", igt_output_name(output), kmstest_pipe_name(pipe))
-				run_transition_test(&data, pipe, output, TRANSITION_PLANES, true, true);
-			test_cleanup(&data, pipe, output, true);
-		}
-		pipe_count = 0;
-	}
-
-	igt_describe("Transition test with non blocking commit and make sure commit of disabled plane has "
-		       "to complete before atomic commit on that plane");
-	igt_subtest_with_dynamic("plane-use-after-nonblocking-unbind") {
-		for_each_pipe_with_valid_output(&data.display, pipe, output) {
-			if (pipe_count == 2 * count && !data.extended)
-				break;
-			pipe_count++;
-			igt_dynamic_f("%s-pipe-%s", igt_output_name(output), kmstest_pipe_name(pipe))
-				run_transition_test(&data, pipe, output, TRANSITION_AFTER_FREE, true, false);
-			test_cleanup(&data, pipe, output, false);
-		}
-		pipe_count = 0;
-	}
-
-	igt_describe("Transition test with non blocking and fencing commit and make sure commit of "
-		       "disabled plane has to complete before atomic commit on that plane");
-	igt_subtest_with_dynamic("plane-use-after-nonblocking-unbind-fencing") {
-		for_each_pipe_with_valid_output(&data.display, pipe, output) {
-			if (pipe_count == 2 * count && !data.extended)
-				break;
-			pipe_count++;
-			igt_dynamic_f("%s-pipe-%s", igt_output_name(output), kmstest_pipe_name(pipe))
-				run_transition_test(&data, pipe, output, TRANSITION_AFTER_FREE, true, true);
-			test_cleanup(&data, pipe, output, true);
-		}
-		pipe_count = 0;
-	}
-
-	/*
-	 * Test modeset cases on internal panels separately with a reduced
-	 * number of combinations, to avoid long runtimes due to modesets on
-	 * panels with long power cycle delays.
-	 */
-	igt_describe("Modeset test for all plane combinations");
-	igt_subtest_with_dynamic("plane-all-modeset-transition") {
-		for_each_pipe_with_valid_output(&data.display, pipe, output) {
-			if (pipe_count == 2 * count && !data.extended)
-				break;
-			pipe_count++;
-			if (output_is_internal_panel(output))
+			if (!pipe_output_combo_valid(&data.display, pipe, output))
 				continue;
 
-			igt_dynamic_f("%s-pipe-%s", igt_output_name(output), kmstest_pipe_name(pipe))
-				run_transition_test(&data, pipe, output, TRANSITION_MODESET, false, false);
+			pipe_count++;
+			igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output))
+				run_primary_test(&data, pipe, output);
 			test_cleanup(&data, pipe, output, false);
 		}
-		pipe_count = 0;
 	}
 
-	igt_describe("Modeset test for all plane combinations with fencing commit");
-	igt_subtest_with_dynamic("plane-all-modeset-transition-fencing") {
-		for_each_pipe_with_valid_output(&data.display, pipe, output) {
-			if (pipe_count == 2 * count && !data.extended)
-				break;
-			pipe_count++;
-			if (output_is_internal_panel(output))
-				continue;
+	for (i = 0; i < ARRAY_SIZE(transition_tests); i++) {
+		if (strstr(transition_tests[i].name, "modeset"))
+			update_debug_mask_if_ci(DRM_UT_DRIVER);
 
-			igt_dynamic_f("%s-pipe-%s", igt_output_name(output), kmstest_pipe_name(pipe))
-				run_transition_test(&data, pipe, output, TRANSITION_MODESET, false, true);
-			test_cleanup(&data, pipe, output, true);
-		}
-		pipe_count = 0;
-	}
+		igt_describe(transition_tests[i].desc);
+		igt_subtest_with_dynamic_f("%s", transition_tests[i].name) {
+			pipe_count = 0;
 
-	igt_describe("Modeset test for all plane combinations on internal panels");
-	igt_subtest_with_dynamic("plane-all-modeset-transition-internal-panels") {
-		for_each_pipe_with_valid_output(&data.display, pipe, output) {
-			if (pipe_count == 2 * count && !data.extended)
-				break;
-			pipe_count++;
-			if (!output_is_internal_panel(output))
-				continue;
+			for_each_pipe_with_valid_output(&data.display, pipe, output) {
+				/*
+				 * Test modeset cases on internal panels separately with a reduced
+				 * number of combinations, to avoid long runtimes due to modesets on
+				 * panels with long power cycle delays.
+				 */
+				if ((transition_tests[i].type == TRANSITION_MODESET) &&
+				    output_is_internal_panel(output))
+					continue;
 
-			igt_dynamic_f("%s-pipe-%s", igt_output_name(output), kmstest_pipe_name(pipe))
-				run_transition_test(&data, pipe, output, TRANSITION_MODESET_FAST, false, false);
-			test_cleanup(&data, pipe, output, false);
-		}
-		pipe_count = 0;
-	}
+				if ((transition_tests[i].type == TRANSITION_MODESET_FAST) &&
+				    !output_is_internal_panel(output))
+					continue;
 
-	igt_describe("Modeset test for all plane combinations on internal panels with fencing commit");
-	igt_subtest_with_dynamic("plane-all-modeset-transition-fencing-internal-panels") {
-		for_each_pipe_with_valid_output(&data.display, pipe, output) {
-			if (pipe_count == 2 * count && !data.extended)
-				break;
-			pipe_count++;
-			if (!output_is_internal_panel(output))
-				continue;
+				if (pipe_count == 2 * count && !data.extended)
+					break;
 
-			igt_dynamic_f("%s-pipe-%s", igt_output_name(output), kmstest_pipe_name(pipe))
-				run_transition_test(&data, pipe, output, TRANSITION_MODESET_FAST, false, true);
-			test_cleanup(&data, pipe, output, true);
-		}
-		pipe_count = 0;
-	}
+				if (!pipe_output_combo_valid(&data.display, pipe, output))
+					continue;
 
-	igt_describe("Check toggling and modeset transition on plane");
-	igt_subtest("plane-toggle-modeset-transition") {
-		for_each_pipe_with_valid_output(&data.display, pipe, output) {
-			if (pipe_count == 2 * count && !data.extended)
-				break;
-			pipe_count++;
-			run_transition_test(&data, pipe, output, TRANSITION_MODESET_DISABLE, false, false);
-			test_cleanup(&data, pipe, output, false);
-		}
-		pipe_count = 0;
-	}
+				pipe_count++;
+				igt_dynamic_f("pipe-%s-%s",
+					      kmstest_pipe_name(pipe),
+					      igt_output_name(output))
+					run_transition_test(&data, pipe, output,
+							    transition_tests[i].type,
+							    transition_tests[i].nonblocking,
+							    transition_tests[i].fencing);
 
-	igt_describe("Modeset transition tests for combinations of crtc enabled");
-	igt_subtest_with_dynamic("modeset-transition") {
-		for (i = 1; i <= count; i++) {
-			igt_dynamic_f("%ix-outputs", i)
-				run_modeset_transition(&data, i, false, false);
+				test_cleanup(&data, pipe, output,
+					     transition_tests[i].fencing);
+			}
 		}
 	}
 
-	igt_describe("Modeset transition tests for combinations of crtc enabled with nonblocking commit");
-	igt_subtest_with_dynamic("modeset-transition-nonblocking") {
-		for (i = 1; i <= count; i++) {
-			igt_dynamic_f("%ix-outputs", i)
-				run_modeset_transition(&data, i, true, false);
+	for (i = 0; i < ARRAY_SIZE(modeset_tests); i++) {
+		if (igt_get_connected_output_count(&data.display) > 2)
+			update_debug_mask_if_ci(DRM_UT_DRIVER);
+
+		igt_describe_f("%s", modeset_tests[i].desc);
+		igt_subtest_with_dynamic_f("%s", modeset_tests[i].name) {
+			for (j = 1; j <= count; j++) {
+				run_modeset_transition(&data, j,
+						       modeset_tests[i].nonblocking,
+						       modeset_tests[i].fencing);
+			}
 		}
 	}
 
-	igt_describe("Modeset transition tests for combinations of crtc enabled with fencing commit");
-	igt_subtest_with_dynamic("modeset-transition-fencing") {
-		for (i = 1; i <= count; i++) {
-			igt_dynamic_f("%ix-outputs", i)
-				run_modeset_transition(&data, i, false, true);
-		}
-	}
-
-	igt_describe("Modeset transition tests for combinations of crtc enabled with nonblocking &"
-		       " fencing commit");
-	igt_subtest_with_dynamic("modeset-transition-nonblocking-fencing") {
-		for (i = 1; i <= count; i++) {
-			igt_dynamic_f("%ix-outputs", i)
-				run_modeset_transition(&data, i, true, true);
-		}
-	}
-
-	igt_fixture {
+	igt_fixture() {
 		igt_display_fini(&data.display);
+		drm_close_driver(data.drm_fd);
 	}
 }
