@@ -22,9 +22,24 @@
  *
  */
 
+/**
+ * TEST: kms concurrent
+ * Category: Display
+ * Description: Test atomic mode setting concurrently with multiple planes and
+ *              screen resolution
+ * Driver requirement: i915, xe
+ * Mega feature: General Display Features
+ */
+
 #include "igt.h"
 #include "drmtest.h"
 
+/**
+ * SUBTEST: multi-plane-atomic-lowres
+ * Description: Test atomic mode setting concurrently with multiple planes and
+ *              screen resolution.
+ */
+
 IGT_TEST_DESCRIPTION("Test atomic mode setting concurrently with multiple planes and screen resolution");
 
 #define SIZE_PLANE      256
@@ -316,23 +331,25 @@ run_test(data_t *data, enum pipe pipe, igt_output_t *output)
 }
 
 static void
-run_tests_for_pipe(data_t *data, enum pipe pipe)
+run_tests_for_pipe(data_t *data)
 {
 	igt_output_t *output;
-
-	igt_fixture {
-		igt_require_pipe(&data->display, pipe);
-		igt_require(data->display.pipes[pipe].n_planes > 0);
-
-		igt_display_require_output(&data->display);
-	}
+	enum pipe pipe;
 
 	igt_describe("Test atomic mode setting concurrently with multiple planes and screen "
 		     "resolution.");
-	igt_subtest_with_dynamic_f("pipe-%s", kmstest_pipe_name(pipe)) {
-		for_each_valid_output_on_pipe(&data->display, pipe, output)
-			igt_dynamic_f("%s", igt_output_name(output))
+	igt_subtest_with_dynamic("multi-plane-atomic-lowres") {
+		for_each_valid_output_on_pipe(&data->display, pipe, output) {
+			igt_display_reset(&data->display);
+
+			igt_output_set_pipe(output, pipe);
+			if (!intel_pipe_output_combo_valid(&data->display))
+				continue;
+
+			igt_require(data->display.pipes[pipe].n_planes > 0);
+			igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output))
 				run_test(data, pipe, output);
+		}
 	}
 }
 
@@ -370,28 +387,25 @@ struct option long_options[] = {
 
 static data_t data;
 
-igt_main_args("", long_options, help_str, opt_handler, NULL)
+int igt_main_args("", long_options, help_str, opt_handler, NULL)
 {
-	enum pipe pipe;
-
-	igt_fixture {
+	igt_fixture() {
 		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
 		kmstest_set_vt_graphics_mode();
 		igt_display_require(&data.display, data.drm_fd);
 		igt_require(data.display.is_atomic);
-		if (is_i915_device(data.drm_fd))
-			intel_allocator_multiprocess_start();
+		igt_display_require_output(&data.display);
+		if (is_intel_device(data.drm_fd))
+			igt_require_f(intel_allocator_multiprocess_start(),
+				      "Allocator failure, no SYSV IPC\n");
 	}
 
-	for_each_pipe_static(pipe) {
-		igt_subtest_group
-			run_tests_for_pipe(&data, pipe);
-	}
+	run_tests_for_pipe(&data);
 
-	igt_fixture {
-		if (is_i915_device(data.drm_fd))
+	igt_fixture() {
+		if (is_intel_device(data.drm_fd))
 			intel_allocator_multiprocess_stop();
 		igt_display_fini(&data.display);
-		close(data.drm_fd);
+		drm_close_driver(data.drm_fd);
 	}
 }
