@@ -26,9 +26,7 @@
  * Category: Display
  * Description: Tests for validating modeset, dpms and pageflips
  * Driver requirement: i915, xe
- * Functionality: vblank
  * Mega feature: General Display Features
- * Test category: functionality test
  */
 
 #include "config.h"
@@ -62,13 +60,11 @@
  * SUBTEST: %s
  * Description: %arg[1] test to validate pageflips with available fences
  * Driver requirement: i915
- * Functionality: gtt, vblank
  *
  * SUBTEST: 2x-%s
  * Description: %arg[1] test to validate pageflips along with available fences
  *              on a pair of connected displays
  * Driver requirement: i915
- * Functionality: gtt, vblank
  *
  * arg[1]:
  *
@@ -79,7 +75,6 @@
 /**
  * SUBTEST: dpms-off-%s
  * Description: %arg[1] test to validate pageflips by disabling other connectors usng dpms
- * Functionality: dpms, vblank
  *
  * arg[1]:
  *
@@ -90,7 +85,6 @@
 /**
  * SUBTEST: %s
  * Description: %arg[1] test to validate pageflips with large BO in size
- * Functionality: kms_gem_interop, vblank
  *
  * arg[1]:
  *
@@ -101,11 +95,9 @@
 /**
  * SUBTEST: %s
  * Description: Basic test to validate %arg[1]
- * Functionality: hang, vblank
  *
  * SUBTEST: 2x-%s
  * Description: Test to validate %arg[1] on a pair of connected displays
- * Functionality: hang, vblank
  *
  * arg[1]:
  *
@@ -141,12 +133,10 @@
 /**
  * SUBTEST: %s
  * Description: %arg[1] test to validate pageflips with suspend cycle
- * Functionality: suspend, vblank
  *
  * SUBTEST: 2x-%s
  * Description: %arg[1] test to validate pageflips with suspend cycle on a pair
  *              of connected displays
- * Functionality: suspend, vblank
  *
  * arg[1]:
  *
@@ -157,11 +147,9 @@
 /**
  * SUBTEST: %s
  * Description: Basic test to validate %arg[1]
- * Functionality: dpms, vblank
  *
  * SUBTEST: 2x-%s
  * Description: Basic test to validate %arg[1] on a pair of connected displays
- * Functionality: dpms, vblank
  *
  * SUBTEST: %s-interruptible
  * Description: Basic test to validate %arg[1]
@@ -176,13 +164,13 @@
  * @single-buffer-flip-vs-dpms-off-vs-modeset: pageflip of same buffer along with
  *                                             the modeset and dpms off
  * @dpms-vs-vblank-race:                       vblank along with the dpms & modeset
+ * @flip-vs-dpms-on-nop:                       pageflip and issue nop DPMS ON
  */
 
 /**
  * SUBTEST: 2x-flip-vs-dpms
  * Description: Basic test to validate pageflip along with dpms on a pair of
  *              connected displays
- * Functionality: dpms, vblank
  *
  * SUBTEST: 2x-%s
  * Description: Basic test to validate %arg[1] on a pair of connected displays
@@ -228,7 +216,6 @@
  *
  * SUBTEST: basic-flip-vs-dpms
  * Description: Basic test to valide pageflip with dpms
- * Functionality: dpms, vblank
  *
  * SUBTEST: basic-flip-vs-%s
  * Description: Basic test to valide pageflip with %arg[1]
@@ -240,6 +227,7 @@
  */
 
 #define TEST_DPMS		(1 << 0)
+#define TEST_DPMS_ON_NOP	(1 << 1)
 
 #define TEST_PAN		(1 << 3)
 #define TEST_MODESET		(1 << 4)
@@ -294,6 +282,8 @@ static drmModeConnector *last_connector;
 
 uint32_t *fb_ptr;
 
+static igt_display_t display;
+
 struct type_name {
 	int type;
 	const char *name;
@@ -319,6 +309,7 @@ struct event_state {
 	unsigned int current_seq;		/* kernel reported seq. num */
 
 	int count;				/* # of events of this type */
+	int err_frames;				/* # of unexpected events */
 
 	/* Step between the current and next 'target' sequence number. */
 	int seq_step;
@@ -330,7 +321,19 @@ static bool should_skip_ts_checks(void) {
 	 * timestamp to drift with a relatively larger standard deviation over a large sample.
 	 * As it's a known issue, skip any Timestamp or Sequence checks for MTK drivers.
 	 */
-	return is_mtk_device(drm_fd);
+	if (is_mtk_device(drm_fd))
+		return true;
+
+	/*
+	 * In simulation environments, hardware behavior may not accurately reflect real-world
+	 * timing characteristics. To avoid false negatives in tests due to simulated timing
+	 * artifacts, skip timestamp and sequence checks when the INTEL_SIMULATION environment
+	 * variable is set to a truthy value.
+	 */
+	if (igt_run_in_simulation())
+		return true;
+
+	return false;
 }
 
 static bool vblank_dependence(int flags)
@@ -687,7 +690,7 @@ static void vblank_handler(int fd, unsigned int frame, unsigned int sec,
 	fixup_premature_vblank_ts(o, &o->vblank_state);
 }
 
-static bool check_state(const struct test_output *o, const struct event_state *es)
+static bool check_state(const struct test_output *o, struct event_state *es)
 {
 	struct timeval diff;
 
@@ -716,7 +719,8 @@ static bool check_state(const struct test_output *o, const struct event_state *e
 	    es->current_seq - (es->last_seq + o->seq_step) > 1UL << 23) {
 		igt_debug("unexpected %s seq %u, should be >= %u\n",
 			  es->name, es->current_seq, es->last_seq + o->seq_step);
-		return false;
+		es->err_frames++;
+		return true;
 	}
 
 	if (o->flags & TEST_CHECK_TS) {
@@ -737,16 +741,16 @@ static bool check_state(const struct test_output *o, const struct event_state *e
 				  es->name, timeval_float(&es->last_ts), es->last_seq,
 				  timeval_float(&es->current_ts), es->current_seq,
 				  elapsed, expected);
-
-			return false;
+			es->err_frames++;
+			return true;
 		}
 
 		if (es->current_seq != es->last_seq + o->seq_step) {
 			igt_debug("unexpected %s seq %u, expected %u\n",
 				  es->name, es->current_seq,
 				  es->last_seq + o->seq_step);
-
-			return false;
+			es->err_frames++;
+			return true;
 		}
 	}
 
@@ -957,7 +961,7 @@ static bool run_test_step(struct test_output *o, unsigned int *events)
 	if (o->flags & TEST_MODESET)
 		igt_assert(set_mode(o, o->fb_ids[o->current_fb_id], 0, 0) == 0);
 
-	if (o->flags & TEST_DPMS)
+	if (o->flags & (TEST_DPMS | TEST_DPMS_ON_NOP))
 		set_dpms(o, DRM_MODE_DPMS_ON);
 
 	if (o->flags & TEST_VBLANK_RACE) {
@@ -1236,6 +1240,7 @@ static bool check_final_state(const struct test_output *o,
 			      const struct event_state *es,
 			      unsigned int elapsed)
 {
+	int threshold = 85;
 	igt_assert_f(es->count > 0,
 		     "no %s event received\n", es->name);
 
@@ -1243,17 +1248,25 @@ static bool check_final_state(const struct test_output *o,
 	 * those use some funny fake timings behind userspace's back. */
 	if (o->flags & TEST_CHECK_TS) {
 		int count = es->count * o->seq_step;
-		unsigned int min = actual_frame_time(o) * (count - 1);
-		unsigned int max = actual_frame_time(o) * (count + 1);
+		int error_count = es->err_frames * o->seq_step;
+		int expected = elapsed / actual_frame_time(o);
+		float pass_rate = ((float)(count - error_count) / count) * 100;
 
-		igt_debug("expected %d, counted %d, encoder type %d\n",
-			  (int)(elapsed / actual_frame_time(o)), count,
-			  o->kencoder[0]->encoder_type);
-		if (elapsed < min || elapsed > max) {
-			igt_debug("dropped frames, expected %d, counted %d, encoder type %d\n",
-				  (int)(elapsed / actual_frame_time(o)), count,
-				  o->kencoder[0]->encoder_type);
+		if ((1000000.0/actual_frame_time(o)) > 120)
+			threshold = 75;
 
+		igt_info("Event %s: expected %d, counted %d, passrate = %.2f%%, encoder type %d\n",
+			 es->name, expected, count, pass_rate, o->kencoder[0]->encoder_type);
+
+		/*
+		 * TODO: Review the use of the hardcoded threshold (85/75). This value is
+		 * currently a placeholder for the acceptable pass rate. In the future,
+		 * we should either justify this value or refine the logic to skip
+		 * frames near the evasion time.
+		 */
+		if (pass_rate < threshold) {
+			igt_debug("dropped frames, expected %d, counted %d, passrate = %.2f%%, encoder type %d\n",
+				  expected, count, pass_rate, o->kencoder[0]->encoder_type);
 			return false;
 		}
 	}
@@ -1545,7 +1558,15 @@ restart:
 	if (o->flags & TEST_PAN)
 		o->fb_width *= 2;
 
-	modifier = DRM_FORMAT_MOD_LINEAR;
+	if (igt_display_has_format_mod(&display, DRM_FORMAT_XRGB8888,
+				       I915_FORMAT_MOD_4_TILED))
+		modifier = I915_FORMAT_MOD_4_TILED;
+	else if (igt_display_has_format_mod(&display, DRM_FORMAT_XRGB8888,
+					    I915_FORMAT_MOD_X_TILED))
+		modifier = I915_FORMAT_MOD_X_TILED;
+	else
+		modifier = DRM_FORMAT_MOD_LINEAR;
+
 	if (o->flags & TEST_FENCE_STRESS)
 		modifier = I915_FORMAT_MOD_X_TILED;
 
@@ -1558,6 +1579,8 @@ restart:
 			igt_require(bo_size < gem_global_aperture_size(drm_fd));
 		else
 			igt_require(bo_size < (1ULL << xe_va_bits(drm_fd)));
+
+		igt_suspend_signal_helper();
 	}
 
 	o->fb_ids[0] = igt_create_fb(drm_fd, o->fb_width, o->fb_height,
@@ -1569,6 +1592,9 @@ restart:
 						  IGT_COLOR_YCBCR_LIMITED_RANGE,
 						  &o->fb_info[1], bo_size, 0);
 
+	if (o->flags & TEST_BO_TOOBIG)
+		igt_resume_signal_helper();
+
 	igt_assert(o->fb_ids[0]);
 	igt_assert(o->fb_ids[1]);
 
@@ -1632,7 +1658,16 @@ retry:
 
 	/* quiescent the hw a bit so ensure we don't miss a single frame */
 	if (o->flags & TEST_CHECK_TS && !calibrate_ts(o, crtc_idxs[0])) {
-		igt_assert(!retried && needs_retry_after_link_reset(mon));
+		igt_assert(!retried);
+
+		/*
+		 * FIXME: Retried logic is currently breaking due to an HPD
+		 * (Hot Plug Detect) issue. Temporarily removing this from
+		 * the assertion. This needs to be debugged separately.
+		 * Revert this patch once the HPD issue is resolved.
+		 */
+		if (!needs_retry_after_link_reset(mon))
+			igt_debug("Retrying without a hotplug event\n");
 
 		retried = true;
 
@@ -1671,7 +1706,16 @@ retry:
 		state_ok &= check_final_state(o, &o->vblank_state, elapsed);
 
 	if (!state_ok) {
-		igt_assert(!retried && needs_retry_after_link_reset(mon));
+		igt_assert(!retried);
+
+		/*
+		 * FIXME: Retried logic is currently breaking due to an HPD
+		 * (Hot Plug Detect) issue. Temporarily removing this from
+		 * the assertion. This needs to be debugged separately.
+		 * Revert this patch once the HPD issue is resolved.
+		 */
+		if (!needs_retry_after_link_reset(mon))
+			igt_debug("Retrying without a hotplug event\n");
 
 		retried = true;
 
@@ -1756,11 +1800,11 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 				 o->kconnector[i - 1]->connector_type_id);
 
 		if (((igt_check_force_joiner_status(drm_fd, conn_name) ||
-		      igt_bigjoiner_possible(&o->kmode[i], max_dotclock)) &&
+		      igt_bigjoiner_possible(drm_fd, &o->kmode[i], max_dotclock)) &&
 		     ((crtc_idxs[i] >= (total_crtcs - 1)) ||
 		      ((i < (crtc_count - 1)) && (abs(crtc_idxs[i + 1] - crtc_idxs[i]) <= 1)))) ||
 		    ((i > 0) && (igt_check_force_joiner_status(drm_fd, prev_conn_name) ||
-				 igt_bigjoiner_possible(&o->kmode[i - 1], max_dotclock)) &&
+				 igt_bigjoiner_possible(drm_fd, &o->kmode[i - 1], max_dotclock)) &&
 		     (abs(crtc_idxs[i] - crtc_idxs[i - 1]) <= 1))) {
 
 			igt_debug("Combo: %s is not possible with selected mode(s).\n", test_name);
@@ -1896,7 +1940,7 @@ static void run_pair(int duration, int flags)
 
 	/* If we have fewer than 2 connected outputs then we won't have any
 	 * configuration at all. So skip in that case. */
-	igt_require_f(modes, "At least two displays required\n");
+	igt_require_f(modes, "At least two displays with same modes are required\n");
 
 	if (duration) {
 		duration = duration * 1000 / modes;
@@ -1998,7 +2042,7 @@ static int opt_handler(int opt, int opt_index, void *data)
 const char *help_str =
 	"  -e \tRun on all pipes. (By default subtests will run on two pipes)\n";
 
-igt_main_args("e", NULL, help_str, opt_handler, NULL)
+int igt_main_args("e", NULL, help_str, opt_handler, NULL)
 {
 	struct {
 		int duration;
@@ -2020,6 +2064,7 @@ igt_main_args("e", NULL, help_str, opt_handler, NULL)
 			"plain-flip-fb-recreate" },
 		{ 30, TEST_FLIP | TEST_RMFB | TEST_MODESET , "flip-vs-rmfb" },
 		{ 2, TEST_FLIP | TEST_DPMS | TEST_EINVAL | TEST_BASIC, "flip-vs-dpms" },
+		{ 2, TEST_FLIP | TEST_DPMS_ON_NOP | TEST_CHECK_TS, "flip-vs-dpms-on-nop" },
 		{ 30,  TEST_FLIP | TEST_PAN, "flip-vs-panning" },
 		{ 2, TEST_FLIP | TEST_MODESET | TEST_EINVAL | TEST_BASIC, "flip-vs-modeset" },
 		{ 30,  TEST_FLIP | TEST_VBLANK_EXPIRED_SEQ,
@@ -2045,10 +2090,9 @@ igt_main_args("e", NULL, help_str, opt_handler, NULL)
 		{ 0, TEST_BO_TOOBIG | TEST_NO_2X_OUTPUT, "bo-too-big" },
 		{ 10, TEST_FLIP | TEST_SUSPEND, "flip-vs-suspend" },
 	};
-	igt_display_t display;
 	int i;
 
-	igt_fixture {
+	igt_fixture() {
 		drm_fd = drm_open_driver_master(DRIVER_ANY);
 
 		igt_display_require(&display, drm_fd);
@@ -2123,7 +2167,7 @@ igt_main_args("e", NULL, help_str, opt_handler, NULL)
 	}
 	igt_stop_signal_helper();
 
-	igt_fixture {
+	igt_fixture() {
 		igt_display_fini(&display);
 		drm_close_driver(drm_fd);
 	}
