@@ -27,8 +27,12 @@
  * Category: Display
  * Description: Make sure all modesets are rejected when the requested mode is
  *              invalid
+ * Driver requirement: i915, xe
+ * Mega feature: General Display Features
  */
+
 #include "igt.h"
+#include "xe/xe_query.h"
 #include <limits.h>
 #include <stdbool.h>
 
@@ -36,10 +40,6 @@
  * SUBTEST: %s
  * Description: Make sure all modesets are rejected when the requested mode
  *              (%arg[1]) is invalid
- * Driver requirement: i915, xe
- * Functionality: kms_core
- * Mega feature: General Display Features
- * Test category: functionality test
  *
  * arg[1]:
  *
@@ -55,6 +55,7 @@
  * @zero-clock:       Clock as zero
  * @zero-hdisplay:    hdisplay as zero
  * @zero-vdisplay:    vdisplay as zero
+ * @overflow-vrefresh: vrefresh calculation overflow
  */
 
 IGT_TEST_DESCRIPTION("Make sure all modesets are rejected when the requested mode is invalid");
@@ -98,6 +99,28 @@ can_bigjoiner(data_t *data)
 	return false;
 }
 
+static bool
+can_ultrajoiner(data_t *data)
+{
+	bool ultra_joiner_supported = false;
+	bool is_dgfx;
+	int display_ver;
+
+	is_dgfx = is_xe_device(data->drm_fd) ? xe_has_vram(data->drm_fd) :
+					       gem_has_lmem(data->drm_fd);
+	display_ver = intel_display_ver(intel_get_drm_devid(data->drm_fd));
+
+	if ((is_dgfx && display_ver == 14) || display_ver > 14)
+		ultra_joiner_supported = true;
+
+	if (ultra_joiner_supported) {
+		igt_debug("Platform supports ultrajoiner\n");
+		return true;
+	}
+
+	return false;
+}
+
 static bool
 adjust_mode_clock_too_high(data_t *data, drmModeModeInfoPtr mode)
 {
@@ -127,6 +150,12 @@ adjust_mode_clock_too_high(data_t *data, drmModeModeInfoPtr mode)
 		max_dotclock *= 2;
 	}
 
+	if (can_ultrajoiner(data)) {
+		igt_info("Platform supports ultrajoiner with %s\n",
+			 data->output->name);
+		max_dotclock *= 4;
+	}
+
 	mode->clock = max_dotclock + 1;
 
 	return true;
@@ -209,6 +238,20 @@ adjust_mode_bad_vtotal(data_t *data, drmModeModeInfoPtr mode)
 	return true;
 }
 
+static bool
+adjust_mode_overflow_vrefresh(data_t *data, drmModeModeInfoPtr mode)
+{
+	/*
+	 * htotal * vtotal * vscan == 2^32
+	 * overflow during vrefresh calculation
+	 */
+	mode->htotal = 32768;
+	mode->vtotal = 32768;
+	mode->vscan = 4;
+
+	return true;
+}
+
 static void
 test_output(data_t *data)
 {
@@ -284,17 +327,20 @@ static const struct {
 	{ .name = "bad-vtotal",
 	  .adjust_mode = adjust_mode_bad_vtotal,
 	},
+	{ .name = "overflow-vrefresh",
+	  .adjust_mode = adjust_mode_overflow_vrefresh,
+	},
 };
 
 static data_t data;
 
-igt_main
+int igt_main()
 {
 
 	enum pipe pipe;
 	igt_output_t *output;
 
-	igt_fixture {
+	igt_fixture() {
 		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
 
 		kmstest_set_vt_graphics_mode();
@@ -314,10 +360,10 @@ igt_main
 				igt_display_reset(&data.display);
 
 				igt_output_set_pipe(output, pipe);
-				if (!i915_pipe_output_combo_valid(&data.display))
+				if (!intel_pipe_output_combo_valid(&data.display))
 					continue;
 
-				igt_dynamic_f("%s-pipe-%s", igt_output_name(output), kmstest_pipe_name(pipe)) {
+				igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), igt_output_name(output)) {
 					data.output = output;
 					data.pipe = pipe;
 					data.adjust_mode = subtests[i].adjust_mode;
@@ -327,7 +373,7 @@ igt_main
 		}
 	}
 
-	igt_fixture {
+	igt_fixture() {
 		igt_display_fini(&data.display);
 		igt_reset_connectors();
 		drmModeFreeResources(data.res);
