@@ -22,12 +22,34 @@
  *
  */
 
+/**
+ * TEST: kms panel fitting
+ * Category: Display
+ * Description: Test display panel fitting
+ * Driver requirement: i915, xe
+ * Mega feature: General Display Features
+ */
+
 #include "igt.h"
+#include <limits.h>
 #include <math.h>
 #include <sys/stat.h>
 
+/**
+ * SUBTEST: atomic-fastset
+ * Description: Tests panel fitting usages with atomic fastset.
+ *
+ * SUBTEST: legacy
+ * Description: Tests panel fitting usages with legacy style commit.
+ */
+
 IGT_TEST_DESCRIPTION("Test display panel fitting");
 
+enum test_type {
+	TEST_LEGACY,
+	TEST_ATOMIC,
+};
+
 typedef struct {
 	int drm_fd;
 	igt_display_t display;
@@ -61,7 +83,7 @@ static void prepare_crtc(data_t *data, igt_output_t *output, enum pipe pipe,
 	igt_create_pattern_fb(data->drm_fd,
 			      mode->hdisplay, mode->vdisplay,
 			      DRM_FORMAT_XRGB8888,
-			      LOCAL_DRM_FORMAT_MOD_NONE,
+			      DRM_FORMAT_MOD_LINEAR,
 			      &data->fb1);
 
 	/*
@@ -80,62 +102,59 @@ static void prepare_crtc(data_t *data, igt_output_t *output, enum pipe pipe,
 	igt_display_commit2(display, s);
 }
 
-static void test_panel_fitting(data_t *d)
+static void
+test_panel_fitting_legacy(data_t *d, igt_display_t *display,
+			  const enum pipe pipe, igt_output_t *output)
 {
-	igt_display_t *display = &d->display;
-	igt_output_t *output;
-	enum pipe pipe;
-	int valid_tests = 0;
+	drmModeModeInfo *mode, native_mode;
+	bool is_plane_scaling_active = true;
 
-	for_each_pipe_with_valid_output(display, pipe, output) {
-		drmModeModeInfo *mode, native_mode;
-		uint32_t devid = intel_get_drm_devid(display->drm_fd);
+	igt_output_set_pipe(output, pipe);
 
-		/* Check that the "scaling mode" property has been set. */
-		if (!igt_output_has_prop(output, IGT_CONNECTOR_SCALING_MODE))
-			continue;
+	mode = igt_output_get_mode(output);
+	native_mode = *mode;
 
-		cleanup_crtc(d);
-		igt_output_set_pipe(output, pipe);
+	/* allocate fb2 with image */
+	igt_create_pattern_fb(d->drm_fd, mode->hdisplay / 2, mode->vdisplay / 2,
+			      DRM_FORMAT_XRGB8888,
+			      DRM_FORMAT_MOD_LINEAR, &d->fb2);
 
-		mode = igt_output_get_mode(output);
-		native_mode = *mode;
-
-		/* allocate fb2 with image */
-		igt_create_pattern_fb(d->drm_fd, mode->hdisplay / 2, mode->vdisplay / 2,
-				      DRM_FORMAT_XRGB8888,
-				      LOCAL_DRM_FORMAT_MOD_NONE, &d->fb2);
-
-		/* Set up display to enable panel fitting */
-		mode->hdisplay = 640;
-		mode->vdisplay = 480;
-		d->plane1 = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
-		prepare_crtc(d, output, pipe, d->plane1, mode, COMMIT_LEGACY);
-
-		/* disable panel fitting */
-		prepare_crtc(d, output, pipe, d->plane1, &native_mode, COMMIT_LEGACY);
-
-		/* enable panel fitting */
+	/* set up display to enable panel fitting */
+	if (is_amdgpu_device(display->drm_fd)) {
 		mode->hdisplay = 800;
 		mode->vdisplay = 600;
-		prepare_crtc(d, output, pipe, d->plane1, mode, COMMIT_LEGACY);
+	} else {
+		mode->hdisplay = 640;
+		mode->vdisplay = 480;
+	}
+	d->plane1 = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
+	prepare_crtc(d, output, pipe, d->plane1, mode, COMMIT_LEGACY);
 
-		/* disable panel fitting */
-		prepare_crtc(d, output, pipe, d->plane1, &native_mode, COMMIT_LEGACY);
+	/* disable panel fitting */
+	prepare_crtc(d, output, pipe, d->plane1, &native_mode, COMMIT_LEGACY);
 
-		/* Set up fb2->plane2 mapping. */
-		d->plane2 = igt_output_get_plane_type(output, DRM_PLANE_TYPE_OVERLAY);
-		igt_plane_set_fb(d->plane2, &d->fb2);
+	/* enable panel fitting */
+	mode->hdisplay = 800;
+	mode->vdisplay = 600;
+	prepare_crtc(d, output, pipe, d->plane1, mode, COMMIT_LEGACY);
 
-		/* enable sprite plane */
-		igt_fb_set_position(&d->fb2, d->plane2, 100, 100);
-		igt_fb_set_size(&d->fb2, d->plane2, d->fb2.width-200, d->fb2.height-200);
-		igt_plane_set_position(d->plane2, 100, 100);
-		igt_plane_set_size(d->plane2, mode->hdisplay-200, mode->vdisplay-200);
-		igt_display_commit2(display, COMMIT_UNIVERSAL);
+	/* disable panel fitting */
+	prepare_crtc(d, output, pipe, d->plane1, &native_mode, COMMIT_LEGACY);
 
+	/* set up fb2->plane2 mapping. */
+	d->plane2 = igt_output_get_plane_type(output, DRM_PLANE_TYPE_OVERLAY);
+	igt_plane_set_fb(d->plane2, &d->fb2);
+
+	/* enable sprite plane */
+	igt_fb_set_position(&d->fb2, d->plane2, 100, 100);
+	igt_fb_set_size(&d->fb2, d->plane2, d->fb2.width-200, d->fb2.height-200);
+	igt_plane_set_position(d->plane2, 100, 100);
+
+	if (is_intel_device(display->drm_fd)) {
+		uint32_t devid = intel_get_drm_devid(display->drm_fd);
 		/*
-		 * most of gen7 and all of gen8 doesn't support scaling at all.
+		 * Most of gen7 and all of gen8 doesn't support plane scaling
+		 * at all.
 		 *
 		 * gen9 pipe C has only 1 scaler shared with the crtc, which
 		 * means pipe scaling can't work simultaneously with panel
@@ -144,18 +163,34 @@ static void test_panel_fitting(data_t *d)
 		 * Since this is the legacy path, userspace has to know about
 		 * the HW limitations, whereas atomic can ask.
 		 */
-		if (IS_GEN8(devid) || (IS_GEN7(devid) && !IS_IVYBRIDGE(devid)) ||
-		    (IS_GEN9(devid) && pipe == PIPE_C))
-			igt_plane_set_size(d->plane2, d->fb2.width-200, d->fb2.height-200);
-
-		/* enable panel fitting along with sprite scaling */
-		mode->hdisplay = 1024;
-		mode->vdisplay = 768;
-		prepare_crtc(d, output, pipe, d->plane1, mode, COMMIT_LEGACY);
-
-		valid_tests++;
+		if (IS_GEN8(devid) ||
+			(IS_GEN7(devid) && !IS_IVYBRIDGE(devid)) ||
+			(IS_GEN9(devid) && pipe == PIPE_C)) {
+			is_plane_scaling_active = false;
+		}
 	}
-	igt_require_f(valid_tests, "no valid crtc/connector combinations found\n");
+	if (is_plane_scaling_active) {
+		/*
+		 * different than visible area of fb => plane scaling
+		 * active
+		 */
+		igt_plane_set_size(d->plane2,
+				   mode->hdisplay-200,
+				   mode->vdisplay-200);
+	} else {
+		/* same as visible area of fb => no scaling */
+		igt_plane_set_size(d->plane2,
+			d->fb2.width - 200,
+			d->fb2.height - 200);
+	}
+
+	/* Plane scaling active (if possible), pfit off */
+	igt_display_commit2(display, COMMIT_UNIVERSAL);
+
+	/* enable panel fitting along with sprite scaling */
+	mode->hdisplay = 1024;
+	mode->vdisplay = 768;
+	prepare_crtc(d, output, pipe, d->plane1, mode, COMMIT_LEGACY);
 }
 
 static void
@@ -173,15 +208,15 @@ test_panel_fitting_fastset(igt_display_t *display, const enum pipe pipe, igt_out
 	sprite = igt_output_get_plane_type(output, DRM_PLANE_TYPE_OVERLAY);
 
 	igt_create_color_fb(display->drm_fd, mode.hdisplay, mode.vdisplay,
-			    DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE,
+			    DRM_FORMAT_XRGB8888, DRM_FORMAT_MOD_LINEAR,
 			    0.f, 0.f, 1.f, &blue);
 
 	igt_create_color_fb(display->drm_fd, 640, 480,
-			    DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE,
+			    DRM_FORMAT_XRGB8888, DRM_FORMAT_MOD_LINEAR,
 			    1.f, 0.f, 0.f, &red);
 
 	igt_create_color_fb(display->drm_fd, 800, 600,
-			    DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE,
+			    DRM_FORMAT_XRGB8888, DRM_FORMAT_MOD_LINEAR,
 			    0.f, 1.f, 0.f, &green);
 
 	igt_plane_set_fb(primary, &blue);
@@ -207,50 +242,68 @@ test_panel_fitting_fastset(igt_display_t *display, const enum pipe pipe, igt_out
 	igt_display_commit_atomic(display, 0, NULL);
 }
 
-static void test_atomic_fastset(data_t *data)
+static void test_panel_fitting(data_t *data, enum test_type type)
 {
 	igt_display_t *display = &data->display;
 	igt_output_t *output;
 	enum pipe pipe;
-	int valid_tests = 0;
 	struct stat sb;
 
-	/* Until this is force enabled, force modeset evasion. */
-	if (stat("/sys/module/i915/parameters/fastboot", &sb) == 0)
-		igt_set_module_param_int("fastboot", 1);
+	if (type == TEST_ATOMIC) {
+		char path[PATH_MAX];
 
-	igt_require(display->is_atomic);
-	igt_require(intel_gen(intel_get_drm_devid(display->drm_fd)) >= 5);
+		igt_require_f(is_intel_device(display->drm_fd), "not valid for non-intel devices\n");
+
+		/* Until this is force enabled, force modeset evasion. */
+		snprintf(path, PATH_MAX, "/sys/module/%s/parameters/fastboot",
+					 is_i915_device(data->drm_fd) ? "i915" : "xe");
+
+		if (stat(path, &sb) == 0)
+			igt_set_module_param_int(data->drm_fd, "fastboot", 1);
+
+		igt_require(intel_display_ver(intel_get_drm_devid(display->drm_fd)) >= 5);
+
+	}
 
 	for_each_pipe_with_valid_output(display, pipe, output) {
+		/* Check that the "scaling mode" property has been set. */
 		if (!igt_output_has_prop(output, IGT_CONNECTOR_SCALING_MODE))
 			continue;
 
 		cleanup_crtc(data);
-		test_panel_fitting_fastset(display, pipe, output);
-		valid_tests++;
+
+		igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), output->name) {
+			if (type == TEST_ATOMIC)
+				test_panel_fitting_fastset(display, pipe, output);
+			if (type == TEST_LEGACY)
+				test_panel_fitting_legacy(data, display, pipe, output);
+		}
 	}
-	igt_require_f(valid_tests, "no valid crtc/connector combinations found\n");
 }
 
-igt_main
+int igt_main()
 {
 	data_t data = {};
 
-	igt_fixture {
-		igt_skip_on_simulation();
-
-		data.drm_fd = drm_open_driver(DRIVER_ANY);
+	igt_fixture() {
+		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
 		igt_display_require(&data.display, data.drm_fd);
 		igt_display_require_output(&data.display);
 	}
 
-	igt_subtest("legacy")
-		test_panel_fitting(&data);
+	igt_describe("Tests panel fitting usages with legacy style commit.");
+	igt_subtest_with_dynamic("legacy")
+		test_panel_fitting(&data, TEST_LEGACY);
 
-	igt_subtest("atomic-fastset")
-		test_atomic_fastset(&data);
+	igt_fixture()
+		igt_require(&data.display.is_atomic);
 
-	igt_fixture
+	igt_describe("Tests panel fitting usages with atomic fastset.");
+	igt_subtest_with_dynamic("atomic-fastset")
+		test_panel_fitting(&data, TEST_ATOMIC);
+
+	igt_fixture() {
 		igt_display_fini(&data.display);
+		drm_close_driver(data.drm_fd);
+	}
 }
