@@ -28,7 +28,10 @@
  * TEST: kms plane
  * Category: Display
  * Description: Testes for KMS Plane
+ * Driver requirement: i915, xe
+ * Mega feature: General Display Features
  */
+
 #include "igt.h"
 #include "igt_vec.h"
 #include <errno.h>
@@ -37,6 +40,47 @@
 #include <string.h>
 #include "xe/xe_query.h"
 
+/**
+ * SUBTEST: planar-pixel-format-settings
+ * Description: verify planar settings for pixel format are handled correctly
+ *
+ * SUBTEST: plane-position-%s
+ * Description: Verify plane position using two planes to create a %arg[1]
+ *
+ * SUBTEST: plane-position-hole-dpms
+ * Description: Verify plane position using two planes to create a partially
+ *              covered screen and check for DPMS
+ *
+ * arg[1]:
+ *
+ * @covered:           fully covered screen
+ * @hole:              partially covered screen
+ */
+
+/**
+ * SUBTEST: plane-panning-%s
+ * Description: Verify plane panning at %arg[1] using primary plane
+ *
+ * SUBTEST: plane-panning-bottom-right-suspend
+ * Description: Verify plane panning at bottom-right position with suspend using
+ *              primary plane
+ *
+ * arg[1]:
+ *
+ * @bottom-right:               bottom-right position
+ * @top-left:                   top-left position
+ */
+
+/**
+ * SUBTEST: pixel-%s
+ * Description: verify the pixel formats for given plane and pipe
+ *
+ * arg[1]:
+ *
+ * @format:
+ * @format-source-clamping:          with source clamping
+ */
+
 /*
  * Throw away enough lsbs in pixel formats tests
  * to get a match despite some differences between
@@ -84,6 +128,8 @@ static color_t blue  = { 0.0f, 0.0f, 1.0f };
 static void test_init(data_t *data, enum pipe pipe)
 {
 	igt_require(data->display.pipes[pipe].n_planes > 0);
+	if (data->pipe_crc)
+		igt_pipe_crc_free(data->pipe_crc);
 	data->pipe_crc = igt_pipe_crc_new(data->drm_fd, pipe,
 					  IGT_PIPE_CRC_SOURCE_AUTO);
 	igt_display_reset(&data->display);
@@ -92,6 +138,7 @@ static void test_init(data_t *data, enum pipe pipe)
 static void test_fini(data_t *data)
 {
 	igt_pipe_crc_free(data->pipe_crc);
+	data->pipe_crc = NULL;
 }
 
 enum {
@@ -221,8 +268,8 @@ test_plane_position_with_output(data_t *data,
 	drmModeModeInfo *mode;
 	igt_crc_t crc, crc2;
 
-	igt_info("Testing connector %s using pipe %s plane %d\n",
-		 igt_output_name(output), kmstest_pipe_name(pipe), plane);
+	igt_debug("Testing connector %s using pipe %s plane %d\n", igt_output_name(output),
+		  kmstest_pipe_name(pipe), plane);
 
 	igt_output_set_pipe(output, pipe);
 
@@ -231,16 +278,19 @@ test_plane_position_with_output(data_t *data,
 	sprite = igt_output_get_plane(output, plane);
 
 	if (primary->drm_plane->plane_id > sprite->drm_plane->plane_id) {
-		igt_info("primary plane ID (%d) > sprite plane ID (%d), skipping plane %d\n",
-				primary->drm_plane->plane_id, sprite->drm_plane->plane_id, plane);
+		igt_debug("primary plane ID (%d) > sprite plane ID (%d), skipping plane %d\n",
+			  primary->drm_plane->plane_id, sprite->drm_plane->plane_id, plane);
 		return;
 	}
 
 	create_fb_for_mode(data, mode, &green, &rect, 1, &primary_fb);
 	igt_plane_set_fb(primary, &primary_fb);
 
-	if (!igt_plane_has_format_mod(sprite, DRM_FORMAT_XRGB8888, DRM_FORMAT_MOD_LINEAR))
+	if (!igt_plane_has_format_mod(sprite, DRM_FORMAT_XRGB8888, DRM_FORMAT_MOD_LINEAR)) {
+		igt_info("plane-%d does not supports XRGB format and Linear modifier\n",
+			 sprite->index);
 		return;
+	}
 
 	igt_create_color_fb(data->drm_fd,
 			    64, 64, /* width, height */
@@ -284,27 +334,6 @@ test_plane_position_with_output(data_t *data,
 	igt_remove_fb(data->drm_fd, &sprite_fb);
 }
 
-/**
- * SUBTEST: plane-position-%s
- * Description: Verify plane position using two planes to create a %arg[1]
- * Driver requirement: i915, xe
- * Functionality: plane
- * Mega feature: General Display Features
- * Test category: functionality test
- *
- * SUBTEST: plane-position-hole-dpms
- * Description: Verify plane position using two planes to create a partially
- *              covered screen and check for DPMS
- * Driver requirement: i915, xe
- * Functionality: dpms, plane
- * Mega feature: General Display Features
- * Test category: functionality test
- *
- * arg[1]:
- *
- * @covered:           fully covered screen
- * @hole:              partially covered screen
- */
 static void
 test_plane_position(data_t *data, enum pipe pipe)
 {
@@ -318,10 +347,11 @@ test_plane_position(data_t *data, enum pipe pipe)
 	test_init(data, pipe);
 	test_grab_crc(data, output, pipe, &green, data->flags, &reference_crc);
 
-	for (int plane = 1; plane < n_planes; plane++)
-		test_plane_position_with_output(data, pipe, plane,
-						output, &reference_crc,
-						data->flags);
+	for (int plane = 1; plane < n_planes; plane++) {
+		igt_dynamic_f("pipe-%s-plane-%d", kmstest_pipe_name(pipe), plane)
+			test_plane_position_with_output(data, pipe, plane, output,
+							&reference_crc, data->flags);
+	}
 
 	test_fini(data);
 }
@@ -382,8 +412,8 @@ test_plane_panning_with_output(data_t *data,
 	mode = igt_output_get_mode(output);
 	primary = igt_output_get_plane(output, 0);
 
-	igt_info("Testing connector %s using pipe %s, mode %s\n",
-		 igt_output_name(output), kmstest_pipe_name(pipe), mode->name);
+	igt_debug("Testing connector %s using pipe %s, mode %s\n", igt_output_name(output),
+		  kmstest_pipe_name(pipe), mode->name);
 
 	create_fb_for_mode_panning(data, mode, &primary_fb);
 	igt_plane_set_fb(primary, &primary_fb);
@@ -413,27 +443,6 @@ test_plane_panning_with_output(data_t *data,
 	igt_remove_fb(data->drm_fd, &primary_fb);
 }
 
-/**
- * SUBTEST: plane-panning-%s
- * Description: Verify plane panning at %arg[1] using primary plane
- * Driver requirement: i915, xe
- * Functionality: plane
- * Mega feature: General Display Features
- * Test category: functionality test
- *
- * SUBTEST: plane-panning-bottom-right-suspend
- * Description: Verify plane panning at bottom-right position with suspend using
- *              primary plane
- * Driver requirement: i915, xe
- * Functionality: plane, suspend
- * Mega feature: General Display Features
- * Test category: functionality test
- *
- * arg[1]:
- *
- * @bottom-right:               bottom-right position
- * @top-left:                   top-left position
- */
 static void
 test_plane_panning(data_t *data, enum pipe pipe)
 {
@@ -455,7 +464,7 @@ test_plane_panning(data_t *data, enum pipe pipe)
 	}
 
 	if (is_xe_device(data->drm_fd)) {
-		struct drm_xe_query_mem_region *memregion;
+		struct drm_xe_mem_region *memregion;
 		uint64_t memreg = all_memory_regions(data->drm_fd), region;
 
 		xe_for_each_mem_region(data->drm_fd, memreg, region) {
@@ -480,7 +489,7 @@ test_plane_panning(data_t *data, enum pipe pipe)
 		mode_found = true;
 		break;
 	}
-	igt_require(mode_found);
+	igt_require_f(mode_found, "All connector modes are skipped due to low memory\n");
 
 	if (data->flags & TEST_PANNING_TOP_LEFT)
 		test_grab_crc(data, output, pipe, &red, data->flags, &ref_crc);
@@ -514,13 +523,13 @@ static void set_legacy_lut(data_t *data, enum pipe pipe,
 			   uint16_t mask)
 {
 	igt_pipe_t *pipe_obj = &data->display.pipes[pipe];
-	drmModeCrtc *crtc;
+	drmModeCrtc *drm_crtc;
 	uint16_t *lut;
 	int i, lut_size;
 
-	crtc = drmModeGetCrtc(data->drm_fd, pipe_obj->crtc_id);
-	lut_size = crtc->gamma_size;
-	drmModeFreeCrtc(crtc);
+	drm_crtc = drmModeGetCrtc(data->drm_fd, pipe_obj->crtc_id);
+	lut_size = drm_crtc->gamma_size;
+	drmModeFreeCrtc(drm_crtc);
 
 	/* Skip if legacy LUT is not supported: */
 	if (!lut_size)
@@ -541,13 +550,13 @@ static bool set_c8_legacy_lut(data_t *data, enum pipe pipe,
 			      uint16_t mask)
 {
 	igt_pipe_t *pipe_obj = &data->display.pipes[pipe];
-	drmModeCrtc *crtc;
+	drmModeCrtc *drm_crtc;
 	uint16_t *r, *g, *b;
 	int i, lut_size;
 
-	crtc = drmModeGetCrtc(data->drm_fd, pipe_obj->crtc_id);
-	lut_size = crtc->gamma_size;
-	drmModeFreeCrtc(crtc);
+	drm_crtc = drmModeGetCrtc(data->drm_fd, pipe_obj->crtc_id);
+	lut_size = drm_crtc->gamma_size;
+	drmModeFreeCrtc(drm_crtc);
 
 	if (lut_size != 256)
 		return false;
@@ -862,10 +871,11 @@ static bool test_format_plane_colors(data_t *data, enum pipe pipe,
 		}
 	}
 
-	if (crc_mismatch_count)
-		igt_warn("CRC mismatches with format " IGT_FORMAT_FMT " on %s.%u with %d/%d solid colors tested (0x%X)\n",
-			 IGT_FORMAT_ARGS(format), kmstest_pipe_name(pipe),
-			 plane->index, crc_mismatch_count, data->num_colors, crc_mismatch_mask);
+	igt_warn_on_f(crc_mismatch_count,
+		      "CRC mismatches with format " IGT_FORMAT_FMT " on %s.%u with %d/%d solid colors tested (0x%X)\n",
+		      IGT_FORMAT_ARGS(format), kmstest_pipe_name(pipe),
+		      plane->index, crc_mismatch_count, data->num_colors,
+		      crc_mismatch_mask);
 
 	return result;
 }
@@ -915,11 +925,13 @@ static bool test_format_plane_yuv(data_t *data, enum pipe pipe,
 						     igt_color_range_to_str(r)))
 				continue;
 
-			igt_info("Testing format " IGT_FORMAT_FMT " / modifier " IGT_MODIFIER_FMT " (%s, %s) on %s.%u\n",
-				 IGT_FORMAT_ARGS(format), IGT_MODIFIER_ARGS(modifier),
+			igt_info("Testing format " IGT_FORMAT_FMT " / modifier " IGT_MODIFIER_FMT
+				 " (%s, %s) on %s.%u\n", IGT_FORMAT_ARGS(format),
+				 IGT_MODIFIER_ARGS(modifier),
 				 igt_color_encoding_to_str(e),
 				 igt_color_range_to_str(r),
-				 kmstest_pipe_name(pipe), plane->index);
+				 kmstest_pipe_name(pipe),
+				 plane->index);
 
 			result &= test_format_plane_colors(data, pipe, plane,
 							   format, modifier,
@@ -987,7 +999,37 @@ static void check_allowed_plane_size_64x64(data_t *data, igt_plane_t *plane,
 	igt_remove_fb(data->drm_fd, &test_fb);
 }
 
-static bool test_format_plane(data_t *data, enum pipe pipe,
+static bool skip_format_mod(data_t *data,
+			    uint32_t format, uint64_t modifier,
+			    struct igt_vec *tested_formats)
+{
+	/* igt doesn't know how to sw generate UBWC: */
+	if (is_msm_device(data->drm_fd) &&
+	    modifier == DRM_FORMAT_MOD_QCOM_COMPRESSED)
+		return true;
+
+	/* VEBOX just hangs with an actual 10bpc format */
+	if (igt_fb_is_gen12_mc_ccs_modifier(modifier) &&
+	    igt_reduce_format(format) == DRM_FORMAT_XRGB2101010)
+		return true;
+
+	/* test each format "class" only once in non-extended tests */
+	if (!data->extended && modifier != DRM_FORMAT_MOD_LINEAR) {
+		struct format_mod rf = {
+			.format = igt_reduce_format(format),
+			.modifier = modifier,
+		};
+
+		if (igt_vec_index(tested_formats, &rf) >= 0)
+			return true;
+
+		igt_vec_push(tested_formats, &rf);
+	}
+
+	return false;
+}
+
+static void test_format_plane(data_t *data, enum pipe pipe,
 			      igt_output_t *output, igt_plane_t *plane, igt_fb_t *primary_fb)
 {
 	struct igt_fb fb = {};
@@ -1004,7 +1046,7 @@ static bool test_format_plane(data_t *data, enum pipe pipe,
 	 * No clamping test for cursor plane
 	 */
 	if (data->crop != 0 && plane->type == DRM_PLANE_TYPE_CURSOR)
-		return true;
+		return;
 
 	igt_vec_init(&tested_formats, sizeof(struct format_mod));
 
@@ -1017,7 +1059,7 @@ static bool test_format_plane(data_t *data, enum pipe pipe,
 	} else {
 		if (!plane->drm_plane) {
 			igt_debug("Only legacy cursor ioctl supported, skipping cursor plane\n");
-			return true;
+			return;
 		}
 		do_or_die(drmGetCap(data->drm_fd, DRM_CAP_CURSOR_WIDTH, &width));
 		do_or_die(drmGetCap(data->drm_fd, DRM_CAP_CURSOR_HEIGHT, &height));
@@ -1060,30 +1102,18 @@ static bool test_format_plane(data_t *data, enum pipe pipe,
 			.modifier = plane->modifiers[i],
 		};
 
-		/* igt doesn't know how to sw generate UBWC: */
-		if (is_msm_device(data->drm_fd) &&
-		    f.modifier == DRM_FORMAT_MOD_QCOM_COMPRESSED)
-			continue;
-
 		if (f.format == ref.format &&
 		    f.modifier == ref.modifier)
 			continue;
 
-		/* test each format "class" only once in non-extended tests */
-		if (!data->extended && f.modifier != DRM_FORMAT_MOD_LINEAR) {
-			struct format_mod rf = {
-				.format = igt_reduce_format(f.format),
-				.modifier = f.modifier,
-			};
-
-			if (igt_vec_index(&tested_formats, &rf) >= 0) {
-				igt_info("Skipping format " IGT_FORMAT_FMT " / modifier " IGT_MODIFIER_FMT " on %s.%u\n",
-					 IGT_FORMAT_ARGS(f.format), IGT_MODIFIER_ARGS(f.modifier),
-					 kmstest_pipe_name(pipe), plane->index);
-				continue;
-			}
-
-			igt_vec_push(&tested_formats, &rf);
+		if (skip_format_mod(data, f.format, f.modifier, &tested_formats)) {
+			igt_info("Skipping format " IGT_FORMAT_FMT " / modifier "
+				 IGT_MODIFIER_FMT " on %s.%u\n",
+				 IGT_FORMAT_ARGS(f.format),
+				 IGT_MODIFIER_ARGS(f.modifier),
+				 kmstest_pipe_name(pipe),
+				 plane->index);
+			continue;
 		}
 
 		if (f.format == DRM_FORMAT_C8) {
@@ -1119,7 +1149,7 @@ static bool test_format_plane(data_t *data, enum pipe pipe,
 
 	igt_vec_fini(&tested_formats);
 
-	return result;
+	igt_assert_f(result, "At least one CRC mismatch happened\n");
 }
 
 static bool skip_plane(data_t *data, igt_plane_t *plane)
@@ -1159,26 +1189,12 @@ static bool skip_plane(data_t *data, igt_plane_t *plane)
 	return index != 0 && index != 3 && index != 5;
 }
 
-/**
- * SUBTEST: pixel-%s
- * Description: verify the pixel formats for given plane and pipe
- * Driver requirement: i915, xe
- * Functionality: pixel_format, plane
- * Mega feature: General Display Features
- * Test category: functionality test
- *
- * arg[1]:
- *
- * @format:
- * @format-source-clamping:          with source clamping
- */
 static void
 test_pixel_formats(data_t *data, enum pipe pipe)
 {
 	struct igt_fb primary_fb;
 	igt_plane_t *primary;
 	drmModeModeInfo *mode;
-	bool result;
 	igt_output_t *output = data->output;
 	igt_plane_t *plane;
 
@@ -1208,11 +1224,11 @@ test_pixel_formats(data_t *data, enum pipe pipe)
 
 	set_legacy_lut(data, pipe, LUT_MASK);
 
-	result = true;
 	for_each_plane_on_pipe(&data->display, pipe, plane) {
 		if (skip_plane(data, plane))
 			continue;
-		result &= test_format_plane(data, pipe, output, plane, &primary_fb);
+		igt_dynamic_f("pipe-%s-plane-%u", kmstest_pipe_name(pipe), plane->index)
+			test_format_plane(data, pipe, output, plane, &primary_fb);
 	}
 
 	test_fini(data);
@@ -1224,32 +1240,27 @@ test_pixel_formats(data_t *data, enum pipe pipe)
 	igt_display_commit2(&data->display, data->display.is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
 
 	igt_remove_fb(data->drm_fd, &primary_fb);
-
-	igt_assert_f(result, "At least one CRC mismatch happened\n");
 }
 
-/**
- * SUBTEST: invalid-pixel-format-settings
- * Description: verify invalid settings for pixel format are not accepted
- * Driver requirement: i915, xe
- * Functionality: pixel_format, plane
- * Mega feature: General Display Features
- * Test category: functionality test
- */
-static void test_invalid_settings(data_t *data)
+static void test_planar_settings(data_t *data)
 {
 	enum pipe pipe = PIPE_A;
 	igt_output_t *output;
 	igt_fb_t fb;
 	igt_plane_t *primary;
+	int devid;
 	int rval;
 
 	/*
 	 * If here is added non-intel tests below require will need to be
 	 * changed to if(..)
 	 */
-	igt_require_intel(data->drm_fd);
-	igt_require(intel_display_ver(intel_get_drm_devid(data->drm_fd)) >= 9);
+	igt_require_f(data->display.is_atomic, "Atomic mode-set not supported\n");
+	if (is_intel_device(data->drm_fd)) {
+		igt_require_intel(data->drm_fd);
+		devid = intel_get_drm_devid(data->drm_fd);
+		igt_require(intel_display_ver(devid) >= 9);
+	}
 
 	output = igt_get_single_output_for_pipe(&data->display, pipe);
 	igt_require(output);
@@ -1257,17 +1268,26 @@ static void test_invalid_settings(data_t *data)
 	igt_output_set_pipe(output, pipe);
 	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
 
-	igt_display_commit2(&data->display, data->display.is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
+	igt_display_commit_atomic(&data->display,
+				  DRM_MODE_ATOMIC_ALLOW_MODESET,
+				  NULL);
 
 	/* test against intel_plane_check_src_coordinates() in i915 */
 	if (igt_plane_has_format_mod(primary, DRM_FORMAT_NV12,
 				     DRM_FORMAT_MOD_LINEAR)) {
+		int expected_rval = -EINVAL;
+
+		if (intel_display_ver(devid) >= 20)
+			expected_rval = 0;
+
 		igt_create_fb(data->drm_fd, 257, 256,
 			      DRM_FORMAT_NV12, DRM_FORMAT_MOD_LINEAR, &fb);
 		igt_plane_set_fb(primary, &fb);
-		rval = igt_display_try_commit_atomic(&data->display, 0, NULL);
+		rval = igt_display_try_commit_atomic(&data->display,
+						     DRM_MODE_ATOMIC_ALLOW_MODESET,
+						     NULL);
 		igt_remove_fb(data->drm_fd, &fb);
-		igt_assert_f(rval == -EINVAL, "Odd width NV12 framebuffer\n");
+		igt_assert_f(rval == expected_rval, "Odd width NV12 framebuffer\n");
 	} else {
 		igt_debug("Odd width NV12 framebuffer test skipped\n");
 	}
@@ -1275,15 +1295,41 @@ static void test_invalid_settings(data_t *data)
 	/* test against intel_plane_check_src_coordinates() in i915 */
 	if (igt_plane_has_format_mod(primary, DRM_FORMAT_NV12,
 				     DRM_FORMAT_MOD_LINEAR)) {
+		int expected_rval = -EINVAL;
+
+		if (intel_display_ver(devid) >= 20)
+			expected_rval = 0;
+
 		igt_create_fb(data->drm_fd, 256, 257,
 			      DRM_FORMAT_NV12, DRM_FORMAT_MOD_LINEAR, &fb);
 		igt_plane_set_fb(primary, &fb);
-		rval = igt_display_try_commit_atomic(&data->display, 0, NULL);
+		rval = igt_display_try_commit_atomic(&data->display,
+						     DRM_MODE_ATOMIC_ALLOW_MODESET,
+						     NULL);
 		igt_remove_fb(data->drm_fd, &fb);
-		igt_assert_f(rval == -EINVAL, "Odd height NV12 framebuffer\n");
+		igt_assert_f(rval == expected_rval, "Odd height NV12 framebuffer\n");
 	} else {
 		igt_debug("Odd height NV12 framebuffer test skipped\n");
 	}
+
+	/* test against intel_plane_check_src_coordinates() in i915 */
+	if (igt_plane_has_format_mod(primary, DRM_FORMAT_NV12,
+				     DRM_FORMAT_MOD_LINEAR)) {
+		int expected_rval = -EINVAL;
+
+		igt_create_fb(data->drm_fd, 810, 590,
+			      DRM_FORMAT_NV12, DRM_FORMAT_MOD_LINEAR, &fb);
+		igt_plane_set_fb(primary, &fb);
+		igt_plane_set_size(primary, 810, 590);
+		igt_plane_set_position(primary, -501, 200);
+		rval = igt_display_try_commit_atomic(&data->display, DRM_MODE_ATOMIC_ALLOW_MODESET |
+						     DRM_MODE_ATOMIC_TEST_ONLY, NULL);
+
+		igt_remove_fb(data->drm_fd, &fb);
+		igt_assert_f(rval == expected_rval, "Odd horizontal pan NV12 framebuffer\n");
+	} else {
+		igt_debug("Odd horizontal pan NV12 framebuffer test skipped\n");
+	}
 }
 
 static bool is_pipe_limit_reached(int count)
@@ -1303,18 +1349,23 @@ static void run_test(data_t *data, void (*test)(data_t *, enum pipe))
 		igt_display_reset(&data->display);
 
 		igt_output_set_pipe(data->output, pipe);
-		if (!i915_pipe_output_combo_valid(&data->display))
+		if (!intel_pipe_output_combo_valid(&data->display))
 			continue;
 
 		igt_output_set_pipe(data->output, PIPE_NONE);
-		igt_dynamic_f("pipe-%s-planes", kmstest_pipe_name(pipe))
-			test(data, pipe);
+		test(data, pipe);
 
 		if (is_pipe_limit_reached(++count))
 			break;
 	}
 }
 
+static void dynamic_test_handler(data_t *data, enum pipe pipe)
+{
+	igt_dynamic_f("pipe-%s", kmstest_pipe_name(pipe))
+		test_plane_panning(data, pipe);
+}
+
 static void
 run_tests_for_pipe_plane(data_t *data)
 {
@@ -1351,25 +1402,25 @@ run_tests_for_pipe_plane(data_t *data)
 	igt_describe("verify plane panning at top-left position using primary plane");
 	igt_subtest_with_dynamic_f("plane-panning-top-left") {
 		data->flags = TEST_PANNING_TOP_LEFT;
-		run_test(data, test_plane_panning);
+		run_test(data, dynamic_test_handler);
 	}
 
 	igt_describe("verify plane panning at bottom-right position using primary plane");
 	igt_subtest_with_dynamic_f("plane-panning-bottom-right") {
 		data->flags = TEST_PANNING_BOTTOM_RIGHT;
-		run_test(data, test_plane_panning);
+		run_test(data, dynamic_test_handler);
 	}
 
 	igt_describe("verify plane panning at bottom-right position using primary plane and executes system"
 		       "suspend cycles");
 	igt_subtest_with_dynamic_f("plane-panning-bottom-right-suspend") {
 		data->flags = TEST_PANNING_BOTTOM_RIGHT | TEST_SUSPEND_RESUME;
-		run_test(data, test_plane_panning);
+		run_test(data, dynamic_test_handler);
 	}
 
-	igt_describe("verify invalid settings for pixel format are not accepted");
-	igt_subtest_f("invalid-pixel-format-settings")
-		test_invalid_settings(data);
+	igt_describe("verify planar settings for pixel format are accepted or rejected correctly");
+	igt_subtest_f("planar-pixel-format-settings")
+		test_planar_settings(data);
 }
 
 static int opt_handler(int opt, int opt_index, void *_data)
@@ -1400,9 +1451,9 @@ static const char help_str[] =
 
 static data_t data;
 
-igt_main_args("", long_opts, help_str, opt_handler, &data)
+int igt_main_args("", long_opts, help_str, opt_handler, &data)
 {
-	igt_fixture {
+	igt_fixture() {
 		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
 
 		kmstest_set_vt_graphics_mode();
@@ -1413,7 +1464,7 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 
 	run_tests_for_pipe_plane(&data);
 
-	igt_fixture {
+	igt_fixture() {
 		igt_display_fini(&data.display);
 		drm_close_driver(data.drm_fd);
 	}
