@@ -27,9 +27,7 @@
  * Category: Display
  * Description: Test display plane scaling
  * Driver requirement: i915, xe
- * Functionality: plane, scaling
  * Mega feature: General Display Features
- * Test category: functionality test
  */
 
 #include "igt.h"
@@ -93,15 +91,12 @@
 /**
  * SUBTEST: plane-scaler-unity-scaling-with-pixel-format
  * Description: Tests scaling with pixel formats, unity scaling.
- * Functionality: pixel_formats, plane, scaling
  *
  * SUBTEST: plane-scaler-with-clipping-clamping-pixel-formats
  * Description: Tests scaling with clipping and clamping, pixel formats.
- * Functionality: pixel_formats, plane, scaling
  *
  * SUBTEST: plane-upscale-%s-with-pixel-format
  * Description: Tests upscaling with pixel formats %arg[1].
- * Functionality: pixel_formats, plane, scaling
  *
  * arg[1]:
  *
@@ -112,7 +107,6 @@
 /**
  * SUBTEST: plane-downscale-factor-%s-with-pixel-format
  * Description: Tests downscaling with pixel formats for %arg[1] scaling factor.
- * Functionality: pixel_formats, plane, scaling
  *
  * arg[1]:
  *
@@ -568,6 +562,13 @@ static void cleanup_crtc(data_t *data)
 	cleanup_fbs(data);
 }
 
+static void skip_if_erange_or_einval(int ret)
+{
+	igt_skip_on_f(ret == -ERANGE || ret == -EINVAL,
+		      "Unsupported scaling operation (ret = %s)\n",
+		      ret == -EINVAL ? "-EINVAL" : "-ERANGE");
+}
+
 static uint32_t
 check_scaling_pipe_plane_rot(data_t *d, igt_plane_t *plane,
 			     uint32_t pixel_format,
@@ -1179,7 +1180,7 @@ static void invalid_parameter_tests(data_t *d)
 		},
 	};
 
-	igt_fixture {
+	igt_fixture() {
 		output = igt_get_single_output_for_pipe(&d->display, pipe);
 		igt_require(output);
 
@@ -1216,7 +1217,7 @@ static void invalid_parameter_tests(data_t *d)
 		}
 	}
 
-	igt_fixture {
+	igt_fixture() {
 		igt_remove_fb(d->drm_fd, &fb);
 		igt_output_set_pipe(output, PIPE_NONE);
 	}
@@ -1271,15 +1272,26 @@ static drmModeModeInfo *find_mode(data_t *data, igt_output_t *output, const stru
  *		max_dst_h = 8192
  */
 static void intel_max_source_size_test(data_t *d, enum pipe pipe, igt_output_t *output,
-				       drmModeModeInfo *mode, const uint32_t planesize[])
+				       const struct invalid_paramtests *param)
 {
 	igt_fb_t fb;
 	igt_plane_t *plane;
 	int rval;
+	drmModeModeInfo *mode = NULL;
 
 	cleanup_crtc(d);
 
 	igt_output_set_pipe(output, pipe);
+
+	/*
+	 * Need to get the mode again, because it may have changed
+	 * after setting the pipe.
+	 */
+	mode = find_mode(d, output, param);
+	igt_assert(mode);
+	if (!mode)
+		return;
+
 	igt_output_override_mode(output, mode);
 	plane = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
 
@@ -1290,7 +1302,7 @@ static void intel_max_source_size_test(data_t *d, enum pipe pipe, igt_output_t *
 
 	igt_plane_set_position(plane, 0, 0);
 	igt_plane_set_fb(plane, &fb);
-	igt_plane_set_size(plane, planesize[0], planesize[1]);
+	igt_plane_set_size(plane, param->planesize[0], param->planesize[1]);
 
 	rval = igt_display_try_commit2(&d->display, COMMIT_ATOMIC);
 
@@ -1342,12 +1354,12 @@ static const char help_str[] =
 
 static data_t data;
 
-igt_main_args("", long_opts, help_str, opt_handler, &data)
+int igt_main_args("", long_opts, help_str, opt_handler, &data)
 {
 	enum pipe pipe;
 	uint32_t ret = -EINVAL;
 
-	igt_fixture {
+	igt_fixture() {
 		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
 		igt_display_require(&data.display, data.drm_fd);
 		data.devid = is_intel_device(data.drm_fd) ?
@@ -1355,7 +1367,7 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 		igt_require(data.display.is_atomic);
 	}
 
-	igt_subtest_group {
+	igt_subtest_group() {
 		igt_output_t *output;
 
 		for (int index = 0; index < ARRAY_SIZE(scaler_with_pixel_format_tests); index++) {
@@ -1380,9 +1392,7 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 							igt_info("Required scaling operation not supported on %s trying on next output\n",
 								 igt_output_name(output));
 						}
-						igt_skip_on_f(ret == -ERANGE || ret == -EINVAL,
-							      "Unsupported scaling operation in driver with return value %s\n",
-							      (ret == -EINVAL) ? "-EINVAL" : "-ERANGE");
+						skip_if_erange_or_einval(ret);
 					}
 				}
 			}
@@ -1410,9 +1420,7 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 							igt_info("Required scaling operation not supported on %s trying on next output\n",
 								 igt_output_name(output));
 						}
-						igt_skip_on_f(ret == -ERANGE || ret == -EINVAL,
-							      "Unsupported scaling operation in driver with return value %s\n",
-							      (ret == -EINVAL) ? "-EINVAL" : "-ERANGE");
+						skip_if_erange_or_einval(ret);
 					}
 				}
 			}
@@ -1440,9 +1448,7 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 							igt_info("Required scaling operation not supported on %s trying on next output\n",
 								 igt_output_name(output));
 						}
-						igt_skip_on_f(ret == -ERANGE || ret == -EINVAL,
-							      "Unsupported scaling operation in driver with return value %s\n",
-							      (ret == -EINVAL) ? "-EINVAL" : "-ERANGE");
+						skip_if_erange_or_einval(ret);
 					}
 				}
 			}
@@ -1467,9 +1473,7 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 						igt_info("Required scaling operation not supported on %s trying on next output\n",
 							 igt_output_name(output));
 					}
-					igt_skip_on_f(ret == -ERANGE || ret == -EINVAL,
-						      "Unsupported scaling operation in driver with return value %s\n",
-						      (ret == -EINVAL) ? "-EINVAL" : "-ERANGE");
+					skip_if_erange_or_einval(ret);
 				}
 			}
 		}
@@ -1494,9 +1498,7 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 						igt_info("Required scaling operation not supported on %s trying on next output\n",
 							 igt_output_name(output));
 					}
-					igt_skip_on_f(ret == -ERANGE || ret == -EINVAL,
-						      "Unsupported scaling operation in driver with return value %s\n",
-						      (ret == -EINVAL) ? "-EINVAL" : "-ERANGE");
+					skip_if_erange_or_einval(ret);
 				}
 			}
 		}
@@ -1520,9 +1522,7 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 						igt_info("Required scaling operation not supported on %s trying on next output\n",
 							 igt_output_name(output));
 					}
-					igt_skip_on_f(ret == -ERANGE || ret == -EINVAL,
-						      "Unsupported scaling operation in driver with return value %s\n",
-						      (ret == -EINVAL) ? "-EINVAL" : "-ERANGE");
+					skip_if_erange_or_einval(ret);
 				}
 			}
 		}
@@ -1549,9 +1549,7 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 							igt_info("Required scaling operation not supported on %s trying on next output\n",
 								 igt_output_name(output));
 						}
-						igt_skip_on_f(ret == -ERANGE || ret == -EINVAL,
-							      "Unsupported scaling operation in driver with return value %s\n",
-							      (ret == -EINVAL) ? "-EINVAL" : "-ERANGE");
+						skip_if_erange_or_einval(ret);
 					}
 				}
 			}
@@ -1563,21 +1561,23 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 				igt_require_intel(data.drm_fd);
 				for_each_pipe(&data.display, pipe) {
 					for_each_valid_output_on_pipe(&data.display, pipe, output) {
-						drmModeModeInfo *mode = NULL;
-
 						if (get_num_scalers(&data.display, pipe) < 1)
 							continue;
 						/*
 						 * Need to find mode with lowest vrefresh else
 						 * we can exceed cdclk limits.
 						 */
-						mode = find_mode(&data, output, &intel_paramtests[index]);
-						if (mode) {
+						if (find_mode(&data, output, &intel_paramtests[index]))
 							igt_dynamic_f("pipe-%s-%s",
-								       kmstest_pipe_name(pipe), igt_output_name(output))
-								intel_max_source_size_test(&data, pipe, output, mode,
-											   intel_paramtests[index].planesize);
-						}
+								      kmstest_pipe_name(pipe), igt_output_name(output))
+								intel_max_source_size_test(&data, pipe, output,
+											   &intel_paramtests[index]);
+						else
+							igt_info("Unable to find the lowest " \
+								 "refresh rate mode on output " \
+								 "%s pipe %s\n",
+								 igt_output_name(output),
+								 kmstest_pipe_name(pipe));
 						continue;
 					}
 					break;
@@ -1600,14 +1600,14 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 		}
 	}
 
-	igt_subtest_group
+	igt_subtest_group()
 		invalid_parameter_tests(&data);
 
 	igt_describe("Tests scaling with multi-pipe.");
 	igt_subtest_f("2x-scaler-multi-pipe")
 		test_scaler_with_multi_pipe_plane(&data);
 
-	igt_fixture {
+	igt_fixture() {
 		igt_display_fini(&data.display);
 		drm_close_driver(data.drm_fd);
 	}
