@@ -21,6 +21,15 @@
  * IN THE SOFTWARE.
  */
 
+/**
+ * TEST: kms rmfb
+ * Category: Display
+ * Description: This tests rmfb and close-fd behavior. In these cases the
+ *              framebuffers should be removed from the crtc.
+ * Driver requirement: i915, xe
+ * Mega feature: General Display Features
+ */
+
 #include "igt.h"
 #include "drmtest.h"
 #include <errno.h>
@@ -29,6 +38,17 @@
 #include <string.h>
 #include <time.h>
 
+/**
+ * SUBTEST: close-fd
+ * Description: Kernel driver is supposed to free the framebuffers from any and all planes
+ *		when the fd is closed. Ensure that is the case by closing and re-opening
+ *		it.
+ *
+ * SUBTEST: rmfb-ioctl
+ * Description: Kernel driver is supposed to free the framebuffers from any and all planes
+ *              when DRM_IOCTL_MODE_RMFB ioctl is called. Ensure that is the case.
+ */
+
 #ifndef DRM_CAP_CURSOR_WIDTH
 #define DRM_CAP_CURSOR_WIDTH 0x8
 #endif
@@ -36,6 +56,9 @@
 #define DRM_CAP_CURSOR_HEIGHT 0x9
 #endif
 
+IGT_TEST_DESCRIPTION("This tests rmfb and close-fd behavior. In these cases "
+		     "the framebuffers should be removed from the crtc.");
+
 struct rmfb_data {
 	int drm_fd;
 	igt_display_t display;
@@ -54,24 +77,26 @@ static void
 test_rmfb(struct rmfb_data *data, igt_output_t *output, enum pipe pipe, bool reopen)
 {
 	struct igt_fb fb, argb_fb;
+	igt_display_t *display = &data->display;
 	drmModeModeInfo *mode;
 	igt_plane_t *plane;
-	drmModeCrtc *crtc;
+	drmModeCrtc *drm_crtc;
 	uint64_t cursor_width, cursor_height;
 	int num_active_planes = 0;
 
+	igt_display_reset(display);
 	igt_output_set_pipe(output, pipe);
 
 	mode = igt_output_get_mode(output);
 
 	igt_create_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
-		      DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE, &fb);
+		      DRM_FORMAT_XRGB8888, DRM_FORMAT_MOD_LINEAR, &fb);
 
 	do_or_die(drmGetCap(data->drm_fd, DRM_CAP_CURSOR_WIDTH, &cursor_width));
 	do_or_die(drmGetCap(data->drm_fd, DRM_CAP_CURSOR_HEIGHT, &cursor_height));
 
 	igt_create_fb(data->drm_fd, cursor_width, cursor_height,
-		      DRM_FORMAT_ARGB8888, LOCAL_DRM_FORMAT_MOD_NONE, &argb_fb);
+		      DRM_FORMAT_ARGB8888, DRM_FORMAT_MOD_LINEAR, &argb_fb);
 
 	/*
 	 * Make sure these buffers are suited for display use
@@ -87,8 +112,10 @@ test_rmfb(struct rmfb_data *data, igt_output_t *output, enum pipe pipe, bool reo
 			igt_plane_set_fb(plane, &fb);
 		}
 
-		if (igt_display_try_commit2(&data->display, data->display.is_atomic ?
-					    COMMIT_ATOMIC : COMMIT_LEGACY)) {
+		if (igt_display_try_commit2(&data->display,
+					    data->display.is_atomic ? COMMIT_ATOMIC :
+					    plane->type == DRM_PLANE_TYPE_PRIMARY ?
+					    COMMIT_LEGACY : COMMIT_UNIVERSAL)) {
 			/*
 			 * Disable any plane that fails (presumably
 			 * due to exceeding some hardware limit).
@@ -107,14 +134,14 @@ test_rmfb(struct rmfb_data *data, igt_output_t *output, enum pipe pipe, bool reo
 
 	igt_display_commit2(&data->display, data->display.is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
 
-	crtc = drmModeGetCrtc(data->drm_fd, output->config.crtc->crtc_id);
+	drm_crtc = drmModeGetCrtc(data->drm_fd, output->config.crtc->crtc_id);
 
-	igt_assert_eq(crtc->buffer_id, fb.fb_id);
+	igt_assert_eq(drm_crtc->buffer_id, fb.fb_id);
 
-	drmModeFreeCrtc(crtc);
+	drmModeFreeCrtc(drm_crtc);
 
 	if (reopen) {
-		close(data->drm_fd);
+		drm_close_driver(data->drm_fd);
 
 		data->drm_fd = drm_open_driver_master(DRIVER_ANY);
 		drmSetClientCap(data->drm_fd, DRM_CLIENT_CAP_UNIVERSAL_PLANES, 1);
@@ -126,11 +153,11 @@ test_rmfb(struct rmfb_data *data, igt_output_t *output, enum pipe pipe, bool reo
 		igt_remove_fb(data->drm_fd, &argb_fb);
 	}
 
-	crtc = drmModeGetCrtc(data->drm_fd, output->config.crtc->crtc_id);
+	drm_crtc = drmModeGetCrtc(data->drm_fd, output->config.crtc->crtc_id);
 
-	igt_assert_eq(crtc->buffer_id, 0);
+	igt_assert_eq(drm_crtc->buffer_id, 0);
 
-	drmModeFreeCrtc(crtc);
+	drmModeFreeCrtc(drm_crtc);
 
 	for_each_plane_on_pipe(&data->display, pipe, plane) {
 		drmModePlanePtr planeres = drmModeGetPlane(data->drm_fd, plane->drm_plane->plane_id);
@@ -140,7 +167,7 @@ test_rmfb(struct rmfb_data *data, igt_output_t *output, enum pipe pipe, bool reo
 		drmModeFreePlane(planeres);
 	}
 
-	igt_output_set_pipe(output, PIPE_ANY);
+	igt_output_set_pipe(output, PIPE_NONE);
 }
 
 static void
@@ -148,19 +175,44 @@ run_rmfb_test(struct rmfb_data *data, bool reopen)
 {
 	igt_output_t *output;
 	enum pipe pipe;
+	igt_display_t *display = &data->display;
 
-	for_each_pipe_with_single_output(&data->display, pipe, output)
-		test_rmfb(data, output, pipe, reopen);
+	for_each_pipe_with_single_output(display, pipe, output) {
+		igt_display_reset(display);
+
+		igt_output_set_pipe(output, pipe);
+		if (!intel_pipe_output_combo_valid(display))
+			continue;
+
+		igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe),
+			      igt_output_name(output))
+			test_rmfb(data, output, pipe, reopen);
+	}
 }
 
-igt_main
+int igt_main()
 {
+	const struct {
+		bool reopen;
+		const char *name;
+		const char *description;
+	} tests[] = {
+		{ false, "rmfb-ioctl", "Kernel driver is supposed to free the framebuffers from any and all planes "
+				       "when DRM_IOCTL_MODE_RMFB ioctl is called. Ensure that is the case." },
+		{ true, "close-fd", "Kernel driver is supposed to free the framebuffers from any and all planes "
+				    "when the fd is closed. Ensure that is the case by closing and re-opening "
+				    "it" },
+	};
 	struct rmfb_data data = {};
+	int i, other_fd;
 
-	igt_skip_on_simulation();
-
-	igt_fixture {
+	igt_fixture() {
 		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
+		/*
+		 * Prevent fb from changing underneath so we can check by
+		 * fb_id == 0 after removing the fb
+		 */
+		other_fd = drm_reopen_driver(data.drm_fd);
 
 		kmstest_set_vt_graphics_mode();
 
@@ -168,13 +220,16 @@ igt_main
 		igt_display_require_output(&data.display);
 	}
 
-	igt_subtest_f("rmfb-ioctl")
-		run_rmfb_test(&data, false);
+	for (i = 0; i < ARRAY_SIZE(tests); i++) {
+		igt_describe(tests[i].description);
+		igt_subtest_with_dynamic(tests[i].name) {
+			run_rmfb_test(&data, tests[i].reopen);
+		}
+	}
 
-	igt_subtest_f("close-fd")
-		run_rmfb_test(&data, true);
-
-	igt_fixture {
+	igt_fixture() {
 		igt_display_fini(&data.display);
+		drm_close_driver(data.drm_fd);
+		drm_close_driver(other_fd);
 	}
 }
