@@ -27,9 +27,7 @@
  * Category: Display
  * Description: Tests different rotations with different planes & formats
  * Driver requirement: i915, xe
- * Functionality: plane, rotation
  * Mega feature: General Display Features
- * Test category: functionality test
  */
 
 #include "igt.h"
@@ -65,7 +63,6 @@
  *
  * SUBTEST: bad-tiling
  * Description: Checking unsupported tiling for gen9+ with 90 degree of rotation
- * Functionality: plane, rotation, tiling
  *
  * SUBTEST: exhaust-fences
  * Description: This test intends to check for fence leaks exhaustively
@@ -74,7 +71,6 @@
 /**
  * SUBTEST: primary-%s-tiled-reflect-x-%d
  * Description: Test for %arg[1] & %arg[2] degree rotation on primary plane
- * Functionality: plane, rotation, tiling
  *
  * arg[1]:
  *
@@ -89,7 +85,6 @@
 /**
  * SUBTEST: primary-%s-tiled-reflect-x-%d
  * Description: Test for %arg[1] & %arg[2] degree rotation on primary plane
- * Functionality: plane, rotation, tiling
  *
  * arg[1]:
  *
@@ -564,6 +559,29 @@ static bool test_format(data_t *data,
 	return true;
 }
 
+static bool plane_rotation_requirements(data_t *data, igt_plane_t *plane)
+{
+	if (!igt_plane_has_prop(plane, IGT_PLANE_ROTATION))
+		return false;
+
+	if (!igt_plane_has_rotation(plane, data->rotation))
+		return false;
+
+	/* CHV can't rotate and reflect simultaneously */
+	if (!(!is_intel_device(data->gfx_fd) || !IS_CHERRYVIEW(data->devid) ||
+	    data->rotation != (IGT_ROTATION_180 | IGT_REFLECT_X)))
+		return false;
+
+	/* Intel display version 20 onwards cannot do reflect-x with tile4 */
+	if (!(!is_intel_device(data->gfx_fd) ||
+	    intel_display_ver(data->devid) < 20 ||
+	    !(data->override_modifier == I915_FORMAT_MOD_4_TILED &&
+	    data->rotation & IGT_REFLECT_X)))
+		return false;
+
+	return true;
+}
+
 static void test_plane_rotation(data_t *data, int plane_type, bool test_bad_format)
 {
 	igt_display_t *display = &data->display;
@@ -632,12 +650,7 @@ static void test_plane_rotation(data_t *data, int plane_type, bool test_bad_form
 		igt_output_set_pipe(output, pipe);
 
 		plane = igt_output_get_plane_type(output, plane_type);
-		igt_require(igt_plane_has_prop(plane, IGT_PLANE_ROTATION));
-		igt_require(igt_plane_has_rotation(plane, data->rotation));
-		/* CHV can't rotate and reflect simultaneously */
-		igt_require(!is_intel_device(data->gfx_fd) ||
-			    !IS_CHERRYVIEW(data->devid) ||
-			    data->rotation != (IGT_ROTATION_180 | IGT_REFLECT_X));
+		igt_require(plane_rotation_requirements(data, plane));
 
 		prepare_crtc(data, output, pipe, plane, true);
 
@@ -1091,8 +1104,8 @@ static void test_plane_rotation_exhaust_fences(data_t *data,
 	int fd = data->gfx_fd;
 	drmModeModeInfo *mode;
 	struct igt_fb fb[MAX_FENCES+1] = {};
-	uint64_t size;
-	unsigned int stride, w, h;
+	struct igt_fb tmp_fb;
+	unsigned int w, h;
 	uint64_t total_aperture_size, total_fbs_size;
 	int i;
 
@@ -1106,13 +1119,15 @@ static void test_plane_rotation_exhaust_fences(data_t *data,
 	w = mode->hdisplay;
 	h = mode->vdisplay;
 
-	igt_calc_fb_size(fd, w, h, format, modifier, &size, &stride);
+	igt_init_fb(&tmp_fb, fd, w, h, format, modifier,
+		    IGT_COLOR_YCBCR_BT709, IGT_COLOR_YCBCR_LIMITED_RANGE);
+	igt_calc_fb_size(&tmp_fb);
 
 	/*
 	 * Make sure there is atleast 90% of the available GTT space left
 	 * for creating (MAX_FENCES+1) framebuffers.
 	 */
-	total_fbs_size = size * (MAX_FENCES + 1);
+	total_fbs_size = tmp_fb.size * (MAX_FENCES + 1);
 	total_aperture_size = gem_available_aperture_size(fd);
 	igt_require(total_fbs_size < total_aperture_size * 0.9);
 
@@ -1169,7 +1184,7 @@ static const char help_str[] =
 
 static data_t data;
 
-igt_main_args("", long_opts, help_str, opt_handler, &data)
+int igt_main_args("", long_opts, help_str, opt_handler, &data)
 {
 	struct rot_subtest {
 		unsigned plane;
@@ -1206,7 +1221,7 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 
 	int gen = 0;
 
-	igt_fixture {
+	igt_fixture() {
 		data.gfx_fd = drm_open_driver_master(DRIVER_ANY);
 		if (is_intel_device(data.gfx_fd)) {
 			data.devid = intel_get_drm_devid(data.gfx_fd);
@@ -1272,7 +1287,7 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 
 	igt_describe("Tiling and Rotation test for gen 10+ for primary plane");
 	for (reflect_x = reflect_x_subtests; reflect_x->modifier; reflect_x++) {
-		igt_fixture
+		igt_fixture()
 			igt_require_intel(data.gfx_fd);
 
 		igt_subtest_f("primary-%s-tiled-reflect-x-%s",
@@ -1345,7 +1360,7 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 		}
 	}
 
-	igt_fixture {
+	igt_fixture() {
 		igt_display_fini(&data.display);
 		drm_close_driver(data.gfx_fd);
 	}
