@@ -20,6 +20,16 @@
  * Authors:
  *    Imre Deak <imre.deak@intel.com>
  */
+
+/**
+ * TEST: kms setmode
+ * Category: Display
+ * Description: Tests the mode by iterating through all valid/invalid crtc/connector
+ *              combinations
+ * Driver requirement: i915, xe
+ * Mega feature: General Display Features
+ */
+
 #include "config.h"
 
 #include "igt.h"
@@ -31,6 +41,35 @@
 #include <sys/time.h>
 #include <math.h>
 
+#include "i915/intel_drrs.h"
+#include "xe/xe_query.h"
+
+/**
+ * SUBTEST: basic
+ * Description: Tests the vblank timing by iterating through all valid crtc/connector
+ *              combinations
+ *
+ * SUBTEST: basic-clone-single-crtc
+ * Description: Test allows the use of a single CRTC for two connectors, such as VGA/HDMI,
+ *		VGA/DP, and HDMI/HDMI
+ *
+ * SUBTEST: clone-exclusive-crtc
+ * Description: Tests the mode by cloning the exclusive crtc by iterating through
+ *              all valid crtc/connector combinations
+ *
+ * SUBTEST: invalid-clone-exclusive-crtc
+ * Description: Test is allowed when a connector uses CRTC that is not compatible with the
+ *		connector, for instance, DP/HDMI can be used on certain pipes only on older platforms
+ *
+ * SUBTEST: invalid-clone-single-crtc
+ * Description: Tests the mode by cloning the single crtc by iterating through all
+ *              invalid crtc/connector combinations
+ *
+ * SUBTEST: invalid-clone-single-crtc-stealing
+ * Description: Tests the stealing mode by cloning the single crtc by iterating
+ *              through all invalid crtc/connector combinations
+ */
+
 #define MAX_CONNECTORS  10
 #define MAX_CRTCS       6
 
@@ -44,7 +83,8 @@ static int drm_fd;
 static drmModeRes *drm_resources;
 static int filter_test_id;
 static bool dry_run;
-static bool all_pipes = false;
+static bool extended = false;
+static int max_dotclock;
 
 const drmModeModeInfo mode_640_480 = {
 	.name		= "640x480",
@@ -181,7 +221,7 @@ static void create_fb_for_crtc(struct crtc_config *crtc,
 	fb_id = igt_create_pattern_fb(drm_fd, crtc->mode.hdisplay,
 				      crtc->mode.vdisplay,
 				      igt_bpp_depth_to_drm_format(bpp, depth),
-				      LOCAL_DRM_FORMAT_MOD_NONE,
+				      DRM_FORMAT_MOD_LINEAR,
 				      fb_info);
 	igt_assert_lt(0, fb_id);
 }
@@ -198,8 +238,10 @@ static void get_mode_for_crtc(struct crtc_config *crtc,
 	 */
 	for (i = 0; i < crtc->connector_count; i++) {
 		mode = &crtc->cconfs[i].default_mode;
-		if (crtc_supports_mode(crtc, mode))
-			goto found;
+		if (crtc_supports_mode(crtc, mode)) {
+			*mode_ret = *mode;
+			return;
+		}
 	}
 
 	/*
@@ -208,8 +250,29 @@ static void get_mode_for_crtc(struct crtc_config *crtc,
 	 */
 	for (i = 0; i < crtc->cconfs[0].connector->count_modes; i++) {
 		mode = &crtc->cconfs[0].connector->modes[i];
-		if (crtc_supports_mode(crtc, mode))
-			goto found;
+		if (crtc_supports_mode(crtc, mode)) {
+			*mode_ret = *mode;
+			return;
+		}
+	}
+
+	/* If an eDP connector is present, iterate through its modes and
+	 * pick a mode with lowest clock, since internal panels typically
+	 * dictate the clone mode and may have HRR support, making them
+	 * incompatible with modes supported by external displays.
+	 */
+	for (i = 0; i < crtc->connector_count; i++) {
+		drmModeConnector *conn = crtc->cconfs[i].connector;
+
+		if (conn->connector_type == DRM_MODE_CONNECTOR_eDP) {
+			mode = &conn->modes[0];
+			for (int j = 1; j < conn->count_modes; j++) {
+				if (conn->modes[j].clock < mode->clock)
+					mode = &conn->modes[j];
+			}
+			*mode_ret = *mode;
+			return;
+		}
 	}
 
 	/*
@@ -218,10 +281,11 @@ static void get_mode_for_crtc(struct crtc_config *crtc,
 	 * scaling etc.
 	 */
 	mode = &crtc->cconfs[0].default_mode;
-	for (i = 1; i < crtc->connector_count; i++)
+	for (i = 1; i < crtc->connector_count; i++) {
 		if (crtc->cconfs[i].default_mode.clock < mode->clock)
 			mode = &crtc->cconfs[i].default_mode;
-found:
+	}
+
 	*mode_ret = *mode;
 }
 
@@ -418,17 +482,10 @@ static int test_stealing(int fd, struct crtc_config *crtc, uint32_t *ids)
 	return ret;
 }
 
-static double frame_time(const drmModeModeInfo *kmode)
-{
-	return 1000.0 * kmode->htotal * kmode->vtotal / kmode->clock;
-}
+#define frame_time(km) (1000.0 * (km)->htotal * (km)->vtotal / (km)->clock)
+#define line_time(km) (1000.0 * (km)->htotal / (km)->clock)
 
-static double line_time(const drmModeModeInfo *kmode)
-{
-	return 1000.0 * kmode->htotal / kmode->clock;
-}
-
-static void check_timings(int crtc_idx, const drmModeModeInfo *kmode)
+static bool check_timings(int crtc_idx, const drmModeModeInfo *kmode)
 {
 #define CALIBRATE_TS_STEPS 120 /* ~2s has to be less than 128! */
 	drmVBlank wait;
@@ -507,9 +564,12 @@ static void check_timings(int crtc_idx, const drmModeModeInfo *kmode)
 		 100 * accuracy / mean, accuracy / line_time(kmode));
 
 	/* 99.7% samples within one scanline on each side of mean */
-	igt_assert_f(accuracy < line_time(kmode),
-		     "vblank accuracy (%.3fus, %.1f%%) worse than a scanline (%.3fus)\n",
-		     accuracy, 100 * accuracy / mean, line_time(kmode));
+	if (accuracy >= line_time(kmode)) {
+		igt_info("vblank accuracy (%.3fus, %.1f%%) worse than a scanline (%.3fus)\n",
+			  accuracy, 100 * accuracy / mean, line_time(kmode));
+
+		return false;
+	}
 
 	/* At least 90% of frame times fall within the one scanline on each
 	 * side of expected mean.
@@ -532,12 +592,18 @@ static void check_timings(int crtc_idx, const drmModeModeInfo *kmode)
 	 * See:
 	 * https://en.wikipedia.org/wiki/Standard_deviation#Rules_for_normally_distributed_data
 	 */
-	igt_assert_f(fabs(mean - expected) < max(line_time(kmode), 1.718 * stddev),
-		     "vblank interval differs from modeline! expected %.1fus, measured %1.fus +- %.3fus, difference %.1fus (%.1f sigma, %.1f scanlines)\n",
-		     expected, mean, stddev,
-		     fabs(mean - expected),
-		     fabs(mean - expected) / stddev,
-		     fabs(mean - expected) / line_time(kmode));
+	if (fabs(mean - expected) >= max(line_time(kmode), 1.718 * stddev)) {
+		igt_info("vblank interval differs from modeline! expected %.1fus, "
+			 "measured %1.fus +- %.3fus, difference %.1fus (%.1f sigma, %.1f scanlines)\n",
+			  expected, mean, stddev,
+			  fabs(mean - expected),
+			  fabs(mean - expected) / stddev,
+			  fabs(mean - expected) / line_time(kmode));
+
+		return false;
+	}
+
+	return true;
 }
 
 static void test_crtc_config(const struct test_config *tconf,
@@ -548,6 +614,7 @@ static void test_crtc_config(const struct test_config *tconf,
 	struct crtc_config *crtc;
 	static int test_id;
 	bool config_failed = false;
+	bool retry = false;
 	int ret = 0;
 	int i;
 
@@ -558,6 +625,19 @@ static void test_crtc_config(const struct test_config *tconf,
 
 	igt_info("  Test id#%d CRTC count %d\n", test_id, crtc_count);
 
+retry:
+	if (retry) {
+		kmstest_unset_all_crtcs(drm_fd, tconf->resources);
+
+		for (i = 0; i < crtc_count; i++) {
+			/* Sort the modes in asending order by clock freq. */
+			igt_sort_connector_modes(crtcs[i].cconfs->connector,
+						 sort_drm_modes_by_clk_asc);
+
+			crtcs[i].mode = crtcs[i].cconfs->connector->modes[0];
+		}
+	}
+
 	for (i = 0; i < crtc_count; i++) {
 		get_crtc_config_str(&crtcs[i], str_buf[i], sizeof(str_buf[i]));
 		crtc_strs[i] = &str_buf[i][0];
@@ -587,9 +667,19 @@ static void test_crtc_config(const struct test_config *tconf,
 					     crtc->fb_info.fb_id, 0, 0, ids,
 					     crtc->connector_count, &crtc->mode);
 
+		if (is_intel_device(drm_fd))
+			intel_drrs_disable(drm_fd, crtc->pipe_id);
+
 		free(ids);
 
 		if (ret < 0) {
+			if (errno == ENOSPC) {
+				igt_skip_on_f(retry, "No suitable mode(s) found to fit into the link BW.\n");
+
+				retry = true;
+				goto retry;
+			}
+
 			igt_assert_eq(errno, EINVAL);
 			config_failed = true;
 		}
@@ -597,12 +687,42 @@ static void test_crtc_config(const struct test_config *tconf,
 
 	igt_assert(config_failed == !!(tconf->flags & TEST_INVALID));
 
-	if (ret == 0 && tconf->flags & TEST_TIMINGS)
-		check_timings(crtcs[0].crtc_idx, &crtcs[0].mode);
+	if (ret == 0 && tconf->flags & TEST_TIMINGS) {
+		bool status = false;
+
+		for (int attempt = 1; attempt <= 2; attempt++) {
+			status = check_timings(crtcs[0].crtc_idx, &crtcs[0].mode);
+			if (status)
+				break;
+			igt_info("Timing check failed on attempt %d, retrying...\n", attempt);
+		}
+
+		igt_assert_f(status, "VBlank timing test failed after 2 attempt(s)\n");
+	}
 
 	return;
 }
 
+static int get_test_name_str(struct crtc_config *crtc, char *buf,
+				size_t buf_size)
+{
+	int pos;
+	int i;
+
+	pos = snprintf(buf, buf_size, "pipe-%s-", kmstest_pipe_name(crtc->pipe_id));
+
+	for (i = 0; i < crtc->connector_count; i++) {
+		drmModeConnector *connector = crtc->cconfs[i].connector;
+
+		pos += snprintf(&buf[pos], buf_size - pos,
+			"%s%s-%d", i ? "-" : "",
+			kmstest_connector_type_str(connector->connector_type),
+			connector->connector_type_id);
+	}
+
+	return pos;
+}
+
 static void test_one_combination(const struct test_config *tconf,
 				 struct connector_config *cconfs,
 				 int connector_count)
@@ -614,9 +734,64 @@ static void test_one_combination(const struct test_config *tconf,
 	setup_crtcs(tconf, cconfs, connector_count, crtcs,
 		    &crtc_count, &config_valid);
 
-	if (config_valid == !(tconf->flags & TEST_INVALID))
-		test_crtc_config(tconf, crtcs, crtc_count);
+	if (config_valid == !(tconf->flags & TEST_INVALID)) {
+		int i, pos = 0;
+		char test_name[256];
 
+		for (i = 0; i < crtc_count; i++) {
+			if (i > 0)
+				pos += snprintf(&test_name[pos], ARRAY_SIZE(test_name) - pos, "-");
+			pos += get_test_name_str(&crtcs[i], &test_name[pos], ARRAY_SIZE(test_name) - pos);
+		}
+
+		if (!is_intel_device(drm_fd))
+			goto test;
+
+		for (i = 0; i < crtc_count; i++) {
+			struct crtc_config *crtc = &crtcs[i];
+			char conn_name[24], prev_conn_name[24];
+
+			snprintf(conn_name, sizeof(conn_name),
+				 "%s-%d",
+				 kmstest_connector_type_str(crtcs[i].cconfs->connector->connector_type),
+				 crtcs[i].cconfs->connector->connector_type_id);
+
+			if (i > 0)
+				snprintf(prev_conn_name, sizeof(prev_conn_name),
+					 "%s-%d",
+					 kmstest_connector_type_str(crtcs[i - 1].cconfs->connector->connector_type),
+					 crtcs[i - 1].cconfs->connector->connector_type_id);
+
+			/*
+			 * Handle BW limitations on intel hardware:
+			 *
+			 * if force joiner (or) mode resolution > 5K (or) mode clock > max_dotclock,
+			 * then ignore
+			 *   - last crtc in single/multi-connector config
+			 *   - consecutive crtcs in multi-connector config
+			 *
+			 * in multi-connector config ignore if
+			 *   - previous crtc (force joiner (or) mode resolution > 5K (or)
+			 *     mode clock > max_dotclock) and
+			 *   - current & previous crtcs are consecutive
+			 */
+			if (((igt_check_force_joiner_status(drm_fd, conn_name) ||
+			      igt_bigjoiner_possible(drm_fd, &crtc->mode, max_dotclock)) &&
+			     ((crtc->crtc_idx >= (tconf->resources->count_crtcs - 1)) ||
+			      ((i < (crtc_count - 1)) && (abs(crtcs[i + 1].crtc_idx - crtc->crtc_idx) <= 1)))) ||
+			    ((i > 0) && (igt_check_force_joiner_status(drm_fd, prev_conn_name) ||
+					 igt_bigjoiner_possible(drm_fd, &crtc[i - 1].mode, max_dotclock)) &&
+			     (abs(crtc->crtc_idx - crtcs[i - 1].crtc_idx) <= 1))) {
+				igt_info("Combo: %s is not possible with selected mode(s).\n", test_name);
+				goto out;
+			}
+		}
+test:
+		igt_dynamic_f("%s", test_name)
+			test_crtc_config(tconf, crtcs, crtc_count);
+	}
+
+out:
 	cleanup_crtcs(crtcs, crtc_count);
 }
 
@@ -635,7 +810,7 @@ static int assign_crtc_to_connectors(const struct test_config *tconf,
 		    crtc_idx_mask & ~(1 << crtc_idx))
 			return -1;
 
-		if ((tconf->flags & TEST_EXCLUSIVE_CRTC_CLONE) &&
+		if ((tconf->flags & (TEST_EXCLUSIVE_CRTC_CLONE | TEST_TIMINGS)) &&
 		    crtc_idx_mask & (1 << crtc_idx))
 			return -1;
 
@@ -727,7 +902,7 @@ static void iterate_combinations(int n, int k, bool allow_repetitions,
 	int v;
 
 	if (!k) {
-		igt_assert(set->count < set->capacity);
+		igt_assert_lt(set->count, set->capacity);
 		set->items[set->count++] = *comb;
 		return;
 	}
@@ -751,9 +926,9 @@ static void get_combinations(int n, int k, bool allow_repetitions,
 	iterate_combinations(n, k, allow_repetitions, 0, 0, &comb, set);
 }
 
-static int get_limit_crtc_count(int count_crtcs, bool all_pipe)
+static int get_crtc_count(int count_crtcs, bool extend)
 {
-	if ((count_crtcs <= CRTC_RESTRICT_CNT) || all_pipe)
+	if ((count_crtcs <= CRTC_RESTRICT_CNT) || extend)
 		return count_crtcs;
 	else
 		return CRTC_RESTRICT_CNT;
@@ -766,7 +941,7 @@ static void test_combinations(const struct test_config *tconf,
 	struct combination_set crtc_combs;
 	struct connector_config *cconfs;
 	int i;
-	int crtc_count = get_limit_crtc_count(tconf->resources->count_crtcs, all_pipes);
+	int crtc_count = get_crtc_count(tconf->resources->count_crtcs, extended);
 
 	if (connector_count > 2 && (tconf->flags & TEST_STEALING))
 		return;
@@ -786,7 +961,6 @@ static void test_combinations(const struct test_config *tconf,
 
 	igt_info("Testing: %s %d connector combinations\n", tconf->name,
 		 connector_count);
-
 	for (i = 0; i < connector_combs.count; i++) {
 		int *connector_idxs;
 		int ret;
@@ -824,7 +998,7 @@ free_cconfs:
 static void run_test(const struct test_config *tconf)
 {
 	int connector_num;
-	int crtc_count = get_limit_crtc_count(tconf->resources->count_crtcs, all_pipes);
+	int crtc_count = get_crtc_count(tconf->resources->count_crtcs, extended);
 
 	connector_num = tconf->flags & TEST_CLONE ? 2 : 1;
 	for (; connector_num <= crtc_count; connector_num++)
@@ -838,7 +1012,7 @@ static int opt_handler(int opt, int opt_index, void *data)
 		dry_run = true;
 		break;
 	case 'e':
-		all_pipes = true;
+		extended = true;
 		break;
 	case 't':
 		filter_test_id = atoi(optarg);
@@ -853,9 +1027,10 @@ static int opt_handler(int opt, int opt_index, void *data)
 const char *help_str =
 	"  -d\t\tDon't run any test, only print what would be done. (still needs DRM access)\n"
 	"  -t <test id>\tRun only the test with this id\n"
-	"  -e \t\tRun on all pipes. (Default it will Run only two pipes)\n";
+	"  -e \t\tExtend to run on all pipes. (By default tests will run on two pipes only)\n";
 
-igt_main_args("det:", NULL, help_str, opt_handler, NULL)
+
+int igt_main_args("det:", NULL, help_str, opt_handler, NULL)
 {
 	const struct {
 		enum test_flags flags;
@@ -878,18 +1053,20 @@ igt_main_args("det:", NULL, help_str, opt_handler, NULL)
 	igt_assert_f(!(dry_run && filter_test_id),
 		     "only one of -d and -t is accepted\n");
 
-	igt_fixture {
+	igt_fixture() {
 		drm_fd = drm_open_driver_master(DRIVER_ANY);
 		if (!dry_run)
 			kmstest_set_vt_graphics_mode();
 
 		drm_resources = drmModeGetResources(drm_fd);
 		igt_require(drm_resources);
+
+		max_dotclock = igt_get_max_dotclock(drm_fd);
 	}
 
 	for (i = 0; i < ARRAY_SIZE(tests); i++) {
 		igt_describe("Tests the mode by iterating through all valid/invalid crtc/connector combinations");
-		igt_subtest(tests[i].name) {
+		igt_subtest_with_dynamic(tests[i].name) {
 			struct test_config tconf = {
 				.flags		= tests[i].flags,
 				.name		= tests[i].name,
@@ -899,9 +1076,8 @@ igt_main_args("det:", NULL, help_str, opt_handler, NULL)
 		}
 	}
 
-	igt_fixture {
+	igt_fixture() {
 		drmModeFreeResources(drm_resources);
-
-		close(drm_fd);
+		drm_close_driver(drm_fd);
 	}
 }
