@@ -38,6 +38,52 @@
 
 #include "sw_sync.h"
 
+/**
+ * TEST: sw sync
+ * Description: Test SW Sync Framework
+ * Category: Core
+ * Mega feature: General Core features
+ * Sub-category: Synchronization
+ * Functionality: semaphore
+ * Feature: synchronization
+ * Test category: GEM_Legacy
+ *
+ * SUBTEST: alloc_fence
+ *
+ * SUBTEST: alloc_fence_invalid_timeline
+ *
+ * SUBTEST: alloc_merge_fence
+ *
+ * SUBTEST: alloc_timeline
+ *
+ * SUBTEST: sync_busy
+ *
+ * SUBTEST: sync_busy_fork
+ *
+ * SUBTEST: sync_busy_fork_unixsocket
+ *
+ * SUBTEST: sync_expired_merge
+ *
+ * SUBTEST: sync_merge
+ *
+ * SUBTEST: sync_merge_invalid
+ *
+ * SUBTEST: sync_merge_same
+ *
+ * SUBTEST: sync_multi_consumer
+ *
+ * SUBTEST: sync_multi_consumer_producer
+ *
+ * SUBTEST: sync_multi_producer_single_consumer
+ *
+ * SUBTEST: sync_multi_timeline_wait
+ *
+ * SUBTEST: sync_random_merge
+ *
+ * SUBTEST: timeline_closed
+ *
+ * SUBTEST: timeline_closed_signaled
+ */
 
 IGT_TEST_DESCRIPTION("Test SW Sync Framework");
 
@@ -832,9 +878,9 @@ static void test_sync_random_merge(void)
 		close(timeline_arr[i]);
 }
 
-igt_main
+int igt_main()
 {
-	igt_fixture {
+	igt_fixture() {
 		igt_require_sw_sync();
 		multi_consumer_threads =
 			min_t(multi_consumer_threads,
